﻿/******************************************************************************
 * Copyright 2019 The siasun Transport Authors. All Rights Reserved.
 * Author: limengqi
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/

#include "canbus.h"
#include"candatatype.h"
#include"messagecoder.h"
#include "pid.h"
#include "canbusmysqldatabase.h"
#include"../common/message.h"
#include"../common/canmsgobdmsgid.h"
#include "std_msgs/String.h"
#include <canbus/canMsg.h>
#include <canbus/rackTestReplyMsg.h>
#include <iostream>
#include <fstream>
#include <chrono>
#include <sys/time.h>
#include <arpa/inet.h>
#include <sys/socket.h>

#include <sys/ioctl.h>
#include <linux/can.h>
#include <linux/if.h>
#include <linux/can/raw.h>


namespace  robot{
namespace canbusSpace {


std::string Canbus::Name() const { return "canbus"; }

int Canbus::Init() {
  LOG(INFO)<< Name()<<" Init start...";
  node_handle_.reset(new ros::NodeHandle());
  //20240111 by zhaowei: add tcptocan mode
  if (!ros::param::get("rcu_canbus_mode", canbus_mode))
    canbus_mode = "pcitcp";
  LOG(INFO)<< Name()<<": read param rcu_canbus_mode = " << canbus_mode;
  if (canbus_mode == "tcp" || canbus_mode == "pcitcp") {
    for(int i=0; i < sizeof(s_g_pBufSet); ++i ) {
      s_g_pBufSet[i] = 0x00;
      s_g_pBufGet[i] = 0x00;
    }
  }
  /*
  else if (canbus_mode == "usb") {
    //load libpcan.so using dlopen function,return handle for further use
    libm_handle = dlopen("libpcan.so", RTLD_LAZY );
    if (!libm_handle){
        LOG(ERROR)<< Name()<<" Init Open Error:" << dlerror();//if file can't be loaded,return null,get reason using dlerror function
        return ROBOT_FAILTURE;
    }
    char *errorInfo;//error information pointer
    //one-to-one mapping using dlsym function,if return null,mapping would be failed
    funCAN_Init         =(funCAN_Init_TYPE)         dlsym(libm_handle,"CAN_Init");
    funLINUX_CAN_Open   =(funLINUX_CAN_Open_TYPE)   dlsym(libm_handle,"LINUX_CAN_Open");
    funCAN_Close        =(funCAN_Close_TYPE)        dlsym(libm_handle,"CAN_Close");
    funCAN_VersionInfo  =(funCAN_VersionInfo_TYPE)  dlsym(libm_handle,"CAN_VersionInfo");
    funLINUX_CAN_Read   =(funLINUX_CAN_Read_TYPE)   dlsym(libm_handle,"LINUX_CAN_Read");
    funCAN_Status       =(funCAN_Status_TYPE)       dlsym(libm_handle,"CAN_Status");
    funnGetLastError    =(funnGetLastError_TYPE)    dlsym(libm_handle,"nGetLastError");
    funCAN_Write        =(funCAN_Write_TYPE)       dlsym(libm_handle, "CAN_Write");

    errorInfo = dlerror();//get error using dlerror function,and clear the error list in memory
    if (errorInfo != NULL){
        LOG(ERROR)<<Name()<<" Init Dlsym Error:"<<errorInfo;
        return ROBOT_FAILTURE;
    }
  }*/

  //initialize the VehHBrkStat as Hold
  status1.uiVehHBrkStat = 1;

  InitServiceAndMsg();
  LOG(INFO)<<Name()<<" init end.";
  return ROBOT_SUCCESS;
}


//打开Can总线
int Canbus::Start() {
  string canbus_ip;
  int canbus_port;
  if (canbus_mode == "tcp") {
    if (!ros::param::get("rcu_canbus_ip", canbus_ip)) {
        LOG(ERROR)<< Name()<<": read param rcu_canbus_ip error: not exist";
        return ROBOT_FAILTURE;
    }
    if (!ros::param::get("rcu_canbus_port", canbus_port)) {
        LOG(ERROR)<< Name()<<": read param rcu_canbus_port error: not exist";
        return ROBOT_FAILTURE;
    }
    LOG(INFO)<< Name() <<": read param rcu_canbus_ip:port = " << canbus_ip << ":" << canbus_port;
    canbus_socket = socket(AF_INET, SOCK_STREAM | SOCK_NONBLOCK, IPPROTO_TCP);
//    canbus_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    struct sockaddr_in serv_addr;
    memset(&serv_addr, 0, sizeof(serv_addr));  //每个字节都用0填充
    serv_addr.sin_family = AF_INET;  //使用IPv4地址
    serv_addr.sin_addr.s_addr = inet_addr(canbus_ip.c_str());
    serv_addr.sin_port = htons(canbus_port);  //端口
    int result = connect(canbus_socket, (struct sockaddr*)&serv_addr, sizeof(serv_addr));
    if (result == 0) {
      LOG(INFO) << Name() << ": " << canbus_ip << ":" << canbus_port << " connected";
    }
    else {
      LOG(ERROR) << Name() << ": connecting " << canbus_ip << ":" << canbus_port << " failed";
    }
  }
/*  else if (canbus_mode == "usb") {
    char chVersion[VERSIONSTRING_LEN];            //store information of can version
    const char  *szDevNode = DEFAULT_NODE;  //define const pointer point to device name
    pcan_handle = funLINUX_CAN_Open(szDevNode, O_RDWR );//use mapping function

    //judge whether the call is success.if pcan_handle=null,the call would be failed
    if(pcan_handle){
        LOG(INFO)<<Name()<<": "<<szDevNode<<" opened";
        errno = funCAN_VersionInfo(pcan_handle, chVersion);
        if (!errno)
            LOG(INFO)<<Name()<<": driver version ="<<chVersion;
        else {
            LOG(ERROR)<<Name()<<": CAN_VersionInfo() error";
        }
        if (unBaudrate) {
                errno = funCAN_Init(pcan_handle, unBaudrate, nExtended);
                if (errno) {
                    LOG(ERROR)<<Name()<<": CAN_Init() error.";
                }
                else
                    LOG(INFO)<<" Device Info: " << szDevNode <<"Baudrate:"<<unBaudrate;
            }
    }
    else
    {
        LOG(INFO)<<Name()<<": can't open ."<<szDevNode;
    }
  }*/
  else if(canbus_mode == "pcitcp"){
    int ret = 0;
    ret = can_init_for_linux(&can_fd);
    if(ret == -1)
    {
      LOG(INFO)<<Name()<<"socket can init failed!";
    }
else
{
LOG(INFO)<<Name()<<"socket can init OK OK OK!";
cout << "can_fd = " << can_fd << endl;
}

  }
  ros::Rate loop_rate(1/rate);
  // 5. set timer to triger publish info periodly
  timer_ = CreateTimer(ros::Duration(duration), &Canbus::OnTimer, this);

  // this location is necessary

  canRCUData_pub = node_handle_->advertise<canbus::canMsg>(robot::common::CanDataAnalysis, 1);
  canObdData_pub = node_handle_->advertise<canbus::obdMsg>(robot::common::ObdData, 1);
  canRackData_pub = node_handle_->advertise<canbus::rackTestReplyMsg>(robot::common::RackReplyData,1);
  canRackDetailData_pub = node_handle_->advertise<canbus::rackTestReplyDetailMsg>(robot::common::RackReplyDetailData,1);

  ros::ServiceServer ActuatorEnableService = node_handle_->advertiseService(robot::common::ActuatorEnableService,
                                                                &Canbus::ActuatorEnableRequest, this);
  ros::ServiceServer LearnModeService = node_handle_->advertiseService(robot::common::LearnModeService,
                                                            &Canbus::ExcuteLearnModeRequest, this);
  ros::ServiceServer ShiftActCtrService = node_handle_->advertiseService(robot::common::ShiftActCtrService,
                                                              &Canbus::SftActCtrRequest, this);
  ros::ServiceServer SteerActCtrService = node_handle_->advertiseService(robot::common::SteerActCtrService,
                                                              &Canbus::StrActCtrRequest, this);
  ros::ServiceServer AccPedalActCtrService = node_handle_->advertiseService(robot::common::AccPedalActCtrService,
                                                              &Canbus::AccPedalActCtrRequest, this);
  ros::ServiceServer BrkPedalActCtrService = node_handle_->advertiseService(robot::common::BrkPedalActCtrService,
                                                              &Canbus::BrkPedalActCtrRequest, this);
  ros::ServiceServer RBTManagerService = node_handle_->advertiseService(robot::common::RBTManagerService,
                                                              &Canbus::RBTManager, this);
  ros::ServiceServer PowerOnService = node_handle_->advertiseService(robot::common::PowerOnService,
                                                                     &Canbus::PowerOnRequest, this);
  ros::ServiceServer ActuatorCtrLimitService = node_handle_->advertiseService(robot::common::ActuatorCtrLimitService,
                                                                              &Canbus::ActuatorCtrLimit,this);
  ros::ServiceServer VehicleStartService = node_handle_->advertiseService(robot::common::VehicleStartService,
                                                                           &Canbus::VehicleStart,this);
  ros::ServiceServer SlopeModeService = node_handle_->advertiseService(robot::common::SlopeModeService,
                                                                       &Canbus::SlopeMode,this);
  ros::ServiceServer StrLoopService = node_handle_->advertiseService(robot::common::StrLoopService,
                                                                       &Canbus::StrLoopReq,this);
  ros::ServiceServer AccLoopService = node_handle_->advertiseService(robot::common::AccLoopService,
                                                                     &Canbus::AccLoopReq,this);
  ros::ServiceServer BrkLoopService = node_handle_->advertiseService(robot::common::BrkLoopService,
                                                                     &Canbus::BrkLoopReq,this);
  ros::ServiceServer StrTxtService = node_handle_->advertiseService(robot::common::StrTxtService,
                                                                    &Canbus::StrTxtReq,this);
  ros::ServiceServer BrkTxtService = node_handle_->advertiseService(robot::common::BrkTxtService,
                                                                    &Canbus::BrkTxtReq,this);

  ros::ServiceServer AccTxtService = node_handle_->advertiseService(robot::common::AccTxtService,
                                                                    &Canbus::AccTxtReq,this);

  ros::ServiceServer StrTxtLineService = node_handle_->advertiseService(robot::common::StrTxtLineService,
                                                                        &Canbus::StrTxtLineReq,this);

  ros::ServiceServer VehicleStatSetService = node_handle_->advertiseService(robot::common::VehicleStatSetService,
                                                                            &Canbus::VehicleStatSetReq,this);

  ros::ServiceServer RbtDriverCtrlService = node_handle_->advertiseService(robot::common::RbtDriverCtrlService,
                                                                           &Canbus::RbtDriverControl,this);

  ros::ServiceServer StrTangLineCtrService  = node_handle_->advertiseService(robot::common::StrTangLineCtrService,
                                                                             &Canbus::StrTangLineCtrReq,this);

  ros::ServiceServer StrPosTimCtrService   = node_handle_->advertiseService(robot::common::StrPosTimCtrService,
                                                                            &Canbus::StrPosTimCtrReq,this);

  ros::ServiceServer StrSinCtrService   = node_handle_->advertiseService(robot::common::StrSinCtrService ,
                                                                         &Canbus::StrSinCtrReq,this);

  ros::ServiceServer EstpCtrlService   = node_handle_->advertiseService(robot::common::EstpCtrlService ,
                                                                        &Canbus::EstpCtrl,this);

  ros::ServiceServer RobotWorkMdService   = node_handle_->advertiseService(robot::common::RobotWorkMdService ,
                                                                           &Canbus::RobotWorkMd,this);

  ros::ServiceServer SysCtrlModeService   = node_handle_->advertiseService(robot::common::SysCtrlModeService ,
                                                                           &Canbus::SysCtrlMode,this);

  ros::ServiceServer BrkTimCtrlStpService   = node_handle_->advertiseService(robot::common::BrkTimCtrlStpService ,
                                                                             &Canbus::BrkTimCtrlStp,this);

  ros::ServiceServer AccPosCtrlStpService   = node_handle_->advertiseService(robot::common::AccPosCtrlStpService ,
                                                                             &Canbus::AccPosCtrlStp,this);

  ros::ServiceServer StrAccCtrlReqService   = node_handle_->advertiseService(robot::common::StrAccCtrlReqService ,
                                                                             &Canbus::StrAccCtrlReq,this);

  ros::ServiceServer RbtSlfChckCtrlReqService = node_handle_->advertiseService(robot::common::RbtSlfChckCtrlReqService ,
                                                                               &Canbus::RbtSlfChckCtrlReq,this);

  ros::ServiceServer IOModeService = node_handle_->advertiseService(robot::common::IOModeService ,
                                                                    &Canbus::IOMode,this);

  ros::ServiceServer SftTxtService = node_handle_->advertiseService(robot::common::SftTxtService,
                                                                    &Canbus::SftTxtReq,this);
  ros::ServiceServer RackTestService = node_handle_->advertiseService(robot::common::RackTestService,
                                                                     &Canbus::RackTestReq,this);

  ros::Subscriber GeometrySub = node_handle_->subscribe(robot::common::Geometry, 1,
                                                        &Canbus::RTKLocationCallback, this);

  //data receive
  while(ros::ok())
  {
      read_loop();
      ros::spinOnce();
      loop_rate.sleep();
   }

   return ROBOT_SUCCESS;
}


void Canbus::OnTimer(const ros::TimerEvent &) {
  //LOG(INFO)<< Name()<< " timer start ..";
  //TODO dakai
  if(heatbeat > 15)
  {
      heatbeat = 0;
  }
  icuMonitor.uiICUAlive = heatbeat++;

  try
  {
      canObdData_pub.publish(obd);
      TPCANMsg canIcuMonitor; //ICUMonitor Can Message
      MessageCoder::CanMsgPackage((void*)&icuMonitor, robot::common::ICUMonitor, &canIcuMonitor);
      write_data(&canIcuMonitor, false);

      if(workModeCnfg.uiOnlineOffline == VEH_DATA_TYPE_VIHICLE_DEPEND)
      {
        TPCANMsg canStatus1; //VehicleStaus1 Can Message
        MessageCoder::CanMsgPackage((void*)&status1, robot::common::VehicleStatus1, &canStatus1);
        write_data(&canStatus1, false);
        TPCANMsg canStatus2;  //VehicleStatus2 Can Message
        MessageCoder::CanMsgPackage((void*)&status2, robot::common::VehicleStatus2, &canStatus2);
        write_data(&canStatus2, false);
        TPCANMsg canStatus3;  //VehicleStatus3 Can Message
        MessageCoder::CanMsgPackage((void*)&status3, robot::common::VehicleStatus3, &canStatus3);
        write_data(&canStatus3, false);
      }
      else if(workModeCnfg.uiOnlineOffline == VEH_DATA_TYPE_STAND_ALONE)
      {
        status2.iStrAng = 0;
        status2.iStrAngGrd = 0;
        status2.uiEngSpd = 0;
        TPCANMsg canStatus2;  //VehicleStatus2 Can Message
        MessageCoder::CanMsgPackage((void*)&status2, robot::common::VehicleStatus2, &canStatus2);
        write_data(&canStatus2, false);
      }
  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " OnTimer error. "<< ex.what();
  }
  //TODO
}


void Canbus::Stop() {
  timer_.stop();
  sleep(0.5);
  TPCANMsg canIcu; //ICUMonitor Can Message
  icuMonitor.uiICUClsReq = 1;
  MessageCoder::CanMsgPackage((void*)&icuMonitor, robot::common::ICUMonitor, &canIcu);
  write_data(&canIcu, false);
  LOG(INFO)<<Name()<<"send IcuMonitor CloseReq";

  if (canbus_mode == "tcp") {
     close(canbus_socket);
  }
 /* else if (canbus_mode == "usb") {
    if (pcan_handle) {
        funCAN_Close(pcan_handle);
    }
  }*/
  else if (canbus_mode == "pcitcp")
  {
    can_close(can_fd);
  }
  LOG(INFO)<<Name()<<": closed. ";
  ros::shutdown();
  //Must close h handle firstly,then close file using dlclose
  node_handle_.release();
  LOG(INFO)<<Name()<<": node shutdown. ";
}


void Canbus::InitServiceAndMsg()
{
     obd.AccPsng = 0;
     obd.BrkPsng = 0;
     obd.SftPsng = 0;
     obd.StrAng = 0;
     obd.VehHBrkStat = 0;
     obd.VehSpd = 10;
     obd.VehSwStat = 0;
     status2.uiVehSpd = 10;

     icuMonitor.uiICUClsReq = 0;

     workModeCnfg.uiOnlineOffline = 2;
     workModeCnfg.uiRbtAplySenro = 2;
}

//read from CAN forever - until manual break
int Canbus::read_loop()
{
//TODO dakai
    TPCANRdMsg msg;
    __u32 status;
    if (canbus_mode == "tcp") {
      int recv_len = read(canbus_socket, s_g_pBufGet, sizeof(s_g_pBufGet));
      if (recv_len != sizeof(s_g_pBufGet)) {
        int len = 13 - recv_len;
        size_t a = static_cast<size_t>(len);
        unsigned char s_g_pBufGet2[len];
        int recv_len2 = read(canbus_socket, s_g_pBufGet2, sizeof(s_g_pBufGet2));
        int sizeArr1 = sizeof(s_g_pBufGet) / sizeof(s_g_pBufGet[0]);
        int sizeArr2 = sizeof(s_g_pBufGet2) / sizeof(s_g_pBufGet2[0]);
        std::copy(s_g_pBufGet2, s_g_pBufGet2 + sizeArr2, s_g_pBufGet + sizeArr1);
        if(recv_len + recv_len2 != 13)
        {
       //   LOG(ERROR) << Name() <<" canbus socket read error: " << recv_len;
        }
//         LOG(ERROR) << Name() <<" canbus socket read error: " << recv_len;
      }
      else {
        msg.Msg.ID = static_cast<int>(s_g_pBufGet[3]) * 256 + static_cast<int>(s_g_pBufGet[4]);
        memcpy((unsigned char *)msg.Msg.DATA, (unsigned char *)s_g_pBufGet + 5, 8);
        msg.Msg.LEN = 8;
        msg.Msg.MSGTYPE = MSGTYPE_STANDARD;
//        for (int i = 0; i < sizeof(msg.Msg.DATA); ++i) {
//               cout << hex << setw(2) << setfill('0') << static_cast<int>(msg.Msg.DATA[i]) << " ";
//        }
//        cout << endl;
//        LOG(INFO) << Name() << " receive can msg id = " << hex << msg.Msg.ID;
        read_data(&(msg.Msg));
      }
    }
/*    else if (canbus_mode == "usb") {
      if (funLINUX_CAN_Read(pcan_handle, &msg)) {
          LOG(ERROR)<<Name()<<" receive: LINUX_CAN_Read() error.";
          return errno;
      }
      read_data(&(msg.Msg));
      // deal receive data
      // check if a CAN status is pending
      if (msg.Msg.MSGTYPE & MSGTYPE_STATUS) {
          status = funCAN_Status(pcan_handle);
          if ((int)status < 0) {
              errno = funnGetLastError();
              LOG(ERROR)<<Name()<<" receive: CAN_Status() error.";
              return errno;
          }
      }

    }*/
    else if(canbus_mode == "pcitcp"){
      int can_id = 0x00;
      char data[8] = {0};
      int ret = -1;
      ret = can_read_data(can_fd, &can_id, data);
      if(ret == -1){
         return errno;
      }
      msg.Msg.ID = can_id;
      memcpy((unsigned char *)msg.Msg.DATA, (unsigned char *)data, 8);
      msg.Msg.LEN = 8;
      msg.Msg.MSGTYPE = MSGTYPE_STANDARD;
//        for (int i = 0; i < sizeof(msg.Msg.DATA); ++i) {
//               cout << hex << setw(2) << setfill('0') << static_cast<int>(msg.Msg.DATA[i]) << " ";
//        }
//        cout << endl;
//        LOG(INFO) << Name() << " receive can msg id = " << hex << msg.Msg.ID;
      read_data(&(msg.Msg));

    }
//TODO
    return ROBOT_SUCCESS;
}

int Canbus::read_data(TPCANMsg *pMsgBuff)
{
//    char strId[4] = {0};
//    sprintf(strId,"%3xh",pMsgBuff->ID);
//    char strData[16]={0};
  //    MessageCoder::ConvertUnCharToStr(strData,pMsgBuff->DATA,8);

  //  LOG(INFO)<<Name()<<" Receive CanData : ID: "<<std::string(strId)<<",Data: "<<std::string(strData);

  //通过读取车辆OBD的数据，赋值给OBD的变量  原始代码 开始
//    if(pMsgBuff->ID == robot::common::ObdAccPsng)
//    {
//       T_OBD_ACC_PSNG tOBDAccPsng = *((T_OBD_ACC_PSNG*)MessageCoder::CanMsgAnalyse(robot::common::ObdAccPsng,pMsgBuff->DATA));
//       status1.uiAccPsng = tOBDAccPsng.obdAccPsng;
//       obd.AccPsng = status1.uiAccPsng * 0.1;
//    }
//    else if(pMsgBuff->ID == robot::common::ObdBrkPsng)
//    {
//       T_OBD_BRK_PSNG tOBDBrkPsng = *((T_OBD_BRK_PSNG*)MessageCoder::CanMsgAnalyse(robot::common::ObdBrkPsng,pMsgBuff->DATA));
//       status1.uiBrkPsng = tOBDBrkPsng.obdBrkPsng;
//       obd.BrkPsng =  status1.uiBrkPsng * 0.1;
//    }
//    else if(pMsgBuff->ID == robot::common::ObdSftPsng)
//    {
//       T_OBD_SFT_PSNG tOBDSftPsng = *((T_OBD_SFT_PSNG*)MessageCoder::CanMsgAnalyse(robot::common::ObdSftPsng,pMsgBuff->DATA));
//       status1.uiSftPsng = tOBDSftPsng.obdSftPsng;
//       obd.SftPsng = status1.uiSftPsng;
//    }
//    else if(pMsgBuff->ID == robot::common::ObdVehHBrkStat)
//    {
//       T_OBD_VEH_BRK_STAT tOBDVehBrkStat = *((T_OBD_VEH_BRK_STAT*)MessageCoder::CanMsgAnalyse(robot::common::ObdVehHBrkStat,pMsgBuff->DATA));
//       obd.VehHBrkStat = status1.uiVehHBrkStat = tOBDVehBrkStat.obdVehHBrkStat;
//    }
//    else if(pMsgBuff->ID == robot::common::ObdVehSwStat)
//    {
//        T_OBD_VEH_SW_STAT tOBDVehSwStat = *((T_OBD_VEH_SW_STAT*)MessageCoder::CanMsgAnalyse(robot::common::ObdVehSwStat,pMsgBuff->DATA));
//        obd.VehSwStat = status1.uiVehSwStat = tOBDVehSwStat.obdVehSwStat;
//    }
//    else if(pMsgBuff->ID == robot::common::ObdStrAng)
//    {
//       T_OBD_STR_ANG tOBDStrAng = *((T_OBD_STR_ANG*)MessageCoder::CanMsgAnalyse(robot::common::ObdStrAng,pMsgBuff->DATA));
//       status2.iStrAng = tOBDStrAng.obdStrAng;
//       obd.StrAng = status2.iStrAng * 0.01;
//    }
//    else if(pMsgBuff->ID == robot::common::ObdEngSpd)
//    {
//        T_OBD_ENG_SPD tOBDEngSpd = *((T_OBD_ENG_SPD*)MessageCoder::CanMsgAnalyse(robot::common::ObdEngSpd,pMsgBuff->DATA));
//        status2.uiEngSpd = tOBDEngSpd.obdEngSpd;
//        obd.EngSpd = status2.uiEngSpd;
//    }
//    else if(pMsgBuff->ID == robot::common::ObdSpeed)
//    {
//        T_OBD_VEH_SPD tOBDVehSpd = *((T_OBD_VEH_SPD*)MessageCoder::CanMsgAnalyse(robot::common::ObdSpeed,pMsgBuff->DATA));
//        status2.uiVehSpd = tOBDVehSpd.obdVehSpd;
//        obd.VehSpd = tOBDVehSpd.obdVehSpd;
//    }
//    else if((pMsgBuff->ID >= robot::common::PowerOnReq && pMsgBuff->ID <= robot::common::BrakeForceCtrStp) ||
//            (pMsgBuff->ID >= robot::common::SysMonitorBack && pMsgBuff->ID <= robot::common::SteerPosTimeReq) ||
//            (pMsgBuff->ID >= robot::common::RobotAcuatorPos && pMsgBuff->ID <= robot::common::ComputerStaus) ||
//            (pMsgBuff->ID >= robot::common::SteerTranLineCtr && pMsgBuff->ID <= robot::common::VehicleStart) ||
//            pMsgBuff->ID ==robot::common::EstopInfo || pMsgBuff->ID ==robot::common::EstopErr)
//    {
//        publish_data(pMsgBuff);
//    }
    //通过读取车辆OBD的数据，赋值给OBD的变量  原始代码 结束

    //通过设置相关变量，赋值给OBD的变量   测试版代码 开始
  if(pMsgBuff->ID == robot::common::ObdAccPsng)
  {
   T_OBD_ACC_PSNG tOBDAccPsng = *((T_OBD_ACC_PSNG*)MessageCoder::CanMsgAnalyse(robot::common::ObdAccPsng,pMsgBuff->DATA));
     status1.uiAccPsng = tOBDAccPsng.obdAccPsng;
     status1.uiBrkPsng = 0;
     obd.AccPsng = tOBDAccPsng.obdAccPsng;
     obd.BrkPsng = 0;
     status2.iStrAng = 0;
     obd.StrAng = 0;
  }
  else if((pMsgBuff->ID >= robot::common::RobotWorkMd && pMsgBuff->ID <= robot::common::ICUMonitor) ||
          (pMsgBuff->ID >= robot::common::BrkPedalActCtrReq && pMsgBuff->ID <= robot::common::IOMode) ||
          (pMsgBuff->ID >= robot::common::RobotAcuatorPos && pMsgBuff->ID <= robot::common::ComputerStaus) ||
          (pMsgBuff->ID >= robot::common::SysMonitorBack && pMsgBuff->ID <= robot::common::SteerPosTimeReq) ||
          pMsgBuff->ID ==robot::common::EstopInfo || pMsgBuff->ID ==robot::common::EstopErr ||
          pMsgBuff->ID == robot::common::EstpCtrl)
  {
      publish_data(pMsgBuff);
  }

    //通过设置相关变量，赋值给OBD的变量   测试版代码 结束
    return ROBOT_SUCCESS;
}

void Canbus::publish_data(TPCANMsg *pMsgBuff)
{
try
{
    canbus::canMsg rosMsg;
    rosMsg.id = pMsgBuff->ID;
    //std::stringstream ss;
    //ss << pMsgBuff->DATA;
    //rosMsg.data = ss.str();
    char strId[4] = {0};
    sprintf(strId,"%3xh",pMsgBuff->ID);
    char chData[16]={0};
    MessageCoder::ConvertUnCharToStr(chData,pMsgBuff->DATA,8);
    //std::string strData = chData;
    rosMsg.data = chData;
    canRCUData_pub.publish(rosMsg);
if(pMsgBuff->ID == robot::common::SysLearnBack)
{
//    LOG(INFO)<<Name()<<" Publish CanData :" << "ID:" << strId << ",Data:"<<std::string(chData);
}

}
    catch (ros::Exception ex)
    {
        LOG(ERROR)<< "publish failed "<< ex.what();
    }


}

int Canbus::write_data(TPCANMsg *pMsgBuff, bool print)
{
  char strId[4] = {0};
  sprintf(strId,"%3xh", pMsgBuff->ID);
  char strData[16]={0};
  MessageCoder::ConvertUnCharToStr(strData,pMsgBuff->DATA,8);
  if (print)
    LOG(INFO)<<Name()<<" send CanData : ID: "<<std::string(strId)<<",Data:"<<std::string(strData);

  if (canbus_mode == "tcp") {
    s_g_pBufSet[0] = 0x08;
    s_g_pBufSet[1] = 0x00;
    s_g_pBufSet[2] = 0x00;
    s_g_pBufSet[3] = pMsgBuff->ID / 256;
    s_g_pBufSet[4] = pMsgBuff->ID % 256;
    memcpy(s_g_pBufSet + 5, pMsgBuff->DATA, 8);
//    for (int i = 0; i < sizeof(s_g_pBufSet); ++i) {
//           cout << hex << setw(2) << setfill('0') << static_cast<int>(s_g_pBufSet[i]) << " ";
//    }
//    cout << endl;
//    int send_length = write(canbus_socket, s_g_pBufSet, sizeof(s_g_pBufSet));
    int send_length = send(canbus_socket, s_g_pBufSet, sizeof(s_g_pBufSet), 0);
    if (send_length != sizeof(s_g_pBufSet)){
       LOG(ERROR) << Name() <<" canbus socket write error: " << send_length;
       return errno;
    }
  }
 /* else if (canbus_mode == "usb") {
    if (funCAN_Write(pcan_handle, pMsgBuff)){
       LOG(ERROR)<<Name()<< " write: LINUX_CAN_write_timeout error. ";
       return errno;
    }
  }*/
  else if(canbus_mode == "pcitcp"){
    char data[8] = {0};
    memcpy(data, pMsgBuff->DATA, 8);
    int ret = -1;
    ret = can_write_data(can_fd,pMsgBuff->ID,data,8);
    if(ret == -1)
    {
      return errno;
    }
  }
   return ROBOT_SUCCESS;
}

bool Canbus::ActuatorEnableRequest(canbus::ActuatorEnableRequest::Request &req,
                           canbus::ActuatorEnableRequest::Response &res)
{
    LOG(INFO)<< "controlservice start ActuatorEnableRequest";
    TPCANMsg canActuaEnableReqMsg;
    try{
        T_ACTUATOR_ENABLE_REQUEST request;
        request.uiAccClchCtrReq = req.uiAccClchCtrReq;
        request.uiAccMotEnReq = req.uiAccMotEnReq;
        request.uiBrkMotEnReq = req.uiBrkMotEnReq;
        request.uiClchMotEnReq = req.uiClchMotEnReq;
        request.uiStrMotEnReq = req.uiStrMotEnReq;
        request.uiXSftMotEnReq = req.uiXSftMotEnReq;
        request.uiYSftMotEnReq = req.uiYSftMotEnReq;
        request.uiZSftMotEnReq = req.uiZSftMotEnReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::ActuatorEnableReq, &canActuaEnableReqMsg);
        if (write_data(&canActuaEnableReqMsg)!=ROBOT_SUCCESS)
        {
            res.uiReturn = 0;  //= 0
            return true;
        }
        res.uiReturn = 1;

//       TPCANMsg pMsgTest;
//       T_STR_LRN_PSNG test;
//       test.iClbtStrAng0 = 0;
//        test.iClbtStrAngLftMax = -120000;
//       test.uiClbtStrAngRtMax = 120000;
//       MessageCoder::CanMsgPackage((void*)&test, robot::common::SteerLearnPosBack, &pMsgTest);
//        publish_data(& pMsgTest);
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice ActuatorEnableRequest failed "<< ex.what();
        return true;
    }
    return true;
}


bool Canbus::ExcuteLearnModeRequest(canbus::LearnModeRequest::Request & req,
                           canbus::LearnModeRequest::Response &res)
{
    LOG(INFO)<< "controlservice start ExcuteLearnModeRequest";
    // learn mode request
    TPCANMsg canLearnModeReqMsg;
    try{
        T_LEARN_MODE_REQUEST request;
        request.uiClrAccTbl = req.uiClrAccTbl;
        request.uiClrBrkTbl = req.uiClrBrkTbl;
        request.uiClrClchTbl = req.uiClrClchTbl;
        request.uiClrSftTbl = req.uiClrSftTbl;
        request.uiClrStrTbl= req.uiClrStrTbl;
        request.uiPsngLrnReq = req.uiPsngLrnReq;
        request.uiSftAutLrnFnsConf = req.uiSftAutLrnFnsConf;
        request.uiSftPsngLrnReq = req.uiSftPsngLrnReq;
        request.uiStpAutLrn = req.uiStpAutLrn;
        request.uiSysLrnReq = req.uiSysLrnReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::LearnModeReq, &canLearnModeReqMsg);
        if (write_data(&canLearnModeReqMsg)!=ROBOT_SUCCESS)
        {
            res.uiReturn = 0;  //= 0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice ExcuteLearnModeRequest failed "<< ex.what();
        return true;
    }
    return true;
}


bool Canbus::AccPedalActCtrRequest(canbus::AccPedalActCtrReq::Request &req,
                        canbus::AccPedalActCtrReq::Response &res)
{
    LOG(INFO)<< "controlservice start AccPedalActCtrRequest";
    // AccPedalActCtr request
    TPCANMsg canAccPedalActCtrReqMsg;
    try{
        T_ACC_PEDAL_ACT_CTR_REQ request;
        request.uiAccSpdCtrReq = req.uiAccSpdCtrReq;
        request.uiAccAccCtrReq = req.uiAccAccCtrReq;
        request.uiAccPsngCtrReq = req.uiAccPsngCtrReq;
        request.uiAccDecCtrReq = req.uiAccDecCtrReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::AccPedalActCtrReq, &canAccPedalActCtrReqMsg);
        if (write_data(&canAccPedalActCtrReqMsg)!=ROBOT_SUCCESS)
        {
            res.uiReturn = 0;  //= 0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice AccPedalActCtrRequest failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::BrkPedalActCtrRequest(canbus::BrkPedalActCtrReq::Request &req,
                        canbus::BrkPedalActCtrReq::Response &res)
{
    LOG(INFO)<< "controlservice start BrkPedalActCtrRequest";
    // BrkPedalActCtr request
    TPCANMsg canBrkPedalActCtrReqMsg;
    try{
        T_BRK_PEDAL_ACT_CTR_REQ request;
        request.uiBrkSpdCtrReq = req.uiBrkSpdCtrReq;
        request.uiBrkAccCtrReq = req.uiBrkAccCtrReq;
        request.uiBrkCtrMdReq = req.uiBrkCtrMdReq;
        request.uiBrkDecCtrReq = req.uiBrkDecCtrReq;
        request.uiBrkFocCtrReq = req.uiBrkFocCtrReq;
        request.uiBrkPsngCtrReq = req.uiBrkPsngCtrReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::BrkPedalActCtrReq, &canBrkPedalActCtrReqMsg);
        if (write_data(&canBrkPedalActCtrReqMsg)!=ROBOT_SUCCESS)
        {
            res.uiReturn = 0;  //= 0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice BrkPedalActCtrRequest failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::SftActCtrRequest(canbus::SftActCtrReq::Request &req,
                      canbus::SftActCtrReq::Response &res)
{
    LOG(INFO)<< "controlservice start SftActCtrRequest";
    // SftActCtr request
    TPCANMsg canSftActCtrReqMsg;
    try{
        T_SFT_ACT_CTR_REQ request;
        request.iClchSpdCtrReq = req.iClchSpdCtrReq;
        request.uiClchPsngCtrReq = req.uiClchPsngCtrReq;
        request.uiSftMdReq = req.uiSftMdReq;
        request.uiSftPsngCtrReq = req.uiSftPsngCtrReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::ShiftActCtrReq, &canSftActCtrReqMsg);
        if (write_data(&canSftActCtrReqMsg) != ROBOT_SUCCESS)
        {
            res.uiReturn = 0; // =0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice SftActCtrRequest failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::StrActCtrRequest(canbus::StrActCtrReq::Request &req,
                      canbus::StrActCtrReq::Response &res)
{
    LOG(INFO)<< "controlservice start StrActCtrRequest";
    // StrActCtr request
    TPCANMsg canStrActCtrReqMsg;
    try{
        T_STR_ACT_CTR_REQ request;
        request.iStrAngCtrReq = req.iStrAngCtrReq;
        request.iStrSpdCtrReq = req.iStrSpdCtrReq;
        request.iStrTrqCtrReq = req.iStrTrqCtrReq;
        request.uiStrCtrMdReq = req.uiStrCtrMdReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::SteerActCtrReq, &canStrActCtrReqMsg);
        if (write_data(&canStrActCtrReqMsg) !=ROBOT_SUCCESS)
        {
            res.uiReturn = 0; // =0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice StrActCtrRequest failed "<< ex.what();
        return true;
    }
    return true;
}


bool Canbus::RBTManager(canbus::RBTManager::Request &req,
                      canbus::RBTManager::Response &res)
{
    LOG(INFO)<< "RBTManager start ActuatorEnableRequest";
    TPCANMsg canRobotManagerMsg;
    try{
        T_RBT_MANAGER request;
        request.uiAccMotInt = req.uiAccMotInt;
        request.uiBrkMotInt = req.uiBrkMotInt;
        request.uiClchMotInt = req.uiClchMotInt;
        request.uiFltClr = req.uiFltClr;
        request.uiStrMotInt= req.uiStrMotInt;
        request.uiXSftMotInt = req.uiXSftMotInt;
        request.uiYSftMotInt = req.uiYSftMotInt;
        request.uiZSftMotInt =req.uiZSftMotInt;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::RobotManager, &canRobotManagerMsg);
        if (write_data(&canRobotManagerMsg) != ROBOT_SUCCESS)
        {
            res.uiReturn = 0; // =0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "RBTManager ActuatorEnableRequest failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::PowerOnRequest(canbus::PowerOnRequest::Request &req,
                      canbus::PowerOnRequest::Response &res)
{
    LOG(INFO)<< "controlservice start PowerOnRequest";
    // PowerON request
    TPCANMsg canPowerOnReqMsg;
    try{
        T_POWER_ON_REQUEST request;
        request.uiAccCtrPowReq = req.uiAccCtrPowReq;
        request.uiBrkCtrPowReq = req.uiBrkCtrPowReq;
        request.uiClchCtrPowReq = req.uiClchCtrPowReq;
        request.uiRCUPowReq = req.uiRCUPowReq;
        request.uiStrCtrPowReq= req.uiStrCtrPowReq;
        request.uiXSftCtrPowReq = req.uiXSftArmCtrPowReq;
        request.uiYSftCtrPowReq = req.uiYSftArmCtrPowReq;
        request.uiDriveFanReq = req.uiDriveFanReq;
        request.uiZSftArmCtrPowReq = req.uiZSftArmCtrPowReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::PowerOnReq, &canPowerOnReqMsg);

        if (write_data(&canPowerOnReqMsg) != ROBOT_SUCCESS)
        {
            res.uiReturn = 0; // =0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice PowerOnRequest failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::ActuatorCtrLimit(canbus::ActuatorCtrLimit::Request &req,
                              canbus::ActuatorCtrLimit::Response &res)
{
    LOG(INFO)<< "controlservice start ActuatorCtrLimit";
    // ActuatorCtrLimit request
    TPCANMsg canActCtrLimitMsg;
    try{
        T_ACT_CTR_LIMT request;
        request.iStrMaxLimt = req.iStrMaxLimit;
        request.uiAccMaxLimt = req.uiAccMaxLimit;
        request.uiBrkMaxLimt = req.uiBrkMaxLimit;
        request.uiSftType = req.uiSftType;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::ActuatorCtrLimit, &canActCtrLimitMsg);

        if (write_data(&canActCtrLimitMsg) != ROBOT_SUCCESS)
        {
            res.uiReturn = 0; // =0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice ActuatorCtrLimit failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::VehicleStart(canbus::VehicleStart::Request &req,
                           canbus::VehicleStart::Response &res)
{
    LOG(INFO)<< "controlservice start VehicleStart";
    // VehicleStart request
    TPCANMsg canVechStrtMsg;
    try{
        T_VEHICLE_START request;
        request.uiVehSwReq = req.uiVehSwReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::VehicleStart, &canVechStrtMsg);

        if (write_data(&canVechStrtMsg) != ROBOT_SUCCESS)
        {
            res.uiReturn = 0; // =0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice VehicleStart failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::SlopeMode(canbus::SlopeMode::Request &req,
                       canbus::SlopeMode::Response &res)
{
    LOG(INFO)<< "controlservice start SlopeMode";
    // SlopeMode request
    TPCANMsg canSlpMdMsg;
    try{
        T_SLOPE_MODE request;
        request.iSlopeRequest = req.iSlopeRequest;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::SlopeMODE, &canSlpMdMsg);

        if (write_data(&canSlpMdMsg) != ROBOT_SUCCESS)
        {
            res.uiReturn = 0; // =0
            return true;
        }
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice SlopeMode failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::StrLoopReq(canbus::StrLoopReq::Request &req,
                       canbus::StrLoopReq::Response &res)
{
    LOG(INFO)<< "controlservice start StrLoopReq";
    // StrLoopReq request
    try{
      if(req.uiStart == 0)
      {
        if(strTimerState)
        {
          str_timer_.stop();
          strTimerState = false;
          strLoopNum = 0;
          tStrActCtrReq = {0};
        }
        res.uiReturn = 1;
        return true;
      }
      if(strTimerState)
      {
        str_timer_.stop();
        strTimerState = false;
        strLoopNum = 0;
        tStrActCtrReq = {0};
      }
      strLpReq = req;

      double str_duration = (double)req.uiStrTimeInterval/1000;
      strLoopNum = 0;
      strIncrease = true;
      strAngle = strLpReq.iStrAngStart;
      tStrActCtrReq.iStrSpdCtrReq = strLpReq.uiStrSpdCtrReq;
      tStrActCtrReq.iStrTrqCtrReq = strLpReq.iStrTrqCtrReq;
      tStrActCtrReq.uiStrCtrMdReq = strLpReq.uiStrCtrMdReq;
      str_timer_ = CreateTimer(ros::Duration(str_duration), &Canbus::StrLoopOnTimer, this,false,false);
      str_timer_.start();
      strTimerState = true;
      res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice StrLoopReq failed "<< ex.what();
        return true;
    }
    return true;
}

void Canbus::StrLoopOnTimer(const ros::TimerEvent &event)
{
  //LOG(INFO)<< Name()<< " StrLoopTimer start ..";
  TPCANMsg canStrActCtrReqMsg;
  try
  {
    // 发送当前角度
    tStrActCtrReq.iStrAngCtrReq = strAngle;
    MessageCoder::CanMsgPackage((void*)&tStrActCtrReq, robot::common::SteerActCtrReq, &canStrActCtrReqMsg);
    write_data(&canStrActCtrReqMsg);
    //LOG(INFO)<<Name()<< " Angle "<< std::to_string(tStrActCtrReq.iStrAngCtrReq);
    //LOG(INFO)<<Name()<< " Num "<< std::to_string(strLoopNum);
    if (strAngle <= strLpReq.iStrAngStart)
      strLoopNum++;
    if(strLoopNum > strLpReq.uiStrFrequency)  //如果执行次数等于往复次数，则结束定时器
    {
      str_timer_.stop();
      strTimerState = false;
      strLoopNum = 1;
      tStrActCtrReq = {0};
      return;
    }
    if (strIncrease)
    {
      strAngle += strLpReq.uiStrSpan;
      if (strAngle >= strLpReq.iStrAngEnd)
        strIncrease = false;
    }
    else
    {
      strAngle -= strLpReq.uiStrSpan;
      if (strAngle <= strLpReq.iStrAngStart)
        strIncrease = true;
    }
  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " StrLoopTimer error. "<< ex.what();
  }
}


bool Canbus::AccLoopReq(canbus::AccLoopReq::Request &req,
                        canbus::AccLoopReq::Response &res)
{
  LOG(INFO)<< "controlservice start AccLoopReq";
  // AccLoopReq request
  try{
    if(req.uiStart == 0)
    {
      if(accTimerState)
      {
        acc_timer_.stop();
        accTimerState = false;
        accLoopNum = 0;
        tAccCtrReq = {0};
      }
      res.uiReturn = 1;
      return true;
    }
    if(accTimerState)
    {
      acc_timer_.stop();
      accTimerState = false;
      accLoopNum = 0;
      tAccCtrReq = {0};
    }
    accLpReq = req;

    double acc_duration = (double)req.uiAccTimeInterval/1000;
    accLoopNum = 0;
    accIncrease = true;
    accPsng = accLpReq.uiAccPsngStart;
    tAccCtrReq.uiAccSpdCtrReq = accLpReq.uiAccSpdCtrReq;
    tAccCtrReq.uiAccAccCtrReq = accLpReq.uiAccAccCtrReq;
    acc_timer_ = CreateTimer(ros::Duration(acc_duration), &Canbus::AccLoopOnTimer, this,false,false);
    acc_timer_.start();
    accTimerState = true;
    res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice AccLoopReq failed "<< ex.what();
      return true;
  }
  return true;
}

void Canbus::AccLoopOnTimer(const ros::TimerEvent &event)
{
  LOG(INFO)<< Name()<< " AccLoopTimer start ..";

  TPCANMsg canAccCtrReqMsg;
  try
  {
    tAccCtrReq.uiAccPsngCtrReq = accPsng;
    MessageCoder::CanMsgPackage((void*)&tAccCtrReq, robot::common::AccPedalActCtrReq, &canAccCtrReqMsg);
    write_data(&canAccCtrReqMsg);
    if(accLpReq.uiAccPsngStart < accLpReq.uiAccPsngEnd)
    {
      if (accPsng <= accLpReq.uiAccPsngStart)
        accLoopNum++;
      if(accLoopNum > accLpReq.uiAccFrequency)  //如果执行次数等于往复次数，则结束定时器
      {
        acc_timer_.stop();
        accTimerState = false;
        accLoopNum = 1;
        tAccCtrReq = {0};
        return;
      }
      if (accIncrease)
      {
        accPsng += accLpReq.uiAccSpan;
        if (accPsng >= accLpReq.uiAccPsngEnd)
          accIncrease = false;
      }
      else
      {
        accPsng -= accLpReq.uiAccSpan;
        if (accPsng <= accLpReq.uiAccPsngStart)
          accIncrease = true;
      }
    }
    else
    {
      if (accPsng >= accLpReq.uiAccPsngStart)
        accLoopNum++;
      if(accLoopNum > accLpReq.uiAccFrequency)  //如果执行次数等于往复次数，则结束定时器
      {
        acc_timer_.stop();
        accTimerState = false;
        accLoopNum = 1;
        tAccCtrReq = {0};
        return;
      }
      if (accIncrease)
      {
        accPsng -= accLpReq.uiAccSpan;
        if (accPsng <= accLpReq.uiAccPsngEnd)
          accIncrease = false;
      }
      else
      {
        accPsng += accLpReq.uiAccSpan;
        if (accPsng >= accLpReq.uiAccPsngStart)
          accIncrease = true;
      }
    }

  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " AccLoopTimer error. "<< ex.what();
  }

}


bool Canbus::BrkLoopReq(canbus::BrkLoopReq::Request &req,
                        canbus::BrkLoopReq::Response &res)
{
  LOG(INFO)<< "controlservice start BrkLoopReq";
  // BrkLoopReq request
  try{
    if(req.uiStart == 0)
    {
      if(brkTimerState)
      {
        brk_timer_.stop();
        brkTimerState = false;
        brkLoopNum = 0;
        tBrkCtrReq = {0};
      }
      res.uiReturn = 1;
      return true;
    }
    if(brkTimerState)
    {
      brk_timer_.stop();
      brkTimerState = false;
      brkLoopNum = 0;
      tBrkCtrReq = {0};
    }
    brkLpReq = req;

    double brk_duration = (double)req.uiBrkTimeInterval/1000;
    brkLoopNum = 0;
    brkIncrease = true;
    brkPsng = brkLpReq.uiBrkPsngStart;
    tBrkCtrReq.uiBrkSpdCtrReq = brkLpReq.uiBrkSpdCtrReq;
    tBrkCtrReq.uiBrkCtrMdReq = brkLpReq.uiBrkCtrMdReq;
    tBrkCtrReq.uiBrkFocCtrReq = brkLpReq.uiBrkFocCtrReq;
    tBrkCtrReq.uiBrkAccCtrReq = brkLpReq.uiBrkAccCtrReq;
    tBrkCtrReq.uiBrkDecCtrReq = brkLpReq.uiBrkDecCtrReq;
    brk_timer_ = CreateTimer(ros::Duration(brk_duration), &Canbus::BrkLoopOnTimer, this,false,false);
    brk_timer_.start();
    brkTimerState = true;
    res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice BrkLoopReq failed "<< ex.what();
      return true;
  }
  return true;
}

void Canbus::BrkLoopOnTimer(const ros::TimerEvent &event)
{
  LOG(INFO)<< Name()<< " BrkLoopTimer start ..";

  TPCANMsg canBrkCtrReqMsg;
  try
  {
    tBrkCtrReq.uiBrkPsngCtrReq = brkPsng;
    MessageCoder::CanMsgPackage((void*)&tBrkCtrReq, robot::common::BrkPedalActCtrReq, &canBrkCtrReqMsg);
    write_data(&canBrkCtrReqMsg);
    if (brkPsng <= brkLpReq.uiBrkPsngStart)
      brkLoopNum++;
    if(brkLoopNum > brkLpReq.uiBrkFrequency)  //如果执行次数等于往复次数，则结束定时器
    {
      brk_timer_.stop();
      brkTimerState = false;
      brkLoopNum = 1;
      tBrkCtrReq = {0};
      return;
    }
    if (brkIncrease)
    {
      brkPsng += brkLpReq.uiBrkSpan;
      if (brkPsng >= brkLpReq.uiBrkPsngEnd)
        brkIncrease = false;
    }
    else
    {
      brkPsng -= brkLpReq.uiBrkSpan;
      if (brkPsng <= brkLpReq.uiBrkPsngStart)
        brkIncrease = true;
    }

  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " BrkLoopTimer error. "<< ex.what();
  }

}

bool Canbus::StrTxtReq(canbus::StrTxtReq::Request &req,canbus::StrTxtReq::Response &res)
{
    LOG(INFO)<< "controlservice start StrTxtReq";
    // StrTxtReq request
    try{
      if(req.uiStrTxtFlag == 0)
      {
        if(strTxtTimerState)
        {
          str_txt_timer_.stop();
          strTxtTimerState = false;
          angles_count = 0;
        }
        res.uiReturn = 1;
        return true;
      }
      string str = "/home/madesheng/angle.txt";
      int flag = Canbus::read_txt(str,1);
      if(flag != ROBOT_SUCCESS)
      {
        res.uiReturn = 0;
        return false;
      }
      if(strTxtTimerState)
      {
        str_txt_timer_.stop();
        strTxtTimerState = false;
        angles_count = 0;
      }
      angles_count = 0;
      tStrActCtrReqTxt.iStrSpdCtrReq = req.uiStrSpdCtrReq;
      tStrActCtrReqTxt.iStrTrqCtrReq = req.iStrTrqCtrReq;
      tStrActCtrReqTxt.uiStrCtrMdReq = req.uiStrCtrMdReq;

      str_txt_timer_ = CreateTimer(ros::Duration(duration_txt), &Canbus::StrTxtOnTimer, this,false,false);
      str_txt_timer_.start();
      strTxtTimerState = true;
      res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice StrTxtReq failed "<< ex.what();
        return true;
    }
    return true;
}

void Canbus::StrTxtOnTimer(const ros::TimerEvent &event)
{
  try
  {
    if(angles_count == angles_number)
    {
      str_txt_timer_.stop();
      strTxtTimerState = false;;
      angles_count = 0;
LOG(INFO)<<Name()<< " StrTxt END. ";
      return;
    }

    TPCANMsg canStrActCtrReqMsg;
    int angle = angles[angles_count] * 100;
    tStrActCtrReqTxt.iStrAngCtrReq = angle;
    //20220805zw 增加速度配套转速文件
    if (hasSpeedFile)
    {
      int angle_speed = angle_speeds[angles_count] * 100;
      tStrActCtrReqTxt.iStrSpdCtrReq = angle_speed;
      LOG(INFO)<<Name()<< " angle:. " <<angle<<" angle_speed:"<<angle_speed;
    }
    else
      LOG(INFO)<<Name()<< " angle:. " <<angle;
    MessageCoder::CanMsgPackage((void*)&tStrActCtrReqTxt, robot::common::SteerActCtrReq, &canStrActCtrReqMsg);
    write_data(&canStrActCtrReqMsg,true);
    angles_count ++;
  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " OnTimer error. "<< ex.what();
  }
}

bool Canbus::BrkTxtReq(canbus::BrkTxtReq::Request &req,canbus::BrkTxtReq::Response &res)
{
  LOG(INFO)<< "controlservice start BrkTxtReq";
  // BrkTxtReq request
  try{
    if(req.uiBrkTxtFlag == 0)
    {
      if(brkTxtTimerState)
      {
        brk_txt_timer_.stop();
        brkTxtTimerState = false;
        psng_count = 0;
      }
      res.uiReturn = 1;
      return true;
    }
    string str = "/home/madesheng/psng.txt";
    int flag = Canbus::read_txt(str,2);
    if(flag != ROBOT_SUCCESS)
    {
      res.uiReturn = 0;
      return false;
    }
    if(brkTxtTimerState)
    {
      brk_txt_timer_.stop();
      brkTxtTimerState = false;
      psng_count = 0;
    }
    psng_count = 0;
    tBrkCtrReqTxt.uiBrkAccCtrReq = req.uiBrkAccCtrReq;
    tBrkCtrReqTxt.uiBrkSpdCtrReq = req.uiBrkSpdCtrReq;
    tBrkCtrReqTxt.uiBrkDecCtrReq = req.uiBrkDecCtrReq;
    tBrkCtrReqTxt.uiBrkCtrMdReq = req.uiBrkCtrMdReq;
    tBrkCtrReqTxt.uiBrkFocCtrReq = req.uiBrkFocCtrReq;

    brk_txt_timer_ = CreateTimer(ros::Duration(duration_txt), &Canbus::BrkTxtOnTimer, this,false,false);
    brk_txt_timer_.start();
    brkTxtTimerState = true;
    res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice StrTxtReq failed "<< ex.what();
      return true;
  }
  return true;
}

void Canbus::BrkTxtOnTimer(const ros::TimerEvent &event)
{
  try
  {
    if(psng_count == psng_number)
    {
      brk_txt_timer_.stop();
      brkTxtTimerState = false;;
      psng_count = 0;
LOG(INFO)<<Name()<< " BrkTxt END. ";
      return;
    }

    TPCANMsg canBrkCtrReqMsg;
    int psng = psngs[psng_count] * 10;

    tBrkCtrReqTxt.uiBrkPsngCtrReq = psng;
    LOG(INFO)<<Name()<< " psng:. " <<psng;
    MessageCoder::CanMsgPackage((void*)&tBrkCtrReqTxt, robot::common::BrkPedalActCtrReq, &canBrkCtrReqMsg);
    write_data(&canBrkCtrReqMsg,true);
    psng_count ++;
  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " OnTimer error. "<< ex.what();
  }
}

bool Canbus::AccTxtReq(canbus::AccTxtReq::Request &req,canbus::AccTxtReq::Response &res)
{
  LOG(INFO)<< "controlservice start AccTxtReq";
  // AccTxtReq request
  try{
    if(req.uiAccTxtFlag == 0)
    {
      if(accTxtTimerState)
      {
        acc_txt_timer_.stop();
        accTxtTimerState = false;
        acc_psng_count = 0;
      }
      res.uiReturn = 1;
      return true;
    }
    string str = "/home/madesheng/acc.txt";
    int flag = Canbus::read_txt(str,3);
    if(flag != ROBOT_SUCCESS)
    {
      res.uiReturn = 0;
      return false;
    }
    if(accTxtTimerState)
    {
      acc_txt_timer_.stop();
      accTxtTimerState = false;
      acc_psng_count = 0;
    }
    acc_psng_count = 0;
    tAccCtrReqTxt.uiAccSpdCtrReq = req.uiAccSpdCtrReq;
    tAccCtrReqTxt.uiAccAccCtrReq = req.uiAccAccCtrReq;
    tAccCtrReqTxt.uiAccDecCtrReq = req.uiAccDecCtrReq;

    acc_txt_timer_ = CreateTimer(ros::Duration(duration_txt_acc), &Canbus::AccTxtOnTimer, this,false,false);
    acc_txt_timer_.start();
    accTxtTimerState = true;
    res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice AccTxtReq failed "<< ex.what();
      return true;
  }
  return true;
}

void Canbus::AccTxtOnTimer(const ros::TimerEvent &event)
{
  try
  {
    if(acc_psng_count == acc_psng_number)
    {
      acc_txt_timer_.stop();
      accTxtTimerState = false;;
      acc_psng_count = 0;
      return;
    }

    TPCANMsg canAccCtrReqMsg;
    int psng = acc_psngs[acc_psng_count] * 10;

    tAccCtrReqTxt.uiAccPsngCtrReq = psng;

    LOG(INFO)<<Name()<< "acc psng:. " <<psng;
    MessageCoder::CanMsgPackage((void*)&tAccCtrReqTxt, robot::common::AccPedalActCtrReq, &canAccCtrReqMsg);
    write_data(&canAccCtrReqMsg,true);
    acc_psng_count ++;
  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " OnTimer error. "<< ex.what();
  }
}

bool Canbus::StrTxtLineReq(canbus::StrTxtLineReq::Request &req,canbus::StrTxtLineReq::Response &res)
{
    LOG(INFO)<< "controlservice start StrTxtLineReq";
    // StrTxtLineReq request
    try{
      if(req.uiStrTxtLineFlag == 0)
      {
        if(strTxtLineTimerState)
        {
          str_txt_line_timer_.stop();
          strTxtLineTimerState = false;
          angles_line_count = 0;
        }
        res.uiReturn = 1;
        return true;
      }
      string str = "/home/madesheng/angleLine.txt";
      int flag = Canbus::read_txt(str,4);
      if(flag != ROBOT_SUCCESS)
      {
        res.uiReturn = 0;
        return false;
      }
      if(strTxtLineTimerState)
      {
        str_txt_line_timer_.stop();
        strTxtLineTimerState = false;
        angles_line_count = 0;
      }
      angles_line_count = 0;
      tStrActCtrReqTxtLine.iStrSpdCtrReq = req.uiStrSpdCtrReq;
      tStrActCtrReqTxtLine.iStrTrqCtrReq = req.iStrTrqCtrReq;
      tStrActCtrReqTxtLine.uiStrCtrMdReq = req.uiStrCtrMdReq;

      str_txt_line_timer_ = CreateTimer(ros::Duration(duration_txt), &Canbus::StrTxtLineOnTimer, this,false,false);
      str_txt_line_timer_.start();
      strTxtLineTimerState = true;
      res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice StrTxtLineReq failed "<< ex.what();
        return true;
    }
    return true;
}

void Canbus::StrTxtLineOnTimer(const ros::TimerEvent &event)
{
  try
  {
    if(angles_line_count == angles_line_number)
    {
      str_txt_line_timer_.stop();
      strTxtLineTimerState = false;;
      angles_line_count = 0;
      return;
    }

    TPCANMsg canStrActCtrReqMsg;
    int angle = angles_line[angles_line_count] * 100;

    tStrActCtrReqTxtLine.iStrAngCtrReq = angle;
    LOG(INFO)<<Name()<< " angle:. " <<angle;
    MessageCoder::CanMsgPackage((void*)&tStrActCtrReqTxtLine, robot::common::SteerActCtrReq, &canStrActCtrReqMsg);
    write_data(&canStrActCtrReqMsg,true);
    angles_line_count ++;
  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " OnTimer error. "<< ex.what();
  }
}

bool Canbus::SftTxtReq(canbus::SftTxtReq::Request &req,canbus::SftTxtReq::Response &res)
{
  LOG(INFO)<< "controlservice start SftTxtReq";
  // SftTxtReq request
  try{
    if(req.uiSftTxtFlag == 0)
    {
      if(sftTxtTimerState)
      {
        sft_txt_timer_.stop();
        sftTxtTimerState = false;
        sft_psng_count = 0;
      }
      res.uiReturn = 1;
      return true;
    }
    string str = "/home/madesheng/sft.txt";
    int flag = Canbus::read_txt(str,5);
    if(flag != ROBOT_SUCCESS)
    {
      res.uiReturn = 0;
      return false;
    }
    if(sftTxtTimerState)
    {
      sft_txt_timer_.stop();
      sftTxtTimerState = false;
      sft_psng_count = 0;
    }
    sft_psng_count = 0;
    tSftCtrReqTxt.iClchSpdCtrReq = req.iClchSpdCtrReq;
    tSftCtrReqTxt.uiSftMdReq = req.uiSftMdReq;
    tSftCtrReqTxt.uiClchPsngCtrReq = req.uiClchPsngCtrReq;

    sft_txt_timer_ = CreateTimer(ros::Duration(duration_txt), &Canbus::SftTxtOnTimer, this,false,false);
    sft_txt_timer_.start();
    sftTxtTimerState = true;
    res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice SftTxtReq failed "<< ex.what();
      return true;
  }
  return true;
}

void Canbus::SftTxtOnTimer(const ros::TimerEvent &event)
{
  try
  {
    if(sft_psng_count == sft_psng_number)
    {
      sft_txt_timer_.stop();
      sftTxtTimerState = false;;
      sft_psng_count = 0;
      return;
    }

    TPCANMsg canSftCtrReqMsg;
    int psng = sft_psngs[sft_psng_count];

    tSftCtrReqTxt.uiSftPsngCtrReq = psng;

    LOG(INFO)<<Name()<< "sft psng:. " <<psng;
    MessageCoder::CanMsgPackage((void*)&tSftCtrReqTxt, robot::common::ShiftActCtrReq, &canSftCtrReqMsg);
    write_data(&canSftCtrReqMsg,true);
    sft_psng_count ++;
  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " OnTimer error. "<< ex.what();
  }
}

int Canbus::read_txt(string &path,int flag)
{
  //ifstream myfile("/home/madesheng/data.txt");
  ifstream myfile(path);
  if (!myfile.is_open())
  {
      LOG(ERROR)<<Name()<< "open src File  Error opening file" << endl;
      return ROBOT_FAILTURE;
  }
  //20220805zw 增加读取角度文件
  ifstream speedfile("/home/siasun/speed.txt");
  hasSpeedFile = speedfile.is_open();
  //-------------------
  if (flag == 1)
  {
    for (int i = 0; i < 70000; i++)
    {
        if(myfile.eof())
        {
            angles_number = i;
            break;
        }
        myfile >> angles[i];
        speedfile >> angle_speeds[i];
        //myfile >> temp;
        //cout <<"char:"<< temp<< endl;
       //cout <<"angle:"<< angles[i] <<endl;
    }
    cout << "str NUMBER:" <<angles_number<<endl;
  }
  else if(flag ==2)
  {
    for (int i = 0; i < 50000; i++)
    {
        if(myfile.eof())
        {
            psng_number = i;
            break;
        }
        myfile >> psngs[i];

        //myfile >> temp;
        //cout <<"char:"<< temp<< endl;
      //  cout <<"psngs:"<< psngs[i] <<endl;
    }
    cout << "brk NUMBER:" <<psng_number<<endl;
  }
  else if(flag ==3)
  {
    for (int i = 0; i < 80000; i++)
    {
        if(myfile.eof())
        {
            acc_psng_number = i;
            break;
        }
        myfile >> acc_psngs[i];

      //    cout <<"psngs:"<< acc_psngs[i] <<endl;
    }
    cout << "acc NUMBER:" <<acc_psng_number<<endl;
  }
  else if(flag ==4)
  {
    for (int i = 0; i < 5000; i++)
    {
        if(myfile.eof())
        {
            angles_line_number  = i;
            break;
        }
        myfile >> angles_line[i];

      //    cout <<"psngs:"<< acc_psngs[i] <<endl;
    }
    cout << "angle line NUMBER:" <<angles_line_number<<endl;
  }
  else if(flag == 5)
  {
    for (int i = 0; i < 70000; i++)
    {
        if(myfile.eof())
        {
            sft_psng_number  = i;
            break;
        }
        myfile >> sft_psngs[i];

      //    cout <<"psngs:"<< acc_psngs[i] <<endl;
    }
    cout << "sft NUMBER:" <<sft_psng_number<<endl;
  }
  else if(flag == 6)
  {
    for (int i = 0; i < 19000; i++)
    {
        if(myfile.eof())
        {
            nedc_speed_number  = i;
            break;
        }
        myfile >> nedc_speeds[i];

    }
    cout << "nedc speed NUMBER:" <<nedc_speed_number<<endl;
  }
  else if(flag == 7)
  {
    for (int i = 0; i < 10; i++)
    {
        if(myfile.eof())
        {
            fuzzy_pid_number  = i;
            break;
        }
        myfile >> fuzzy_pid[i];

    }
   // cout << "Fuzzy PID NUMBER:" <<fuzzy_pid_number<<endl;
  }


   myfile.close();
   return ROBOT_SUCCESS;
}


bool Canbus::VehicleStatSetReq(canbus::VehicleStatSetReq::Request &req,
                           canbus::VehicleStatSetReq::Response &res)
{
    LOG(INFO)<< "controlservice start VehicleStatSetReq";

    try{
        obd.SftPsng = status1.uiSftPsng = req.uiSftPsng;
        obd.VehHBrkStat = status1.uiVehHBrkStat = req.uiVehHBrkStat;
        obd.VehSwStat = status1.uiVehSwStat = req.uiVehSwStat;
        obd.VehSpd = status2.uiVehSpd = req.uiVehSpd;

        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice VehicleStatSetReq failed "<< ex.what();
        return true;
    }
    return true;
}

bool Canbus::RbtDriverControl(canbus::RbtDriverCtrl::Request &req,
                              canbus::RbtDriverCtrl::Response &res)
{
  LOG(INFO)<< "controlservice start RbtDriverControl";

  TPCANMsg canRbtDriverCtrl;
  try{
      T_RBT_DRIVER_CTR request;
      request.uiAccMotCtrl = req.uiAccMotCtrl;
      request.uiBrkMotCtrl = req.uiBrkMotCtrl;
      request.uiClchMotCtrl = req.uiClchMotCtrl;
      request.uiMotMove = req.uiMotMove;
      request.uiStrMotCtrl= req.uiStrMotCtrl;
      request.uiXSftMotCtrl = req.uiXSftMotCtrl;
      request.uiYSftMotCtrl = req.uiYSftMotCtrl;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::RobotDriverCtr, &canRbtDriverCtrl);

      if (write_data(&canRbtDriverCtrl) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice RbtDriverControl failed "<< ex.what();
      return true;
  }
  return true;

}


bool Canbus::StrTangLineCtrReq(canbus::StrTangLineCtrReq::Request &req,
                              canbus::StrTangLineCtrReq::Response &res)
{
  LOG(INFO)<< "controlservice start StrTangLineCtrReq";

  TPCANMsg canStrTangLineCtrReq;
  try{
      T_STR_TRIANGLE_LINE_CTR request;
      request.iStrPstBsc = req.iStrPstBsc;
      request.iStrSpdk = req.iStrSpdk;
      request.iStrTA = req.iStrTA;
      request.uiStrLT = req.uiStrLT;
      request.uiStrNum= req.uiStrNum;
      request.uiStrTT = req.uiStrTT;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::SteerTranLineCtr, &canStrTangLineCtrReq);

      if (write_data(&canStrTangLineCtrReq) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice StrTangLineCtrReq failed "<< ex.what();
      return true;
  }
  return true;

}

bool Canbus::StrSinCtrReq(canbus::StrSinCtrReq::Request &req,
                          canbus::StrSinCtrReq::Response &res)
{
  LOG(INFO)<< "controlservice start StrSinCtrReq";

  TPCANMsg canStrSinCtrReq;
  try{
      T_STR_SIN_CTR request;
      request.iStrFi = req.iStrFi;
      request.uiStrA = req.uiStrA;
      request.uiStrDt = req.uiStrDt;
      request.uiStrFrtb = req.uiStrFrtb;
      request.uiStrFrtk= req.uiStrFrtk;
      request.uiStrT = req.uiStrT;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::SteerSinCtr, &canStrSinCtrReq);

      if (write_data(&canStrSinCtrReq) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice StrSinCtrReq failed "<< ex.what();
      return true;
  }
  return true;

}


bool Canbus::StrPosTimCtrReq(canbus::StrPosTimCtrReq::Request &req,
                             canbus::StrPosTimCtrReq::Response &res)
{
  LOG(INFO)<< "controlservice start StrPosTimCtrReq";

  TPCANMsg canStrPosTimCtrReq;
  try{
      T_STR_POSITION_TIME_CTR request;
      request.iStrAngReq = req.iStrAngReq;
      request.uiNum = req.uiNum;
      request.uiRelTime = req.uiRelTime;
      request.uiSequenceID = req.uiSequenceID;
      request.uiTotalNum= req.uiTotalNum;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::SteerPositionTimeCtr, &canStrPosTimCtrReq);

      if (write_data(&canStrPosTimCtrReq) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice StrPosTimCtrReq failed "<< ex.what();
      return true;
  }
  return true;

}


bool Canbus::EstpCtrl(canbus::EstpCtrl::Request &req,
                      canbus::EstpCtrl::Response &res)
{
  LOG(INFO)<< "controlservice start EstpCtrl";

  TPCANMsg canEstpCtrl;
  try{
      T_ESTP_CTRL request;
      request.uiEstpEnReq = req.uiEstpEnReq;
      request.uiPauseEnReq = req.uiPauseEnReq;
      request.uiPmpEnReq = req.uiPmpEnReq;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::EstpCtrl, &canEstpCtrl);

      if (write_data(&canEstpCtrl) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice EstpCtrl failed "<< ex.what();
      return true;
  }
  return true;

}

bool Canbus::RobotWorkMd(canbus::RobotWorkMd::Request &req,
                         canbus::RobotWorkMd::Response &res)
{
  LOG(INFO)<< "controlservice start RobotWorkMd";

  TPCANMsg canRobotWorkMd;
  try{
      T_WORKING_MODE_CONFING request;
      request.uiOnlineOffline = req.uiOnlineOffline;
      request.uiRbtAplySenro = req.uiRbtAplySenro;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::RobotWorkMd, &canRobotWorkMd);

      if (write_data(&canRobotWorkMd) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }

      workModeCnfg.uiOnlineOffline = req.uiOnlineOffline;
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice EstpCtrl failed "<< ex.what();
      return true;
  }
  return true;

}

bool Canbus::SysCtrlMode(canbus::SysCtrlMode::Request &req,
                         canbus::SysCtrlMode::Response &res)
{
  LOG(INFO)<< "controlservice start SysCtrlMode";

  TPCANMsg canSysCtrlMode;
  try{
      T_CONTROL_MODE request;
      request.uiCtrMdReq = req.uiCtrMdReq;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::SysCtrlMode, &canSysCtrlMode);

      if (write_data(&canSysCtrlMode) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice SysCtrlMode failed "<< ex.what();
      return true;
  }
  return true;

}

bool Canbus::BrkTimCtrlStp(canbus::BrkTimCtrlStp::Request &req,
                         canbus::BrkTimCtrlStp::Response &res)
{
  LOG(INFO)<< "controlservice start BrkTimCtrlStp";

  TPCANMsg canBrkTimCtrlStp;
  try{
      T_BRK_FORCE_CTR_SETUP request;
      request.iCtrSpd = req.iCtrSpd;
      request.uiBrkCtrWay = req.uiBrkCtrWay;
      request.uiBrkTm = req.uiBrkTm;
      request.uiBscCtr = req.uiBscCtr;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::BrkTimCtrlStp, &canBrkTimCtrlStp);

      if (write_data(&canBrkTimCtrlStp) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice BrkTimCtrlStp failed "<< ex.what();
      return true;
  }
  return true;

}

bool Canbus::AccPosCtrlStp(canbus::AccPosCtrlStp::Request &req,
                         canbus::AccPosCtrlStp::Response &res)
{
  LOG(INFO)<< "controlservice start AccPosCtrlStp";

  TPCANMsg canAccPosCtrlStp;
  try{
      T_ACC_POSITION_CTR_SETUP request;
      request.iAccSpdP = req.iAccSpdP;
      request.uiAccPstBsc = req.uiAccPstBsc;
      request.uiAccPT = req.uiAccPT;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::AccPositionCtrStp, &canAccPosCtrlStp);

      if (write_data(&canAccPosCtrlStp) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice AccPosCtrlStp failed "<< ex.what();
      return true;
  }
  return true;

}


bool Canbus::StrAccCtrlReq(canbus::StrAccCtrlReq::Request &req,
                         canbus::StrAccCtrlReq::Response &res)
{
  LOG(INFO)<< "controlservice start StrAccCtrlReq";

  TPCANMsg canStrAccCtrlReq;
  try{
      T_STR_ACC_CTR_REQ request;
      request.uiStrAccCtrReq = req.uiStrAccCtrReq;
      request.uiStrDecCtrReq = req.uiStrDecCtrReq;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::ActuatorAccelerateCtrReq, &canStrAccCtrlReq);

      if (write_data(&canStrAccCtrlReq) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice StrAccCtrlReq failed "<< ex.what();
      return true;
  }
  return true;

}

bool Canbus::RbtSlfChckCtrlReq(canbus::RbtSlfChckCtrlReq::Request &req,
                         canbus::RbtSlfChckCtrlReq::Response &res)
{
  LOG(INFO)<< "controlservice start RbtSlfChckCtrlReq";

  TPCANMsg canRbtSlfChckCtrlReq;
  try{
      T_RBT_SLF_CHK_CTR_REQ request;
      request.uiRbtSlfChkReq = req.uiRbtSlfChkReq;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::RobotSelfCheckCtrReq, &canRbtSlfChckCtrlReq);

      if (write_data(&canRbtSlfChckCtrlReq) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice RbtSlfChckCtrlReq failed "<< ex.what();
      return true;
  }
  return true;

}

bool Canbus::IOMode(canbus::IOMode::Request &req,
                         canbus::IOMode::Response &res)
{
  LOG(INFO)<< "controlservice start IOMode";

  TPCANMsg canIOMode;
  try{
      T_IO_REQ request;
      request.IO_0Req = req.uiIO_0Req;
      request.IO_1Req = req.uiIO_1Req;

      MessageCoder::CanMsgPackage((void*)&request, robot::common::IOMode, &canIOMode);

      if (write_data(&canIOMode) != ROBOT_SUCCESS)
      {
          res.uiReturn = 0; // =0
          return true;
      }
      res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice IOMode failed "<< ex.what();
      return true;
  }
  return true;

}

bool Canbus::RackTestReq(canbus::RackTestReq::Request &req,
                         canbus::RackTestReq::Response &res)
{
  LOG(INFO)<< "controlservice start RackTestReq";

  // RackTestReq request
  try{
    if(req.uiStart == 0)
    {
      rackTestState = 0;
      res.uiReturn = 1;
      return true;
    }
    if(req.uiStart == 2)
    {
      if(rack_set_speed == 0 && rack_actual_speed == 0)
      {
        rackTestState = 2;
        res.uiReturn = 1;
        return true;
      }
      else
      {
        res.uiReturn = 3;
        return true;
      }

    }
    if(rackTestState == 1)
    {
      res.uiReturn = 2;
      return true;
    }

    nedc_speeds[1810] = {0};
    string str = "/home/madesheng/publish/wwwroot/Upload/File/CLTC Speed.txt";
    if(req.uiStandard == 1)
    {
      standard_name = "NEDC";
      str = "/home/madesheng/publish/wwwroot/Upload/File/NEDC Speed.txt";
    }
    else if(req.uiStandard == 2)
    {
      standard_name = "WLTC";
      str = "/home/madesheng/publish/wwwroot/Upload/File/WLTC Speed.txt";
    }
    else if(req.uiStandard == 3)
    {
      standard_name = "CLTC";
      str = "/home/madesheng/publish/wwwroot/Upload/File/CLTC Speed.txt";
    }
    else if(req.uiStandard == 100)
    {
      standard_name = "CruiseControl";
      rack_constant_speed = req.uiConstantSpeed;
    }
    if(req.uiStandard != 100)
    {
      int flag = Canbus::read_txt(str,6);
      if(flag != ROBOT_SUCCESS)
      {
        res.uiReturn = 0;
        return false;
      }
    }
    if(rackTestState == 2)
    {
      rackTestState = 1;
      res.uiReturn = 1;
      return true;
    }


    rackTestState = 1;
    nedc_speed_count = 0;
    rack_test_send_count = 0;
    set_speed_last = 0;
    rack_set_speed = 0;
    rack_actual_speed = 0;
    task_id = 0;
    task_num = 0;
    task_already_num = 0;
    task_time_stamp =0;
    task_time_stamp_ui = 0;
    over_speed_flag = false;
    over_speed_num = 0;
//TODO zhushidiao
//    rack_test_timer_ = CreateTimer(ros::Duration(duration_rack), &Canbus::RackTestOnTimer, this,false,false);
//    rack_test_timer_.start();
    //TODO

    //查询参数数据库
    RACK_SYS_PARA tRackSysPara;
    CanbusMySqlDataBase::QuerySysPara(req.uiCarId,&tRackSysPara);
    P =tRackSysPara.P;
    I = tRackSysPara.I;
    D = tRackSysPara.D;
    KP = tRackSysPara.KP;
    KI = tRackSysPara.KI;
    KD = tRackSysPara.KD;
    brk_factor = tRackSysPara.BrkFactor;
    brk_max = tRackSysPara.BrkMax;

    std::string datetime_now;
    datetime_now = get_datetime_mill();
    //插入测试任务信息表
    if(req.uiStandard == 100)
    {
      task_num = 1;
    }
    else
    {
      task_num = req.iTaskNum;
    }

    long result = CanbusMySqlDataBase::InsertTaskInfo(req.uiCarId,standard_name,
                                                      datetime_now,over_threshold_value,task_num);
    if(result <= 0)
    {
      res.uiReturn = 0;
      return true;
    }
    task_id = result;
    rack_reply_timer_ =
        CreateTimer(ros::Duration(duration_rack_reply), &Canbus::RackTestReplyOnTimer, this,false,false);
    rack_reply_timer_.start();
    rack_reply_detail_timer_ =
        CreateTimer(ros::Duration(duration_rack_reply_detail), &Canbus::RackTestReplyDetailOnTimer, this,false,false);
    rack_reply_detail_timer_.start();
    rackTestSendTimerState = true;
    res.uiReturn = 1;
  }
  catch (ros::Exception ex)
  {
      res.uiReturn = 0;
      LOG(ERROR)<< "controlservice RackTestReq failed "<< ex.what();
      return true;
  }
  return true;

}

void Canbus::RackTestOnTimer(const ros::TimerEvent &event)
{
  try
  {
    PID PidControll;
    gettimeofday(&now_time, NULL);
    float delta_time = 0.0;
    if(sizeof(last_time) != 0)
    {
      long elapsed = (now_time.tv_sec-last_time.tv_sec) * 1000000 + (now_time.tv_usec - last_time.tv_usec);
      delta_time = (float)elapsed / 1000000;
    }

    LOG(INFO)<< "delta time:"<<std::setprecision(10)<<delta_time<<" s";

    last_time = now_time;

    float set_speed = 0;
    float actual_speed = msg_RTK.Speed;
    float set_speed_1s = 0;
    if(!rackTestState)
    {
      if(actual_speed == 0)
      {
        rack_test_timer_.stop();
        rack_reply_timer_.stop();
        rack_reply_detail_timer_.stop();
        rackTestSendTimerState = false;
        rackTestReplyState = 0;
        task_id = 0;
        task_num = 0;
        task_already_num = 0;
        task_time_stamp =0;
        task_time_stamp_ui = 0;
        publish_rack_speed();
        return;
      }
      set_speed = 0;

    }
    else
    {
      if(nedc_speed_count == nedc_speed_number)
      {
        task_already_num ++;
        if(task_already_num < task_num)
        {
          nedc_speed_count = 0;
          rack_test_send_count = 0;
          set_speed_last = 0;
          task_time_stamp = 0;
          task_time_stamp_ui = 0;
        }
        else
        {
          rackTestState = false;
          rack_test_timer_.stop();
          rack_reply_timer_.stop();
          rack_reply_detail_timer_.stop();
          rackTestSendTimerState = false;
          rackTestReplyState = 0;
          task_id = 0;
          task_num = 0;
          task_time_stamp =0;
          publish_rack_speed();
          return;

        }

      }
      if(delta_time > 0 && delta_time < 1)
      {
       task_time_stamp += delta_time;
      }

      //根据时间差计算速度插值
      float set_speed1 = nedc_speeds[nedc_speed_count];
      float set_speed2 = 0;
      if(nedc_speed_count == nedc_speed_number - 1)
      {
        set_speed2 = nedc_speeds[nedc_speed_count];
      }
      else
      {
        set_speed2 = nedc_speeds[nedc_speed_count + 1];
      }

      if(set_speed1 == 0 && set_speed2 ==0)
      {
        set_speed = 0;
      }
      else
      {
        set_speed = set_speed_last + (set_speed2 - set_speed1) * delta_time;
      }
      if(set_speed < 0)
      {
        set_speed = 0;
      }

      float set_speed1_1s = 0;
      float set_speed2_1s = 0;
      if(nedc_speed_count > 0)
      {
        set_speed1_1s = nedc_speeds[nedc_speed_count-1];
      }
      if(nedc_speed_count-1 == nedc_speed_number - 2)
      {
        set_speed2_1s = nedc_speeds[nedc_speed_count-1];
      }
      else
      {
        set_speed2_1s = nedc_speeds[nedc_speed_count];
      }

      if(set_speed1_1s == 0 && set_speed2_1s ==0)
      {
        set_speed_1s = 0;
      }
      else
      {
        set_speed_1s = set_speed_last_1s + (set_speed2_1s - set_speed1_1s) * delta_time;
      }
      if(set_speed_1s < 0)
      {
        set_speed_1s = 0;
      }

    }
    LOG(INFO)<< "RackTestOnTimer Start FUZZY PID";

//    if(actual_speed - set_speed > 20 || actual_speed - set_speed < -20)
//    {
//      LOG(INFO)<< "actual_speed too more";
//      return;
//    }
    LOG(INFO)<< "nedc_speed_count:"<<nedc_speed_count;
//    string str = "/home/madesheng/FuzzyPid.txt";
//    int flag = Canbus::read_txt(str,7);
//    if(flag != ROBOT_SUCCESS)
//    {
//      LOG(INFO)<< "Get FuzzyPID.txt error";
//      return;
//    }

//    int brkFactor = 0;
//    int brkMax = 0;
//    brkFactor = fuzzy_pid[6];
//    brkMax = fuzzy_pid[7];

    float pi_out = 0.01;
    pi_out = PidControll.FuzzyPIDcontroller(set_speed,actual_speed,delta_time,
                                            P,I,D,
                                            KP,KI,KD);

    LOG(INFO)<< "P:"<<std::setprecision(10)<<P;
    LOG(INFO)<< "I:"<<std::setprecision(10)<<I;
    LOG(INFO)<< "D:"<<std::setprecision(10)<<D;

    LOG(INFO)<< "P_F:"<<std::setprecision(10)<<KP;
    LOG(INFO)<< "I_F:"<<std::setprecision(10)<<KI;
    LOG(INFO)<< "D_F:"<<std::setprecision(10)<<KD;

    LOG(INFO)<< "Set Speed:"<<std::setprecision(8)<<set_speed;
    LOG(INFO)<< "Actual Speed:"<<std::setprecision(8)<<actual_speed;
    LOG(INFO)<< "pi_out:"<<std::setprecision(8)<<pi_out;

    TPCANMsg canBrkPedalActCtrReqMsg;
    T_BRK_PEDAL_ACT_CTR_REQ requestBrk;
    TPCANMsg canAccPedalActCtrReqMsg;
    T_ACC_PEDAL_ACT_CTR_REQ requestAcc;

    if(set_speed == 0 && actual_speed < 4 || nedc_speed_count == nedc_speed_number - 1)
    {
      requestAcc.uiAccSpdCtrReq = 0;
      requestAcc.uiAccAccCtrReq = 0;
      requestAcc.uiAccPsngCtrReq = 0;
      requestAcc.uiAccDecCtrReq = 0;

      MessageCoder::CanMsgPackage((void*)&requestAcc, robot::common::AccPedalActCtrReq, &canAccPedalActCtrReqMsg);
      write_data(&canAccPedalActCtrReqMsg,true);

          requestBrk.uiBrkSpdCtrReq = 5;
          requestBrk.uiBrkAccCtrReq = 0;
          requestBrk.uiBrkCtrMdReq = 1;
          requestBrk.uiBrkDecCtrReq = 0;
          requestBrk.uiBrkFocCtrReq = 0;
//          requestBrk.uiBrkPsngCtrReq = 50 * 10;
          requestBrk.uiBrkPsngCtrReq = brk_max * 10;

          LOG(INFO)<< "Send Brk Psng:"<<requestBrk.uiBrkPsngCtrReq;

          MessageCoder::CanMsgPackage((void*)&requestBrk, robot::common::BrkPedalActCtrReq, &canBrkPedalActCtrReqMsg);
          write_data(&canBrkPedalActCtrReqMsg,true);
    }
    else
    {
      if(pi_out >= 0)
      {

        requestBrk.uiBrkSpdCtrReq = 0;
        requestBrk.uiBrkAccCtrReq = 0;
        requestBrk.uiBrkCtrMdReq = 0;
        requestBrk.uiBrkDecCtrReq = 0;
        requestBrk.uiBrkFocCtrReq = 0;
        requestBrk.uiBrkPsngCtrReq = 0;

        MessageCoder::CanMsgPackage((void*)&requestBrk, robot::common::BrkPedalActCtrReq, &canBrkPedalActCtrReqMsg);
        write_data(&canBrkPedalActCtrReqMsg,true);

         requestAcc.uiAccSpdCtrReq = 0;
         requestAcc.uiAccAccCtrReq = 0;
         requestAcc.uiAccPsngCtrReq = pi_out *100 * 10;
         requestAcc.uiAccDecCtrReq = 0;

         LOG(INFO)<< "Send Acc Psng:"<<requestAcc.uiAccPsngCtrReq;

         MessageCoder::CanMsgPackage((void*)&requestAcc, robot::common::AccPedalActCtrReq, &canAccPedalActCtrReqMsg);
         write_data(&canAccPedalActCtrReqMsg,true);

      }
      else
      {

        requestAcc.uiAccSpdCtrReq = 0;
        requestAcc.uiAccAccCtrReq = 0;
        requestAcc.uiAccPsngCtrReq = 0;
        requestAcc.uiAccDecCtrReq = 0;

        MessageCoder::CanMsgPackage((void*)&requestAcc, robot::common::AccPedalActCtrReq, &canAccPedalActCtrReqMsg);
        write_data(&canAccPedalActCtrReqMsg,true);

        requestBrk.uiBrkSpdCtrReq = 100;
        requestBrk.uiBrkAccCtrReq = 0;
        requestBrk.uiBrkCtrMdReq = 1;
        requestBrk.uiBrkDecCtrReq = 0;
        requestBrk.uiBrkFocCtrReq = 0;
        requestBrk.uiBrkPsngCtrReq = (brk_factor * (-pi_out * 30) / 30) * 10;
        if(requestBrk.uiBrkPsngCtrReq > brk_max * 10)
        {
          requestBrk.uiBrkPsngCtrReq = brk_max * 10;
          //requestBrk.uiBrkPsngCtrReq = 500;
        }

        LOG(INFO)<< "Send Brk Psng:"<<requestBrk.uiBrkPsngCtrReq;

        MessageCoder::CanMsgPackage((void*)&requestBrk, robot::common::BrkPedalActCtrReq, &canBrkPedalActCtrReqMsg);
        write_data(&canBrkPedalActCtrReqMsg,true);
      }
    }
    set_speed_last = set_speed;
    nedc_speed_count = rack_test_send_count;
    set_speed_last_1s = set_speed_1s;

    if(set_speed_1s - actual_speed > over_threshold_value || set_speed_1s - actual_speed < -over_threshold_value)
    {
      if(!over_speed_flag)
      {
        over_speed_flag = true;
        over_speed_num ++;
      }
    }
    else
    {
      over_speed_flag = false;
    }
    rack_set_speed = set_speed_1s;
    rack_actual_speed = actual_speed;
    //插入测试任务数据采集表
    int data_task_already_num = task_already_num + 1;
    long result = CanbusMySqlDataBase::InsertTaskData(task_id,set_speed,actual_speed,
                                                      requestAcc.uiAccPsngCtrReq/10,
                                                      requestBrk.uiBrkPsngCtrReq / 10,
                                                      task_time_stamp,over_speed_num,
                                                      data_task_already_num);

  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " RackTestOnTimer error. "<< ex.what();
  }
}

void Canbus::RackTestReplyOnTimer(const ros::TimerEvent &event)
{
  try
  {
    if(rackTestState == 2)
    {
      rackTestReplyState = 2;
      publish_rack_speed();
      return;
    }
    rack_test_send_count ++;
    double set_speed_over = 0;
    double set_speed_over_front = 0;
    double set_speed_over_back = 0;
    double delta_over_value_max = 0;
    double delta_over_value_min = 0;
    if(standard_name == "CruiseControl")  //定速巡航模式
    {
      delta_over_value_min = 0;
      delta_over_value_max = 0;
    }
    else  //规范文件模式
    {
      if(rack_test_send_count > 0)
      {
        set_speed_over = nedc_speeds[rack_test_send_count-1];
      }
      if(rack_test_send_count == nedc_speed_number)
      {
        set_speed_over_back = nedc_speeds[rack_test_send_count-1];
      }
      else
      {
        set_speed_over_back = nedc_speeds[rack_test_send_count];
      }
      if(rack_test_send_count -1 > 0)
      {
        set_speed_over_front = nedc_speeds[rack_test_send_count-2];
      }
      delta_over_value_max =
          set_speed_over >= set_speed_over_front ? set_speed_over : set_speed_over_front;
      delta_over_value_max =
          delta_over_value_max >= set_speed_over_back ? delta_over_value_max : set_speed_over_back;

      delta_over_value_min =
          set_speed_over <= set_speed_over_front ? set_speed_over : set_speed_over_front;
      delta_over_value_min =
          delta_over_value_min <= set_speed_over_back ? delta_over_value_min : set_speed_over_back;
    }


    double over_max = 0;
    double over_min = 0;

    over_max = over_threshold_value + delta_over_value_max;
    over_min = delta_over_value_min - over_threshold_value;
    if(over_min < 0)
    {
      over_min = 0;
    }

    if(rack_actual_speed > over_max || rack_actual_speed < over_min)
    {
      if(!over_speed_flag)
      {
        over_speed_flag = true;
        over_speed_num ++;
        LOG(INFO)<< "over speed actual speed:"<<rack_actual_speed;
        LOG(INFO)<< "over speed Index:"<<nedc_speed_count;
        LOG(INFO)<< "over speed second:"<<rack_test_send_count;
        LOG(INFO)<< "over speed set speed:"<<nedc_speeds[nedc_speed_count];
        LOG(INFO)<< "over speed set_speed_over_front:"<<set_speed_over_front;
        LOG(INFO)<< "over speed set_speed_over:"<<set_speed_over;
        LOG(INFO)<< "over speed set_speed_over_back:"<<set_speed_over_back;
        LOG(INFO)<< "over speed set max:"<<over_max;
        LOG(INFO)<< "over speed set min:"<<over_min;
      }
    }
    else
    {
      over_speed_flag = false;
    }
    if(rackTestSendTimerState)
    {
       rackTestReplyState = 1;
    }
    else
    {
      rackTestReplyState = 0;
    }

    // 发布消息，rackTestReplyState,actual_speed 1s回复前端一次消息
    publish_rack_speed();
 //   rack_test_send_count ++;
  }
  catch (ros::Exception ex)
  {
    LOG(ERROR)<<Name()<< " RackTestReplyOnTimer error. "<< ex.what();
  }

}

void Canbus::RackTestReplyDetailOnTimer(const ros::TimerEvent &event)
{
  if(rackTestState == 2)
  {
    return;
  }
  try
  {
    gettimeofday(&now_time_ui, NULL);
    float delta_time = 0.0;
    if(sizeof(last_time_ui) != 0)
    {
      long elapsed = (now_time_ui.tv_sec-last_time_ui.tv_sec) * 1000000 +
          (now_time_ui.tv_usec - last_time_ui.tv_usec);
      delta_time = (float)elapsed / 1000000;
    }

    last_time_ui = now_time_ui;
    if(rackTestSendTimerState)
    {
      if(delta_time > 0 && delta_time < 1)
      {
       task_time_stamp_ui +=delta_time;
      }
       rackTestReplyState = 1;
       //插入测试任务数据采集表界面显示
       int data_task_already_num = task_already_num + 1;
//       long result = CanbusMySqlDataBase::InsertTaskDataUI(task_id,rack_set_speed,rack_actual_speed,
//                                                           task_time_stamp_ui,over_speed_num,data_task_already_num);
       auto ThreadInsertTaskDataUI = [&](long TaskId,
           double SetSpeed,
           double ActualSpeed,
           double StandardSpeedTimeStamp,
           int OverSpeedNum,
           int StandardTaskCount) -> void {
           LOG(INFO)<< "Start ThreadInsertTaskDataUI";
           mtx_taskdataui.lock();

           CanbusMySqlDataBase::InsertTaskDataUI(TaskId,SetSpeed,ActualSpeed,
                                                           StandardSpeedTimeStamp,OverSpeedNum,
                                                           StandardTaskCount);

          mtx_taskdataui.unlock();
          LOG(INFO)<< "END ThreadInsertTaskDataUI";
          };


       std::thread thread_taskdataui (ThreadInsertTaskDataUI,
                                      task_id,rack_set_speed,rack_actual_speed,
                                      task_time_stamp_ui,over_speed_num,
                                      data_task_already_num);
       thread_taskdataui.join();
    }
    else
    {
      rackTestReplyState = 0;
    }
  }
  catch (ros::Exception ex)
  {
    LOG(ERROR)<<Name()<< " RackTestReplyDetailOnTimer error. "<< ex.what();
  }
}

//获取当前时间和毫秒
std::string Canbus::get_datetime_mill()
{
  try
  {
    auto time_now = std::chrono::system_clock::now();

    //通过不同精度获取相差的毫秒数
    uint64_t difference_millseconds =
        std::chrono::duration_cast<std::chrono::milliseconds>(time_now.time_since_epoch()).count()
        - std::chrono::duration_cast<std::chrono::seconds>(time_now.time_since_epoch()).count() * 1000;
    time_t t_time = std::chrono::system_clock::to_time_t(time_now);
    tm *time_date = localtime(&t_time);
    char strTime[25] = { 0 };
    sprintf(strTime, "[%d-%02d-%02d %02d:%02d:%02d.%03d]", time_date->tm_year + 1900,
            time_date->tm_mon + 1, time_date->tm_mday, time_date->tm_hour,
            time_date->tm_min, time_date->tm_sec, (int)difference_millseconds);
    /* 打印时间 */
  //  std::cout << strTime << std::endl;
    std::string str(strTime);
    string str2 = str.substr(1, str.length()-2);
    return str2;
  }
  catch (ros::Exception ex)
  {
    LOG(ERROR)<<Name()<< " Get Now DateTime And Millseconds error. "<< ex.what();
  }

}
//发布台架测试任务实际车速
void Canbus::publish_rack_speed()
{
  try
  {
    canbus::rackTestReplyMsg rackMsg;
    //TODO
    rackMsg.ActualSpeed = rack_actual_speed;
//    rackMsg.ActualSpeed = nedc_speeds[nedc_speed_count];
    rackMsg.RackTestReplyState = rackTestReplyState;
    rackMsg.SetSpeed = rack_set_speed;
    rackMsg.TaskCurrentTime = nedc_speed_count +(nedc_speed_number * task_already_num);
    if(standard_name == "CruiseControl")
    {
      rackMsg.TaskTotalTime = rackMsg.TaskCurrentTime;
    }
    else
    {
      rackMsg.TaskTotalTime = nedc_speed_number * task_num;
    }

    rackMsg.OverSpeedNum =over_speed_num;
    rackMsg.TaskId = task_id;
    rackMsg.TaskAlreadyNum = task_already_num + 1;
    canRackData_pub.publish(rackMsg);
  }
  catch (ros::Exception ex)
  {
    LOG(ERROR)<<Name()<< " publish rackTestReplyMsg error. "<< ex.what();
  }

}

void Canbus::publis_rack_detail_speed()
{
  try
  {
    canbus::rackTestReplyDetailMsg detailMsg;
    canRackDetailData_pub.publish(detailMsg);
  }
  catch (ros::Exception ex)
  {
    LOG(ERROR)<<Name()<< " publish rackTestReplyDetailMsg error. "<< ex.what();
  }
}

// 接收ＲＴＫ数据
void Canbus::RTKLocationCallback(const localization::localMsg::ConstPtr& msg)
{
  msg_RTK.AngleDeviated = msg->AngleDeviated;
  msg_RTK.Height = msg->Height;
  msg_RTK.Latitude = msg->Latitude;
  msg_RTK.Longitude = msg->Longitude;
  msg_RTK.Speed = msg->Speed;
  try
  {
    PID PidControll;
    gettimeofday(&now_time, NULL);
    float delta_time = 0.0;
    if(sizeof(last_time) != 0)
    {
      long elapsed = (now_time.tv_sec-last_time.tv_sec) * 1000000 + (now_time.tv_usec - last_time.tv_usec);
      delta_time = (float)elapsed / 1000000;
    }



    last_time = now_time;

    float set_speed = 0;
    float actual_speed = msg_RTK.Speed;
    float set_speed_1s = 0;
    if(rackTestState == 2)
    {
      return;
    }
    if(rackTestState == 0)
    {
      if(!rackTestSendTimerState)
      {
        return;
      }
      if(actual_speed == 0)
      {
//        rack_test_timer_.stop();
        rack_reply_timer_.stop();
        rack_reply_detail_timer_.stop();
        rackTestSendTimerState = false;
        rackTestReplyState = 0;
        task_id = 0;
        task_num = 0;
        task_already_num = 0;
        task_time_stamp =0;
        task_time_stamp_ui = 0;
        publish_rack_speed();
        return;
      }
      set_speed = 0;

    }
    else if(rackTestState == 1)
    {
      if(standard_name == "CruiseControl")  //定速巡航模式
      {
        set_speed = rack_constant_speed;
        set_speed_1s = rack_constant_speed;
        task_already_num = 0;
        nedc_speed_number = 0;
      }
      else  //规范文件模式
      {
        if(nedc_speed_count == nedc_speed_number)
        {
          task_already_num ++;
          if(task_already_num < task_num)
          {
            nedc_speed_count = 0;
            rack_test_send_count = 0;
            set_speed_last = 0;
            task_time_stamp = 0;
            task_time_stamp_ui = 0;
          }
          else
          {
            rackTestState = 0;
          //  rack_test_timer_.stop();
            rack_reply_timer_.stop();
            rack_reply_detail_timer_.stop();
            rackTestSendTimerState = false;
            rackTestReplyState = 0;
            task_id = 0;
            task_num = 0;
            task_time_stamp =0;
            publish_rack_speed();
            return;

          }

        }
        if(delta_time > 0 && delta_time < 1)
        {
         task_time_stamp += delta_time;
        }

        //根据时间差计算速度插值
        float set_speed1 = nedc_speeds[nedc_speed_count];
        float set_speed2 = 0;
        if(nedc_speed_count == nedc_speed_number - 1)
        {
          set_speed2 = nedc_speeds[nedc_speed_count];
        }
        else
        {
          set_speed2 = nedc_speeds[nedc_speed_count + 1];
        }

        if(set_speed1 == 0 && set_speed2 ==0)
        {
          set_speed = 0;
        }
        else
        {
          set_speed = set_speed_last + (set_speed2 - set_speed1) * delta_time;
        }
        if(set_speed < 0)
        {
          set_speed = 0;
        }

        float set_speed1_1s = 0;
        float set_speed2_1s = 0;
        if(nedc_speed_count > 0)
        {
          set_speed1_1s = nedc_speeds[nedc_speed_count-1];
        }
        if(nedc_speed_count-1 == nedc_speed_number - 2)
        {
          set_speed2_1s = nedc_speeds[nedc_speed_count-1];
        }
        else
        {
          set_speed2_1s = nedc_speeds[nedc_speed_count];
        }

        if(set_speed1_1s == 0 && set_speed2_1s ==0)
        {
          set_speed_1s = 0;
        }
        else
        {
          set_speed_1s = set_speed_last_1s + (set_speed2_1s - set_speed1_1s) * delta_time;
        }
        if(set_speed_1s < 0)
        {
          set_speed_1s = 0;
        }
      }


    }
    LOG(INFO)<< "delta time:"<<std::setprecision(10)<<delta_time<<" s";
    LOG(INFO)<< "RTKLocationCallback Start FUZZY PID";

//    if(actual_speed - set_speed > 20 || actual_speed - set_speed < -20)
//    {
//      LOG(INFO)<< "actual_speed too more";
//      return;
//    }

    LOG(INFO)<< "nedc_speed_count:"<<nedc_speed_count;

    float pi_out = 0.01;
    pi_out = PidControll.FuzzyPIDcontroller(set_speed,actual_speed,delta_time,
                                            P,I,D,
                                            KP,KI,KD);

    LOG(INFO)<< "P:"<<std::setprecision(10)<<P;
    LOG(INFO)<< "I:"<<std::setprecision(10)<<I;
    LOG(INFO)<< "D:"<<std::setprecision(10)<<D;

    LOG(INFO)<< "P_F:"<<std::setprecision(10)<<KP;
    LOG(INFO)<< "I_F:"<<std::setprecision(10)<<KI;
    LOG(INFO)<< "D_F:"<<std::setprecision(10)<<KD;

    LOG(INFO)<< "Set Speed:"<<std::setprecision(8)<<set_speed;
    LOG(INFO)<< "Actual Speed:"<<std::setprecision(8)<<actual_speed;
    LOG(INFO)<< "pi_out:"<<std::setprecision(8)<<pi_out;

    TPCANMsg canBrkPedalActCtrReqMsg;
    T_BRK_PEDAL_ACT_CTR_REQ requestBrk;
    TPCANMsg canAccPedalActCtrReqMsg;
    T_ACC_PEDAL_ACT_CTR_REQ requestAcc;

    if(set_speed == 0 && actual_speed < 4 || nedc_speed_count == nedc_speed_number - 1)
    {
      requestAcc.uiAccSpdCtrReq = 0;
      requestAcc.uiAccAccCtrReq = 0;
      requestAcc.uiAccPsngCtrReq = 0;
      requestAcc.uiAccDecCtrReq = 0;

      MessageCoder::CanMsgPackage((void*)&requestAcc, robot::common::AccPedalActCtrReq, &canAccPedalActCtrReqMsg);
      write_data(&canAccPedalActCtrReqMsg,true);

          requestBrk.uiBrkSpdCtrReq = 5;
          requestBrk.uiBrkAccCtrReq = 0;
          requestBrk.uiBrkCtrMdReq = 1;
          requestBrk.uiBrkDecCtrReq = 0;
          requestBrk.uiBrkFocCtrReq = 0;
//          requestBrk.uiBrkPsngCtrReq = 50 * 10;
          requestBrk.uiBrkPsngCtrReq = brk_max * 10;

          LOG(INFO)<< "Send Brk Psng:"<<requestBrk.uiBrkPsngCtrReq;

          MessageCoder::CanMsgPackage((void*)&requestBrk, robot::common::BrkPedalActCtrReq, &canBrkPedalActCtrReqMsg);
          write_data(&canBrkPedalActCtrReqMsg,true);
    }
    else
    {
      if(pi_out >= 0)
      {

        requestBrk.uiBrkSpdCtrReq = 0;
        requestBrk.uiBrkAccCtrReq = 0;
        requestBrk.uiBrkCtrMdReq = 0;
        requestBrk.uiBrkDecCtrReq = 0;
        requestBrk.uiBrkFocCtrReq = 0;
        requestBrk.uiBrkPsngCtrReq = 0;

        MessageCoder::CanMsgPackage((void*)&requestBrk, robot::common::BrkPedalActCtrReq, &canBrkPedalActCtrReqMsg);
        write_data(&canBrkPedalActCtrReqMsg,true);

         requestAcc.uiAccSpdCtrReq = 0;
         requestAcc.uiAccAccCtrReq = 0;
         requestAcc.uiAccPsngCtrReq = pi_out *100 * 10;
         requestAcc.uiAccDecCtrReq = 0;

         LOG(INFO)<< "Send Acc Psng:"<<requestAcc.uiAccPsngCtrReq;

         MessageCoder::CanMsgPackage((void*)&requestAcc, robot::common::AccPedalActCtrReq, &canAccPedalActCtrReqMsg);
         write_data(&canAccPedalActCtrReqMsg,true);

      }
      else
      {

        requestAcc.uiAccSpdCtrReq = 0;
        requestAcc.uiAccAccCtrReq = 0;
        requestAcc.uiAccPsngCtrReq = 0;
        requestAcc.uiAccDecCtrReq = 0;

        MessageCoder::CanMsgPackage((void*)&requestAcc, robot::common::AccPedalActCtrReq, &canAccPedalActCtrReqMsg);
        write_data(&canAccPedalActCtrReqMsg,true);

        requestBrk.uiBrkSpdCtrReq = 100;
        requestBrk.uiBrkAccCtrReq = 0;
        requestBrk.uiBrkCtrMdReq = 1;
        requestBrk.uiBrkDecCtrReq = 0;
        requestBrk.uiBrkFocCtrReq = 0;
        requestBrk.uiBrkPsngCtrReq = (brk_factor * (-pi_out * 30) / 30) * 10;
        if(requestBrk.uiBrkPsngCtrReq > brk_max * 10)
        {
          requestBrk.uiBrkPsngCtrReq = brk_max * 10;
          //requestBrk.uiBrkPsngCtrReq = 500;
        }

        LOG(INFO)<< "Send Brk Psng:"<<requestBrk.uiBrkPsngCtrReq;

        MessageCoder::CanMsgPackage((void*)&requestBrk, robot::common::BrkPedalActCtrReq, &canBrkPedalActCtrReqMsg);
        write_data(&canBrkPedalActCtrReqMsg,true);
      }
    }
    set_speed_last = set_speed;
    nedc_speed_count = rack_test_send_count;

    rack_set_speed = set_speed_1s;
    rack_actual_speed = actual_speed;
    set_speed_last_1s = set_speed_1s;


    //插入测试任务数据采集表
    int data_task_already_num = task_already_num + 1;
//    CanbusMySqlDataBase::InsertTaskData(task_id,set_speed,actual_speed,
//                                                      requestAcc.uiAccPsngCtrReq/10,
//                                                      requestBrk.uiBrkPsngCtrReq / 10,
//                                                      task_time_stamp,over_speed_num,
//                                                      data_task_already_num);
    auto ThreadInsertTaskData = [&](long TaskId,
        double SetSpeed,
        double ActualSpeed,
        long AccOrderPsng,
        long BrkOrderPsng,
        double StandardSpeedTimeStamp,
        int OverSpeedNum,
        int StandardTaskCount) -> void {
        LOG(INFO)<< "Start ThreadInsertTaskData";
        mtx_taskdata.lock();

        CanbusMySqlDataBase::InsertTaskData(TaskId,SetSpeed,ActualSpeed,
                                                        AccOrderPsng,
                                                        BrkOrderPsng,
                                                        StandardSpeedTimeStamp,OverSpeedNum,
                                                        StandardTaskCount);

         mtx_taskdata.unlock();
         LOG(INFO)<< "End ThreadInsertTaskData";
      };

    std::thread thread_taskdata(ThreadInsertTaskData,
                                task_id,set_speed,actual_speed,
                                requestAcc.uiAccPsngCtrReq/10,
                                requestBrk.uiBrkPsngCtrReq / 10,
                                task_time_stamp,over_speed_num,
                                data_task_already_num);
    thread_taskdata.join();

  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " RTKLocationCallback error. "<< ex.what();
  }

}


/**
  * @brief  Linux socket can通信初始化
  * @param  fd：CAN通信文件描述符
  * @retval 成功返回0，失败返回-1
  */
int Canbus::can_init_for_linux(int *fd)
{
    //0、设置can通信波特率
    system("sudo ip link set can0 down");
    //波特率设置为500Kbps
    system("sudo ip link set can0 type can bitrate 500000 triple-sampling on");
    system("sudo ip link set can0 up");

    int ret = 0;
    //1、创建socket can套接字
    *fd = socket(AF_CAN, SOCK_RAW, CAN_RAW);
    if(*fd == -1)
    {
        LOG(ERROR) << Name() <<" create socket can failed";
        return -1;
    }
cout << "*fd = " << *fd <<endl;
    //2、套接子绑定到can0端口
    struct sockaddr_can addr;
    struct ifreq ifr;
    strcpy(ifr.ifr_name, "can0");
    ioctl(*fd, SIOCGIFINDEX, &ifr);
    memset(&addr, 0, sizeof(addr));
    addr.can_family = AF_CAN;
    addr.can_ifindex = ifr.ifr_ifindex;
    ret = bind(*fd, (struct sockaddr *)&addr, sizeof(addr));
    if(ret < 0)
    {
        LOG(ERROR) << Name() <<" socket can bind can0 error";
        return -1;
    }

    //4、设置CAN过滤规则
    #if 0
    struct can_filter recv_filter;
    recv_filter.can_id = 0x201;
    recv_filter.can_mask = CAN_SFF_MASK;
    setsockopt(*can_fd, SOL_CAN_RAW, CAN_RAW_FILTER, &recv_filter, sizeof(recv_filter));
    #endif

    //5、设置read、write为非堵塞方式
    int flags;
    flags = fcntl(*fd, F_GETFL);
    flags |= O_NONBLOCK;
    fcntl(*fd, F_SETFL, flags);

    return 0;
}

/**
  * @brief  关闭CAN通信文件描述符
  * @param  fd：CAN通信文件描述符
  * @retval void
  */
void Canbus::can_close(int fd)
{
    close(fd);
}

/**
  * @brief  CAN发送数据
  * @param  fd：CAN通信文件描述符
  * @param  can_id：can设备id
  * @param  data：需要发送的数据（一次最大8字节）
  * @param  data_len：需要发送数据长度(Byte)
  * @retval 成功返回0，失败返回-1
  */
int Canbus::can_write_data(int fd, int can_id,char *data,int data_len)
{
    int ret = 0;
    CanRxMsg msg;
    struct can_frame send_data;
    if(fd == -1)
    {
        LOG(ERROR) << Name() <<" socket can write data failed!";
        return -1;
    }
    send_data.can_dlc = data_len;
    send_data.can_id = can_id;
    memcpy(&send_data.data[0], data, data_len);
    ret = write(fd, &send_data, sizeof(struct can_frame));
    if(ret == -1)
    {
        LOG(ERROR) << Name() <<" socket can write data failed!";
        return -1;
    }else{
       /* printf("SEND ID=%03X, DLC=%d, data=%02X %02X %02X %02X %02X %02X %02X %02X \n",  \
        send_data.can_id, send_data.can_dlc,  \
        send_data.data[0],\
        send_data.data[1],\
        send_data.data[2],\
        send_data.data[3],\
        send_data.data[4],\
        send_data.data[5],\
        send_data.data[6],\
        send_data.data[7] );*/
    }

    return 0;
}

/**
  * @brief  CAN发送数据
  * @param  fd：CAN通信文件描述符
  * @param  can_id：can设备id
  * @param  data：存储CAN接收的8字节数据
  * @retval 成功返回0，失败返回-1
  */
int Canbus::can_read_data(int fd, int *can_id, char *data)
{
    if(fd == -1)
    {
        LOG(ERROR) << Name() <<" socket can read data failed!";
        return -1;
    }
    int ret = 0;
    struct can_frame recv_data;
   // recv_data.can_id = *can_id;
    recv_data.can_dlc = 8;

    ret = read(fd, &recv_data,sizeof(struct can_frame));
    if(ret == -1)
    {
      //  LOG(ERROR) << Name() <<" socket can read data failed!";
        return -1;
    }else{
        memcpy(data, &recv_data.data[0], recv_data.can_dlc);
        *can_id = recv_data.can_id;
    /*printf("RECV ID=%03X, DLC=%d, data=%02X %02X %02X %02X %02X %02X %02X %02X \n",  \
      recv_data.can_id, recv_data.can_dlc,  \
      recv_data.data[0],\
      recv_data.data[1],\
      recv_data.data[2],\
      recv_data.data[3],\
      recv_data.data[4],\
      recv_data.data[5],\
      recv_data.data[6],\
      recv_data.data[7] );*/
     // LOG(INFO) << Name() <<" socket can read data success!";
    }

    return 0;
}


}  // namespace canbus
}  // namespace robot
