/******************************************************************************
 * Copyright 2020 The siasun Transport Authors. All Rights Reserved.
 * Author: limengqi
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/

#ifndef CAN_DATA_TYPE_H_
#define CAN_DATA_TYPE_H_
#include <string>
//version 4.5.1.X
namespace robot {
namespace canbusSpace {

/******************************ICU-RCU控制命令******************************/


/*******************上位机报警******************/
typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int uiICUError:1;		//上位机报警
  unsigned int :7;
}T_CONTROL_CMD_200;

typedef struct
{
  unsigned int uiICUError;	//上位机报警

}T_ERROR_STATE;

#define ICU_ALARM_DISMISSED   0  	//报警解除
#define ICU_ALARM   1 				//上位机报警


/*******************急停控制******************/
typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int uiEstpEnReq:1;		//机器人Estop使能
  unsigned int :1;
  unsigned int uiPmpEnReq:1;		//制动急停气泵使能
  unsigned int :1;
  unsigned int uiPauseEnReq:1;	//系统软急停使能
  unsigned int :3;
}T_CONTROL_CMD_201;

typedef struct
{
  unsigned int uiEstpEnReq;	//机器人Estop使能
  unsigned int uiPmpEnReq;	//制动急停气泵使能
  unsigned int uiPauseEnReq;	//系统软急停使能
}T_ESTP_CTRL;

#define	ESTP_EN_REQ_DISABLE	0 			//机器人Estop禁能
#define	ESTP_EN_REQ_ENABLE 	1 			//机器人Estop使能

#define	PMP_EN_REQ_DISABLE	0			//制动急停气泵禁能
#define	PMP_EN_REQ_ENABLE	1			//制动急停气泵使能

#define	PAUSE_EN_REQ_DISABLE	0		//系统软急停禁能
#define	PAUSE_EN_REQ_ENABLE		1		//系统软急停使	能

/***********************机器人工作模式配置*******************/

typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int uiRbtAplySenro:1; //机器人应用场景 0道路 1台架
  unsigned int :1;
  unsigned int uiOnlineOffline:1; //机器人联/脱机模式 0联机 1脱机
  unsigned int :5;

}T_CONTROL_CMD_7F0;
typedef struct
{
  unsigned int uiOnlineOffline;		//机器人联/脱机工作模式
  unsigned int uiRbtAplySenro;		//机器人应用场景
}T_WORKING_MODE_CONFING;


#define CONFIG_TYPE_NORMAL		0	//普通模式
#define CONFIG_TYPE_TAIJIA		1	//台架模式

#define VEH_DATA_TYPE_VIHICLE_DEPEND	0	//依靠车辆数据 联机
#define VEH_DATA_TYPE_STAND_ALONE		1	//依靠ICU命令 脱机

/*******************系统控制模式****************/

typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int uiCtrMdReq:3; //实际系统控制模式请求
  unsigned int :5;

}T_CONTROL_CMD_7F1;
typedef struct
{
  unsigned int uiCtrMdReq;		//实际系统控制模式请求
}T_CONTROL_MODE;

#define	CTR_MD_REQ_HAND_DRIVE		0	//控制模式请求-人工驾驶模式
#define	CTR_MD_REQ_LEARN			1	//控制模式请求-学习模式
#define	CTR_MD_REQ_AUTO_DRIVE		2	//控制模式请求-自动驾驶模式
#define	CTR_MD_REQ_ROBOT_CHECK		4	//控制模式请求-机器人自检模式


/***********************控制器上电命令**********************/
typedef struct
{
  unsigned int :24;
  unsigned int uiZSftArmCtrPowReq:2;	//换挡臂Z电机控制器上电
  unsigned int :6;
  unsigned int uiYSftCtrPowReq:2;	//换挡臂Y电机控制器上电
  unsigned int :2;
  unsigned int uiDriveFanReq:2;   //驱动箱风扇使用请求
  unsigned int :2;
  unsigned int uiStrCtrPowReq:2;		//转向电机控制器上电
  unsigned int :2;
  unsigned int uiXSftCtrPowReq:2;	//换挡臂X电机控制器上电
  unsigned int :2;
  unsigned int uiBrkCtrPowReq:2;		//制动电机控制器上电
  unsigned int :2;
  unsigned int uiClchCtrPowReq:2;		//离合电机控制器上电
  unsigned int :2;
  unsigned int uiRCUPowReq:2;			//RCU 上电
  unsigned int :2;
  unsigned int uiAccCtrPowReq:2;		//油门电机控制器上电
  unsigned int :2;
}T_CONTROL_CMD_7F2;

typedef struct
{
  unsigned int uiRCUPowReq;			//RCU上电
  unsigned int uiAccCtrPowReq;		//油门电机控制器上电
  unsigned int uiBrkCtrPowReq;		//制动电机控制器上电
  unsigned int uiClchCtrPowReq;		//离合电机控制器上电
  unsigned int uiStrCtrPowReq;		//转向电机控制器上电
  unsigned int uiXSftCtrPowReq;	//换挡臂X电机控制器上电
  unsigned int uiYSftCtrPowReq;	//换挡臂Y电机控制器上电
  unsigned int uiDriveFanReq;     //驱动箱风扇使用请求
  unsigned int uiZSftArmCtrPowReq;  //换挡臂Z电机控制器上电
}T_POWER_ON_REQUEST;

#define	POW_REQ_OFF	0		//下电
#define POW_REQ_ON	1		//上电
#define POW_REQ_UNDEFINE	2    //无效命令

#define DRIVE_FAN_REQ_OFF  1  //不使用
#define DRIVE_FAN_REQ_ON   0  //使用

/**************************执行机构限值设置*********************/
typedef struct
{
  unsigned int uiSftType:2;  			//档位类型
  unsigned int :11;
  int iStrMaxLimt:19;					//转向极限值
  unsigned int :6;
  unsigned int uiBrkMaxLimt:10;		//制动行程最大限值
  unsigned int :6;
  unsigned int uiAccMaxLimt:10;		//油门行程最大限值
}T_CONTROL_CMD_7F3;

typedef struct
{
  unsigned int uiAccMaxLimt;		//油门行程最大限值
  unsigned int uiBrkMaxLimt;		//制动行程最大限值
  int iStrMaxLimt;				//转向极限值
  unsigned int uiSftType;			//档位类型
}T_ACT_CTR_LIMT;


/************************力矩保护设置************************/
typedef struct
{
  unsigned int :32;
  unsigned int :15;
  unsigned int uiStrTorqSwitch:1;		//转向力矩限制开关
  unsigned int :7;
  unsigned int uiStrTorqLimit:9;		//转向力矩限值
}T_CONTROL_CMD_7F4;

typedef struct
{
  unsigned int uiStrTorqLimit;		//转向力矩限值
  unsigned int uiStrTorqSwitch;		//转向力矩限制开关
}T_TORQUE_PROTECT_SETUP;



/*************执行机构使能请求*************/
typedef struct
{
  unsigned int :32;
  unsigned int :16;
  unsigned int uiYSftMotEnReq:1;	//换挡臂Y电机使能
  unsigned int :1;
  unsigned int uiClchMotEnReq:1;	//离合电机使能
  unsigned int :1;
  unsigned int uiAccClchCtrReq:2;	//油门离合器控	制
  unsigned int :1;
  unsigned int uiZSftMotEnReq:1;  //换挡臂 Z 电机使能
  unsigned int uiAccMotEnReq:1;	//油门电机使能
  unsigned int :1;
  unsigned int uiBrkMotEnReq:1;	//制动电机使能
  unsigned int :1;
  unsigned int uiStrMotEnReq:1;	//转向电机使能
  unsigned int :1;
  unsigned int uiXSftMotEnReq:1;	//换挡臂X电机使能
  unsigned int :1;
}T_CONTROL_CMD_7F5;

typedef struct
{
  unsigned int uiAccMotEnReq;		//油门电机使能
  unsigned int uiBrkMotEnReq;		//制动电机使能
  unsigned int uiStrMotEnReq;		//转向电机使能
  unsigned int uiXSftMotEnReq;	//换挡臂X电机使能
  unsigned int uiYSftMotEnReq;	//换挡臂Y电机使能
  unsigned int uiClchMotEnReq;	//离合电机使能
  unsigned int uiAccClchCtrReq;	//油门离合器控	制
  unsigned int uiZSftMotEnReq;  //换挡臂 Z 电机使能
}T_ACTUATOR_ENABLE_REQUEST;

#define	MOT_EN_REQ_DISABLE		0		//电机禁能
#define	MOT_EN_REQ_ENABLE		1		//电机使能

#define	ACC_CLCH_CTR_REQ_JOINT		0	//油门离合控制器闭合
#define	ACC_CLCH_CTR_REQ_RELEASE	1	//油门离合控制器释放




#define	VEH_SW_REQ_LOCK				0	//关闭LOCK
#define	VEH_SW_REQ_ACC				1	//上电ACC
#define	VEH_SW_REQ_ON				2	//电启动ON
#define	VEH_SW_REQ_START			3	//开启START

/******************** 设备管理**********************/
typedef struct
{
  unsigned int :8;
  unsigned int uiFltClr:1;		//故障清除
  unsigned int :7;
  unsigned int uiYSftMotInt:1;	//档位Y电机初始化
  unsigned int uiZSftMotInt:1;	//档位Z电机初始化
  unsigned int :6;
  unsigned int uiXSftMotInt:1;	//档位X电机初始化
  unsigned int :7;
  unsigned int uiStrMotInt:1;		//转向电机初始化
  unsigned int :7;
  unsigned int uiClchMotInt:1;	//离合电机初始化
  unsigned int :7;
  unsigned int uiBrkMotInt:1;		//制动电机初始化
  unsigned int :7;
  unsigned int uiAccMotInt:1;		//油门电机初始化
  unsigned int :7;
}T_CONTROL_CMD_7F6;

typedef struct
{
  unsigned int uiAccMotInt;		//油门电机初始化
  unsigned int uiBrkMotInt;		//制动电机初始化
  unsigned int uiClchMotInt;		//离合电机初始化
  unsigned int uiStrMotInt;		//转向电机初始化
  unsigned int uiXSftMotInt;		//档位X电机初始化
  unsigned int uiYSftMotInt;		//档位Y电机初始化
  unsigned int uiFltClr;			//故障清除
  unsigned int uiZSftMotInt;	//档位Z电机初始化
}T_RBT_MANAGER;

#define	MOT_INT_ZERO		1		//电机初始化-标零
#define	FLT_CLR_RESET		1		//故障消除-复位



/******************学习模式请求确认****************/
typedef struct
{
  unsigned int :16;
  unsigned int uiSftAutLrnFnsConf:1;	//档位自动学习完成确认
  unsigned int :7;
  unsigned int uiClrSftTbl:1;			//清空挡位学习表
  unsigned int :1;
  unsigned int uiStpAutLrn:1;			//终止自动学习
  unsigned int :5;
  unsigned int uiClrAccTbl:1;			//清空油门学习表
  unsigned int :1;
  unsigned int uiClrBrkTbl:1;			//清空制动学习表
  unsigned int :1;
  unsigned int uiClrClchTbl:1;		//清空离合学习表
  unsigned int :1;
  unsigned int uiClrStrTbl:1;			//清空转向学习表
  unsigned int :1;
  unsigned int uiSftPsngLrnReq:5;		//挡位位置学习请求确认
  unsigned int :3;
  unsigned int uiPsngLrnReq:3;		//位置学习请求确认
  unsigned int :5;
  unsigned int uiSysLrnReq:4;			//系统学习请求
  unsigned int :4;
}T_CONTROL_CMD_7F7;

typedef struct
{
  unsigned int uiSysLrnReq;		//系统学习请求
  unsigned int uiPsngLrnReq;		//位置学习请求确认
  unsigned int uiSftPsngLrnReq;	//挡位位置学习请求确认
  unsigned int uiClrAccTbl;		//清空油门学习表
  unsigned int uiClrBrkTbl;		//清空制动学习表
  unsigned int uiClrClchTbl;		//清空离合学习表
  unsigned int uiClrStrTbl;		//清空转向学习表
  unsigned int uiClrSftTbl;		//清空挡位学习表
  unsigned int uiStpAutLrn;		//终止自动学习
  unsigned int uiSftAutLrnFnsConf;//档位自动学习完成确认
}T_LEARN_MODE_REQUEST;

#define	SYS_LRN_REQ_ACC			1	//油门学习
#define	SYS_LRN_REQ_BRK			2	//制动踏板学习
#define	SYS_LRN_REQ_CLCH		3	//离合踏板学习
#define	SYS_LRN_REQ_STR			4	//转向学习
#define	SYS_LRN_REQ_AUTO_SFT	5	//自动档学习
#define	SYS_LRN_REQ_HAND_SFT	6	//手动档学习
#define	SYS_LRN_REQ_ACC_AUTO			7	//油门自动学习(脱机)
#define	SYS_LRN_REQ_BRK_AUTO			8	//制动踏板自动学习(脱机)
#define	SYS_LRN_REQ_STR_AUTO			9	//转向自动学习(脱机)

#define	PSNG_LRN_REQ_ZERO		1	//位置学习-零位学习
#define	PSNG_LRN_REQ_MAX		2	//位置学习-最大值学习
#define	PSNG_LRN_REQ_L_MAX		3	//位置学习-左极限学习
#define	PSNG_LRN_REQ_R_MAX		4	//位置学习-右极限学习

#define	SFT_PSNG_LRN_REQ_BETWEEN		0	//挡位位置学习请求-???????????
#define	SFT_PSNG_LRN_REQ_PARK			1	//挡位位置学习请求-停车档（自动档）
#define	SFT_PSNG_LRN_REQ_REVERSE		2	//挡位位置学习请求-倒档
#define	SFT_PSNG_LRN_REQ_NEUTRAL		3	//挡位位置学习请求-空档
#define	SFT_PSNG_LRN_REQ_FORWARD_1		4	//挡位位置学习请求-前进档1(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_2		5	//挡位位置学习请求-前进档2(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_3		6	//挡位位置学习请求-前进档3(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_4		7	//挡位位置学习请求-前进档4(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_5		8	//挡位位置学习请求-前进档5(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_6		9	//挡位位置学习请求-前进档6(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_7		10	//挡位位置学习请求-前进档7(手动档)
#define	SFT_PSNG_LRN_REQ_FORWARD_8		11	//挡位位置学习请求-前进档8(手动档)
#define	SFT_PSNG_LRN_REQ_DRIVE			12	//挡位位置学习请求-前进档（自动档）
#define	SFT_PSNG_LRN_REQ_ADD			13	//挡位位置学习请求-加
#define	SFT_PSNG_LRN_REQ_SUB			14	//挡位位置学习请求-减
#define	SFT_PSNG_LRN_REQ_M				15	//挡位位置学习请求-未定义
#define	SFT_PSNG_LRN_REQ_SPORT			16	//挡位位置学习请求-运动档（自动档）

#define	TBL_CLEAR			1	//清空

#define	STP_AUT_LRN_STOP	1	//终止

#define SFT_AUT_LRN_FNS_CONF_DOWN	1	//学习完成

/*********************上位机监控****************/
typedef struct
{
  unsigned int :32;
  unsigned int :16;
  unsigned int uiICUClsReq:1;		//上位机关闭请求
  unsigned int :7;
  unsigned int uiICUAlive:4;		//上位机心跳
  unsigned int :4;
}T_CONTROL_CMD_7F8;

typedef struct
{
  unsigned int uiICUAlive;		//上位机心跳
  unsigned int uiICUClsReq;		//上位机关闭请求
}T_ICU_MONITOR;

#define	ICU_CLS_REQ_CLOSE	1		//正常关闭请求

/*************制动踏板执行机构控制命令*************/
typedef struct
{
  unsigned int :8;
  unsigned int uiBrkDecCtrReq:8;		//制动机构减速度控制请求
  unsigned int uiBrkAccCtrReq:8;		//制动机构加速度控制请求
  unsigned int :7;
  unsigned int uiBrkFocCtrReqL:1;		//制动踏板力控制请求
  unsigned int uiBrkFocCtrReqH:8;		//制动踏板力控制请求
  unsigned int uiBrkSpdCtrReq:8;		//制动踏板速度控制请求
  unsigned int uiBrkCtrMdReq:2;		//制动踏板控制	模式请求
  unsigned int :4;
  unsigned int uiBrkPsngCtrReq:10;	//制动踏板位置控制请求
}T_CONTROL_CMD_5F0;

typedef struct
{
  unsigned int uiBrkFocCtrReqL:1;		//制动踏板力控制请求
  unsigned int uiBrkFocCtrReqH:8;		//制动踏板力控制请求
  unsigned int :23;
}T_CONTROL_CMD_5F0_SRC;

typedef struct
{
  unsigned int uiBrkFocCtrReq:9;	//制动踏板力控制请求
  unsigned int :23;
}T_CONTROL_CMD_5F0_DEST;

typedef union
{
  T_CONTROL_CMD_5F0_SRC tCtrCmd5F0Src;
  T_CONTROL_CMD_5F0_DEST tCtrCmd5F0Dest;
}T_CONTROL_CMD_5F0_UNION;

typedef struct
{
  unsigned int uiBrkPsngCtrReq;	//制动踏板位置控制请求
  unsigned int uiBrkCtrMdReq;		//制动踏板控制	模式请求
  unsigned int uiBrkSpdCtrReq;	//制动踏板速度控制请求
  unsigned int uiBrkFocCtrReq;	//制动踏板力控制请求
  unsigned int uiBrkAccCtrReq;	//制动机构加速度控制请求
  unsigned int uiBrkDecCtrReq;	//制动机构减速度控制请求
}T_BRK_PEDAL_ACT_CTR_REQ;

/********************制动机构时间控制设置****************/
typedef struct
{
  unsigned int uiBrkCtrWay:1;	//控制方式
  unsigned int :10;
  unsigned int uiBrkTm:13;	//控制时间
  unsigned int :6;
  unsigned int uiBscCtrL:2;	//基准值低字节
  unsigned int uiBscCtrH:8;	//基准值高字节
  unsigned int :2;
  int iCtrSpd:22;				//增长速率
}T_CONTROL_CMD_5F1;

typedef struct
{
  unsigned int uiBscCtrL:2;	//基准值低字节
  unsigned int uiBscCtrH:8;	//基准值高字节
  unsigned int :22;
}T_CONTROL_CMD_5F1_SRC;

typedef struct
{
  unsigned int uiBscCtr:10;	//基准值
  unsigned int :22;
}T_CONTROL_CMD_5F1_DEST;

typedef union
{
  T_CONTROL_CMD_5F1_SRC tCtrCmd5F1Src;
  T_CONTROL_CMD_5F1_DEST tCtrCmd5F1Dest;
}T_CONTROL_CMD_5F1_UNION;

typedef struct
{
  int iCtrSpd;			//增长速率
  unsigned int uiBscCtr;	//基准值
  unsigned int uiBrkTm;	//控制时间
  unsigned int uiBrkCtrWay;	//控制方式
}T_BRK_FORCE_CTR_SETUP;

#define BRK_CTR_WAY_PSNG		0	//制动控制方式-位置
#define BRK_CTR_WAY_STRENGTH	1	//制动控制方式-力

/*************油门踏板执行机构控制命令*************/
typedef struct
{
  unsigned int :24;
  unsigned int uiAccDecCtrReq:8;
  unsigned int uiAccAccCtrReq:8;
  unsigned int uiAccSpdCtrReq:8;
  unsigned int :6;
  unsigned int uiAccPsngCtrReq:10;
}T_CONTROL_CMD_5F2;

typedef struct
{
  unsigned int uiAccPsngCtrReq;	//油门踏板位置控制请求
  unsigned int uiAccSpdCtrReq;	//油门踏板速度控制请求
  unsigned int uiAccAccCtrReq;	//油门机构加速度控制请求
  unsigned int uiAccDecCtrReq;	//油门机构减速度控制请求
}T_ACC_PEDAL_ACT_CTR_REQ;

/*************油门机构位置时间控制*************/
typedef struct
{
  unsigned int :11;
  unsigned int uiAccPT:13;	//控制时间
  unsigned int :5;
  unsigned int uiAccPstBscL:3;//油门位置基准值的低位
  unsigned int uiAccPstBscH:8;//油门位置基准值的高位
  unsigned int :6;
  int iAccSpdP:18;			//油门速度
}T_CONTROL_CMD_5F3;

typedef struct
{
  unsigned int uiAccPstBscL:3;//油门位置基准值的低位
  unsigned int uiAccPstBscH:8;//油门位置基准值的高位
  unsigned int :21;
}T_CONTROL_CMD_5F3_SRC;

typedef struct
{
  unsigned int uiAccPstBsc:11;//油门位置基准值
  unsigned int :21;
}T_CONTROL_CMD_5F3_DEST;

typedef union
{
  T_CONTROL_CMD_5F3_SRC tCtrCmd5F3Src;
  T_CONTROL_CMD_5F3_DEST tCtrCmd5F3Dest;
}T_CONTROL_CMD_5F3_UNION;

typedef struct
{
  int iAccSpdP;				//油门速度
  unsigned int uiAccPstBsc;	//油门位置基准值
  unsigned int uiAccPT;		//控制时间
}T_ACC_POSITION_CTR_SETUP;

/*************转向执行机构控制命令*************/
typedef struct
{
  unsigned int :2;
  int iStrTrqCtrReq:14;			//方向盘转矩控制请求
  unsigned int :5;
  int iStrSpdCtrReqL:11;			//方向盘转速控制请求
  int iStrSpdCtrReqH:8;			//方向盘转速控制请求
  unsigned int uiStrCtrMdReq:2;	//方向盘控制模式请求
  unsigned int :3;
  int iStrAngCtrReq:19;			//方向盘转角控制请求
}T_CONTROL_CMD_5F4;

typedef struct
{
  int iStrSpdCtrReqL:11;			//方向盘转速控制请求
  int iStrSpdCtrReqH:8;			//方向盘转速控制请求
  unsigned int :13;
}T_CONTROL_CMD_5F4_SRC;

typedef struct
{
  int iStrSpdCtrReq:19;			//方向盘转速控制请求
  unsigned int :13;
}T_CONTROL_CMD_5F4_DEST;

typedef union
{
  T_CONTROL_CMD_5F4_SRC tCtrCmd5F4Src;
  T_CONTROL_CMD_5F4_DEST tCtrCmd5F4Dest;
}T_CONTROL_CMD_5F4_UNION;

typedef struct
{
  int iStrAngCtrReq;					//方向盘转角控制请求
  unsigned int uiStrCtrMdReq;			//方向盘控制模式请求
  int iStrSpdCtrReq;					//方向盘转速控制请求
  int iStrTrqCtrReq;					//方向盘转矩控制请求
}T_STR_ACT_CTR_REQ;

#define	STR_CTR_MD_REQ_SPD	1			//方向盘控制模式-转速
#define	STR_CTR_MD_REQ_TRQ	2			//方向盘控制模式-转矩

/*****************执行机构加速度控制请求****************/
typedef struct
{
  unsigned int :32;
  unsigned int :16;
  unsigned int uiStrDecCtrReq:8;		//转向机构减速度控制请求
  unsigned int uiStrAccCtrReq:8;		//转向机构加速度控制请求
}T_CONTROL_CMD_5F5;

typedef struct
{
  unsigned int uiStrDecCtrReq;		//转向机构减速度控制请求
  unsigned int uiStrAccCtrReq;		//转向机构加速度控制请求
}T_STR_ACC_CTR_REQ;


/*************转向三角波/线性控制*************/
typedef struct
{
  unsigned int :5;
  unsigned int uiStrLT:11;//控制时间
  unsigned int :1;
  int iStrPstBsc:11;		//方向盘位置基准值
  unsigned int :1;
  int iStrSpdkL:3;		//方向盘转速L
  int iStrSpdkH:8;		//方向盘转速H
  unsigned int uiStrNum:4;//个数
  unsigned int :3;
  unsigned int uiStrTT:9;	//脉宽
  int iStrTA:8;			//峰值
}T_CONTROL_CMD_5F6;

typedef struct
{
  int iStrSpdkL:3;				//方向盘转速L
  int iStrSpdkH:8;				//方向盘转速H
  int :21;
}T_CONTROL_CMD_5F6_SRC;

typedef struct
{
  int iStrSpdk:11;				//方向盘转速
  int :21;
}T_CONTROL_CMD_5F6_DEST;

typedef union
{
  T_CONTROL_CMD_5F6_SRC tCtrCmd5F6Src;
  T_CONTROL_CMD_5F6_DEST tCtrCmd5F6Dest;
}T_CONTROL_CMD_5F6_UNION;

typedef struct
{
  int iStrTA;				//峰值
  unsigned int uiStrTT;	//脉宽
  unsigned int uiStrNum;	//个数
  int iStrSpdk;			//方向盘转速
  int iStrPstBsc;			//方向盘位置基准值
  unsigned int uiStrLT;	//控制时间
}T_STR_TRIANGLE_LINE_CTR;


/*************转向正弦控制*************/
typedef struct
{
  unsigned int uiStrT:12;		//控制时间
  unsigned int uiStrDt:10;  //迟滞时间
  unsigned int uiStrA:9;		//幅值
  unsigned int :1;
  int iStrFi:10;				//相位差
  unsigned int :1;
  unsigned int uiStrFrtb:10;	//频率基值
  unsigned int :1;
  unsigned int uiStrFrtk:10;	//频率斜率
}T_CONTROL_CMD_5F7;

typedef struct
{
  unsigned int uiStrFrtk;	//频率斜率
  unsigned int uiStrFrtb;	//频率基值
  int iStrFi;				//相位差
  unsigned int uiStrA;	//幅值
  unsigned int uiStrDt;	//迟滞时间
  unsigned int uiStrT;	//控制时间
}T_STR_SIN_CTR;


/*************方向盘位置时间序列控制*************/
typedef struct
{
  unsigned int :5;
  int iStrAngReq:19;			//方向盘转角
  unsigned int uiRelTimeL:8;	//相对时间L
  unsigned int uiRelTimeH:8;	//相对时间H
  unsigned int uiNum:8;		//实际点序号
  unsigned int uiTotalNum:8;	//总个数
  unsigned int uiSequenceID:8;//序列ID
}T_CONTROL_CMD_5F8;

typedef struct
{
  unsigned int uiRelTimeL:8;				//方向盘转速L
  unsigned int uiRelTimeH:8;				//方向盘转速H
  unsigned int :16;
}T_CONTROL_CMD_5F8_SRC;

typedef struct
{
  unsigned int uiRelTime:16;				//方向盘转速
  unsigned int :16;
}T_CONTROL_CMD_5F8_DEST;

typedef union
{
  T_CONTROL_CMD_5F8_SRC tCtrCmd5F8Src;
  T_CONTROL_CMD_5F8_DEST tCtrCmd5F8Dest;
}T_CONTROL_CMD_5F8_UNION;

typedef struct
{
  unsigned int uiSequenceID;	//序列ID
  unsigned int uiTotalNum;	//总个数
  unsigned int uiNum;			//实际点序号
  unsigned int uiRelTime;		//相对时间
  int iStrAngReq;				//方向盘转角
}T_STR_POSITION_TIME_CTR;

/*************档位执行机构控制命令*************/
typedef struct
{
  unsigned int :24;
  int iClchSpdCtrReq:8;				//离合踏板速度控制请求
  unsigned int :6;
  unsigned int uiClchPsngCtrReq:10;	//离合踏板位置控制请求
  unsigned int uiSftMdReq:2;			//换挡模式请求
  unsigned int :6;
  unsigned int uiSftPsngCtrReq:5;		//挡位控制请求
  unsigned int :3;
}T_CONTROL_CMD_5FA;

typedef struct
{
  unsigned int uiSftPsngCtrReq;		//挡位控制请求
  unsigned int uiSftMdReq;			//换挡模式请求
  unsigned int uiClchPsngCtrReq;		//离合踏板位置控制请求
  int iClchSpdCtrReq;					//离合踏板速度控制请求
}T_SFT_ACT_CTR_REQ;

#define	SFT_PSNG_CTR_REQ_NODEFINE		0	//挡位控制请求-???????????
#define	SFT_PSNG_CTR_REQ_PARK			1	//挡位控制请求-停车档（自动档）
#define	SFT_PSNG_CTR_REQ_REVERSE		2	//挡位控制请求-倒档
#define	SFT_PSNG_CTR_REQ_NEUTRAL		3	//挡位控制请求-空档
#define	SFT_PSNG_CTR_REQ_FORWARD_1		4	//挡位控制请求-前进档1(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_2		5	//挡位控制请求-前进档2(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_3		6	//挡位控制请求-前进档3(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_4		7	//挡位控制请求-前进档4(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_5		8	//挡位控制请求-前进档5(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_6		9	//挡位控制请求-前进档6(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_7		10	//挡位控制请求-前进档7(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_8		11	//挡位控制请求-前进档8(手动档)
#define	SFT_PSNG_CTR_REQ_FORWARD_9		12	//挡位控制请求-前进档9(手动档)
#define	SFT_PSNG_CTR_REQ_ADD			13	//挡位控制请求-加
#define	SFT_PSNG_CTR_REQ_SUB			14	//挡位控制请求-减
#define	SFT_PSNG_CTR_REQ_MANUAL			15	//挡位控制请求-手动选择
#define	SFT_PSNG_CTR_REQ_SPORT			16	//挡位控制请求-运动档（自动档）

#define	SFT_MD_REQ_3_SECOND		0			//换挡模式-平滑型
#define	SFT_MD_REQ_2_SECOND		1			//换挡模式-快速型
#define	SFT_MD_REQ_1_SECOND		2			//换挡模式-激进型


/********************车辆一键启动****************/
typedef struct
{
  unsigned int :32;
  unsigned int :30;
  unsigned int uiVehSwReq:2;		//一键启动请求
}T_CONTROL_CMD_5FB;

typedef struct
{
  unsigned int uiVehSwReq;		//一键启动请求
}T_VEHICLE_START;

/************* 机器人自检控制命令*************/
typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int uiRbtSlfChkReq:2;	//机器人自检请求
  unsigned int :6;
}T_CONTROL_CMD_5FC;

typedef struct
{
  unsigned int uiRbtSlfChkReq;	//机器人自检请求
}T_RBT_SLF_CHK_CTR_REQ;

#define	RBT_SLF_CHK_REQ_START	1	//机器人自检-开始
#define	RBT_SLF_CHK_REQ_END		2	//机器人自检-终止


/***************** 机器人手动控制命令****************/
typedef struct
{
  unsigned int :32;
  unsigned int :16;
  unsigned int uiMotMove:2;		//电机转向
  unsigned int :6;
  unsigned int uiAccMotCtrl:1;	//油门电机控制请求
  unsigned int uiBrkMotCtrl:1;	//制动电机控制请求
  unsigned int uiXSftMotCtrl:1;	//X轴控制请求
  unsigned int uiYSftMotCtrl:1;	//Y轴控制请求
  unsigned int uiStrMotCtrl:1;	//转向控制请求
  unsigned int uiClchMotCtrl:1;	//离合控制请求
  unsigned int :2;
}T_CONTROL_CMD_5FD;

typedef struct
{
  unsigned int uiAccMotCtrl;		//油门电机控制请求
  unsigned int uiBrkMotCtrl;		//制动电机控制请求
  unsigned int uiXSftMotCtrl;		//X轴控制请求
  unsigned int uiYSftMotCtrl;		//Y轴控制请求
  unsigned int uiStrMotCtrl;		//转向控制请求
  unsigned int uiClchMotCtrl;		//离合控制请求
  unsigned int uiMotMove;			//电机转向
}T_RBT_DRIVER_CTR;

#define	MOT_CTR_DISABLE		0	//电机控制请求-禁能
#define	MOT_CTR_ENABLE		1	//电机控制请求-使能

#define	MOT_MOVE_FORWARD	1	//电机正转
#define	MOT_MOVE_REVERSE	2	//电机反转


/*************坡度模块*************/
typedef struct
{
  unsigned int :32;
  unsigned int :21;
  int iSlopeRequest:11;		//坡度电压请求
}T_CONTROL_CMD_5FE;

typedef struct
{
  int iSlopeRequest;		//坡度电压请求
}T_SLOPE_MODE;

/***********************IO控制模块************************/
typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int IO_0Req:2;
  unsigned int IO_1Req:2;
  unsigned int :4;

}T_CONTROL_CMD_5FF;

typedef struct
{
  unsigned int IO_1Req;	//Q 0.7
  unsigned int IO_0Req;	//Q 0.6

}T_IO_REQ;












/******************************ICU-RCU状态输入******************************/
/*************车辆状态1*******技术规范添加了部分定义******/
typedef struct
{
  unsigned int uiVehHBrkStat:1;	//车辆手刹状态
  unsigned int :2;
  unsigned int uiVehSwStat:2;		//实际车辆启动状态
  unsigned int :3;
  unsigned int uiSftPsng:5;		//实际挡位
  unsigned int :1;
  unsigned int uiVehSftMd:2;		//挡位模式
  unsigned int uiDrvSftTrgtGear:4;//手动挡目标挡位
  unsigned int :2;
  unsigned int uiClchPsng:10;		//实际离合器踏板行程
  unsigned int :6;
  unsigned int uiBrkPsng:10;		//实际制动踏板行程
  unsigned int :6;
  unsigned int uiAccPsng:10;		//实际油门踏板行程
}T_STATE_IN_6F1;

typedef struct
{
  unsigned int uiAccPsng;			//实际油门踏板行程
  unsigned int uiBrkPsng;			//实际制动踏板行程
  unsigned int uiClchPsng;		//实际离合器踏板行程
  unsigned int uiDrvSftTrgtGear;	//手动挡目标挡位
  unsigned int uiVehSftMd;		//挡位模式
  unsigned int uiSftPsng;			//实际挡位
  unsigned int uiVehHBrkStat;		//车辆手刹状态
  unsigned int uiVehSwStat;		//实际车辆启动状态
}T_VEHICLE_STATUS_1;

#define	SFT_PSNG_BETWEEN		0	//实际挡位-两档间
#define	SFT_PSNG_PARK			1	//实际挡位-停车档（自动档）
#define	SFT_PSNG_REVERSE		2	//实际挡位-倒档
#define	SFT_PSNG_NEUTRAL		3	//实际挡位-空档
#define SFT_PSNG_DRIVE			4	//实际档位-D档
#define	SFT_PSNG_FORWARD_1		4	//实际挡位-前进档1(手动档)
#define	SFT_PSNG_FORWARD_2		5	//实际挡位-前进档2(手动档)
#define	SFT_PSNG_FORWARD_3		6	//实际挡位-前进档3(手动档)
#define	SFT_PSNG_FORWARD_4		7	//实际挡位-前进档4(手动档)
#define	SFT_PSNG_FORWARD_5		8	//实际挡位-前进档5(手动档)
#define	SFT_PSNG_FORWARD_6		9	//实际挡位-前进档6(手动档)
#define	SFT_PSNG_FORWARD_7		10	//实际挡位-前进档7(手动档)
#define	SFT_PSNG_FORWARD_8		11	//实际挡位-前进档8(手动档)
#define	SFT_PSNG_UNKOWN			15	//实际挡位-???????????

#define VEH_SFT_MD_NO_ACTIVATION						0	//挡位模式-未激活
#define VEH_SFT_MD_DRIVER_SHIFT_CTR_ACTIVE				1	//挡位模式-驾驶员控制激活
#define VEH_SFT_MD_ELEC_ELECTRONIC_RANGE_SELECT_ACTIVE	2	//挡位模式-电子选择激活

#define DRV_SFT_TRG_FORWARD_0	0
#define	DRV_SFT_TRG_FORWARD_1	1	//手动挡目标挡位-前进档1(手动档)
#define	DRV_SFT_TRG_FORWARD_2	2	//手动挡目标挡位-前进档2(手动档)
#define	DRV_SFT_TRG_FORWARD_3	3	//手动挡目标挡位-前进档3(手动档)
#define	DRV_SFT_TRG_FORWARD_4	4	//手动挡目标挡位-前进档4(手动档)
#define	DRV_SFT_TRG_FORWARD_5	5	//手动挡目标挡位-前进档5(手动档)
#define	DRV_SFT_TRG_FORWARD_6	6	//手动挡目标挡位-前进档6(手动档)
#define	DRV_SFT_TRG_FORWARD_7	7	//手动挡目标挡位-前进档7(手动档)
#define	DRV_SFT_TRG_FORWARD_8	8	//手动挡目标挡位-前进档8(手动档)

#define	VEH_HBRK_STAT_RELEASE	0	//车辆手刹状态-释放
#define	VEH_HBRK_STAT_HOLD		1	//车辆手刹状态-闭合

#define	VEH_SW_STAT_LOCK	0		//实际车辆启动状态-钥匙插入状态
#define	VEH_SW_STAT_ACC		1		//实际车辆启动状态-接通部分电器电源
#define	VEH_SW_STAT_ON		2		//实际车辆启动状态-接通所有电源
#define	VEH_SW_STAT_START	3		//实际车辆启动状态-点火档


/*************车辆状态2*******技术规范添加了部分定义******/
typedef struct
{
  unsigned int :3;
  unsigned int uiEngSpd:13;		//实际发动机转速
  unsigned int uiVehSpd:8;		//实际车速
  unsigned int :4;
  int iStrAngGrdL:4;				//实际方向盘角度梯度
  int iStrAngGrdH:8;				//实际方向盘角度梯度
  unsigned int :5;
  int iStrAng:19;					//实际方向盘转角
}T_STATE_IN_6F2;

typedef struct
{

  int iStrAngGrdL:4;				//实际方向盘角度梯度
  int iStrAngGrdH:8;				//实际方向盘角度梯度
  int :20;						//实际方向盘转角
}T_STATE_IN_6F2_SRC;

typedef struct
{
  int iStrAngGrd:12;				//实际方向盘角度梯度
  int :20;
}T_STATE_IN_6F2_DEST;

typedef union
{
  T_STATE_IN_6F2_SRC tStateIn6F2Src;
  T_STATE_IN_6F2_DEST tStateIn6F2Dest;
}T_STATE_IN_6F2_UNION;

typedef struct
{
  int iStrAng;					//实际方向盘转角
  int iStrAngGrd;					//实际方向盘角度梯度
  unsigned int uiVehSpd;			//实际车速
  unsigned int uiEngSpd;			//实际发动机转速
}T_VEHICLE_STATUS_2;


/*************车辆状态3*******技术规范添加了部分定义******/
typedef struct
{
  unsigned int :24;
  unsigned int uiVehBatVlt:8;		//实际车辆蓄电池电压
  unsigned int :2;
  unsigned int uiVehBatCur:22;	//实际车辆蓄电池电流
  unsigned int uiVehBatQut:8;		//实际车辆蓄电池电量
}T_STATE_IN_6F3;

typedef struct
{
  unsigned int uiVehBatQut;		//实际车辆蓄电池电量
  unsigned int uiVehBatCur;		//实际车辆蓄电池电流
  unsigned int uiVehBatVlt;		//实际车辆蓄电池电压
}T_VEHICLE_STATUS_3;


/*************工控机状态*******技术规范修改了定义******/
typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int uiCmpStat:2;		//实际工控机工作状态
  unsigned int :6;
}T_STATE_IN_6F4;

typedef struct
{
  unsigned int uiCmpStat;			//实际工控机工作状态
}T_COMPUTER_STATUS;

#define	CMP_STAT_ON			0		//上电
#define	CMP_STAT_RUN		1		//运行中
#define	CMP_STAT_ERR		2		//异常


/******************************ICU-RCU状态输出******************************/
/*************机器人执行机构实际位置*******技术规范修改了定义******/
typedef struct
{
  unsigned int :21;
  int iCcltStrActAngL:11;				//方向盘机构转角
  int iCcltStrActAngH:8;				//方向盘机构转角
  unsigned int uiCcltClchPsng:7;		//离合踏板机构位置
  unsigned int :1;
  unsigned int uiCcltBrkActPsng:8;	//制动踏板机构位置
  unsigned int uiCcltAccActPsng:7;	//油门踏板机构位置
  unsigned int :1;
}T_STATE_OUT_6E0;

typedef struct
{
  int iCcltStrActAngL:11;				//方向盘机构转角
  int iCcltStrActAngH:8;				//方向盘机构转角
  unsigned int :13;
}T_STATE_OUT_6E0_SRC;

typedef struct
{
  int iCcltStrActAng:19;				//方向盘机构转角
  unsigned int :13;
}T_STATE_OUT_6E0_DEST;

typedef union
{
  T_STATE_OUT_6E0_SRC tStateOut6E0Src;
  T_STATE_OUT_6E0_DEST tStateOut6E0Dest;
}T_STATE_OUT_6E0_UNION;

typedef struct
{
  unsigned int uiCcltAccActPsng;		//油门踏板机构位置
  unsigned int uiCcltBrkActPsng;		//制动踏板机构位置
  unsigned int uiCcltClchPsng;		//离合踏板机构位置
  int iCcltStrActAng;					//方向盘机构转角
}T_RBT_ACT_PSNG;


/*************(油门和制动)踏板学习位置反馈*******技术规范修改了定义******/
typedef struct
{
  unsigned int :32;
  unsigned int uiClbtBrkPsngMax:8;	//标定制动踏板最大行程时机器人的行程
  unsigned int uiClbtBrkPsng0:8;		//标定制动踏板0点时的机器人行程
  unsigned int uiClbtAccPsngMax:7;	//标定油门踏板最大行程时机器人的行程
  unsigned int :1;
  unsigned int uiClbtAccPsng0:7;		//标定油门踏板0点时机器人的行程
  unsigned int :1;
}T_STATE_OUT_6E1;

typedef struct
{
  unsigned int uiClbtAccPsng0;		//标定油门踏板0点时机器人的行程
  unsigned int uiClbtAccPsngMax;		//标定油门踏板最大行程时机器人的行程
  unsigned int uiClbtBrkPsng0;		//标定制动踏板0点时的机器人行程
  unsigned int uiClbtBrkPsngMax;		//标定制动踏板最大行程时机器人的行程
}T_ACC_BRK_PSNG;


/*************转向学习位置反馈*************/
typedef struct
{
  unsigned int :10;
  int iClbtStrAngRtMax:18;	//标定的方向盘右最大转角时机器人的方向盘转角
  int iClbtStrAngLftMaxL:4;	//标定的方向盘左最大转角时机器人的方向盘转角
  int iClbtStrAngLftMaxH:14;	//标定的方向盘左最大转角时机器人的方向盘转角
  int iClbtStrAng0:18;		//标定的方向盘零点角度时机器人的方向盘转角
}T_STATE_OUT_6E2;

typedef struct
{
  int iClbtStrAngLftMaxL:4;	//标定的方向盘左最大转角时机器人的方向盘转角
  int iClbtStrAngLftMaxH:14;	//标定的方向盘左最大转角时机器人的方向盘转角
  unsigned int :14;
}T_STATE_OUT_6E2_SRC;

typedef struct
{
  int iClbtStrAngLftMax:18;	//标定的方向盘左最大转角时机器人的方向盘转角
  unsigned int :14;
}T_STATE_OUT_6E2_DEST;

typedef union
{
  T_STATE_OUT_6E2_SRC tStateOut6E2Src;
  T_STATE_OUT_6E2_DEST tStateOut6E2Dest;
}T_STATE_OUT_6E2_UNION;

typedef struct
{
  int iClbtStrAng0;		//标定的方向盘零点角度时机器人的方向盘转角
  int iClbtStrAngLftMax;	//标定的方向盘左最大转角时机器人的方向盘转角
  int iClbtStrAngRtMax;	//标定的方向盘右最大转角时机器人的方向盘转角
}T_STR_LRN_PSNG;


/*************档位学习位置反馈1*******技术规范修改了定义******/
typedef struct
{
  unsigned int :1;
  int iDXMotPst:9;		//D挡X电机位置
  int iNYMotPst:9;		//N挡Y电机位置
  int iNXMotPst:9;		//N挡X电机位置
  int iRYMotPstL:4;		//R挡Y电机位置
  int iRYMotPstH:5;		//R挡Y电机位置
  int iRXMotPst:9;		//R挡X电机位置
  int iPYMotPst:9;		//P挡Y电机位置
  int iPXMotPst:9;		//P挡X电机位置
}T_STATE_OUT_6E3;

typedef struct
{
  int iRYMotPstL:4;		//R挡Y电机位置
  int iRYMotPstH:5;		//R挡Y电机位置
  unsigned int :23;
}T_STATE_OUT_6E3_SRC;

typedef struct
{
  int iRYMotPst:9;		//R挡Y电机位置
  unsigned int :23;
}T_STATE_OUT_6E3_DEST;

typedef union
{
  T_STATE_OUT_6E3_SRC tStateOut6E3Src;
  T_STATE_OUT_6E3_DEST tStateOut6E3Dest;
}T_STATE_OUT_6E3_UNION;

typedef struct
{
  int iPXMotPst;		//P挡X电机位置
  int iPYMotPst;		//P挡Y电机位置
  int iRXMotPst;		//R挡X电机位置
  int iRYMotPst;		//R挡Y电机位置
  int iNXMotPst;		//N挡X电机位置
  int iNYMotPst;		//N挡Y电机位置
  int iDXMotPst;		//D挡X电机位置
}T_SFT_LRN_PST_1;


/*************控制器上电状态*************/
typedef struct
{
  unsigned int :32;
  unsigned int :16;
  unsigned int uiStrCtrPowStat:1;		//转向电机控制器上电状态
  unsigned int :1;
  unsigned int uiXSftArmCtrPowStat:1;	//换挡臂X电机控制器上电状态
  unsigned int :1;
  unsigned int uiYSftArmCtrPowStat:1;	//换挡臂Y电机控制器上电状态
  unsigned int :1;
  unsigned int uiZSftArmCtrPowStat:1;	//换挡臂Z电机控制器上电状态
  unsigned int :1;
  unsigned int uiRCUPowStat:1;		//RCU上电状态
  unsigned int :1;
  unsigned int uiAccCtrPowStat:1;		//油门踏板电机控制器上电状态
  unsigned int :1;
  unsigned int uiBrkCtrPowStat:1;		//制动电机控制器上电状态
  unsigned int :1;
  unsigned int uiClchCtrPowStat:1;	//离合电机控制器上电状态
  unsigned int :1;
}T_STATE_OUT_6E4;

typedef struct
{
  unsigned int uiRCUPowStat;			//RCU上电状态
  unsigned int uiAccCtrPowStat;		//油门踏板电机控制器上电状态
  unsigned int uiBrkCtrPowStat;		//制动电机控制器上电状态
  unsigned int uiClchCtrPowStat;		//离合电机控制器上电状态
  unsigned int uiStrCtrPowStat;		//转向电机控制器上电状态
  unsigned int uiXSftArmCtrPowStat;	//换挡臂X电机控制器上电状态
  unsigned int uiYSftArmCtrPowStat;	//换挡臂Y电机控制器上电状态
  unsigned int uiZSftArmCtrPowStat;  //换挡臂Z电机控制器上电状态
}T_CTR_POW_STATUS;

#define	POW_STAT_OFF	0	//未上电
#define	POW_STAT_ON		1	//已上电


/*************执行机构使能状态*************/
typedef struct
{
  unsigned int :32;
  unsigned int :8;
  unsigned int uiZSftMotEnStat:1;		//换挡臂Z电机使能状态
  unsigned int :7;
  unsigned int uiYSftMotEnStat:1;		//换挡臂Y电机使能状态
  unsigned int :1;
  unsigned int uiClchMotEnStat:1;		//离合电机使能状态
  unsigned int :1;
  unsigned int uiAccClchCtrStat:1;	//油门踏板离合器通电状态
  unsigned int :1;
  unsigned int uiPauseEnStat:1;		//系统软急停使能状态
  unsigned int :1;
  unsigned int uiAccMotEnStat:1;		//油门踏板电机使能状态
  unsigned int :1;
  unsigned int uiBrkMotEnStat:1;		//制动电机使能状态
  unsigned int :1;
  unsigned int uiStrMotEnStat:1;		//转向电机使能状态
  unsigned int :1;
  unsigned int uiXSftMotEnStat:1;		//换挡臂X电机使能状态
  unsigned int :1;
}T_STATE_OUT_6E5;

typedef struct
{
  unsigned int uiAccMotEnStat;		//油门踏板电机使能状态
  unsigned int uiBrkMotEnStat;		//制动电机使能状态
  unsigned int uiStrMotEnStat;		//转向电机使能状态
  unsigned int uiXSftMotEnStat;		//换挡臂X电机使能状态
  unsigned int uiYSftMotEnStat;		//换挡臂Y电机使能状态
  unsigned int uiClchMotEnStat;		//离合电机使能状态
  unsigned int uiAccClchCtrStat;		//油门踏板离合器通电状态
  unsigned int uiPauseEnStat;			//系统软急停使能状态
  unsigned int uiZSftMotEnStat;   //换挡臂Z电机使能状态
}T_ACT_ENABLE_STATUS;

#define	MOT_EN_STAT_DISABLE		0		//电机禁能
#define	MOT_EN_STAT_ENABLE		1		//电机使能

#define	ACC_CLCH_CTR_STAT_JOINT		0	//油门离合控制器闭合(通电)
#define	ACC_CLCH_CTR_STAT_RELEASE	1	//油门离合控制器释放(断电)

#define	PAUSE_EN_STAT_DISABLE	0		//系统软急停禁能
#define	PAUSE_EN_STAT_ENABLE	1		//系统软急停使	能


/*************油门控制状态反馈*******技术规范修改了定义******/
typedef struct
{
  unsigned int :32;
  int iAccMotTrq:5;		//计算油门踏板电机转矩
  int iAccDspSpd:11;		//油门踏板位移速度
  unsigned int :3;
  int iAccMotSpd:13;		//实际油门踏板电机转速
}T_STATE_OUT_6E6;

typedef struct
{
  int iAccMotSpd;		//实际油门踏板电机转速
  int iAccDspSpd;		//油门踏板位移速度
  int iAccMotTrq;		//计算油门踏板电机转矩
}T_ACC_CTR_STATUS;


/*************制动踏板控制状态反馈*******技术规范修改了定义******/
typedef struct
{
  unsigned int uiBrkCtrMd:2;	//制动踏板控制模式请求
  unsigned int :6;
  int iBrkMotTrq:5;			//计算制动踏板电机转矩
  unsigned int :10;
  int iBrkFoc:9;				//计算制动踏板力
  unsigned int :7;
  int iBrkDspSpd:9;			//制动踏板位移速度
  unsigned int :3;
  int iBrkMotSpd:13;			//实际制动踏板电机转速
}T_STATE_OUT_6E7;

typedef struct
{
  int iBrkMotSpd;				//实际制动踏板电机转速
  int iBrkDspSpd;				//制动踏板位移速度
  int iBrkFoc;				//计算制动踏板力
  int iBrkMotTrq;				//计算制动踏板电机转矩
  unsigned int uiBrkCtrMd;	//制动踏板控制模式请求
}T_BRK_CTR_STATUS;

#define	BRK_CTR_MD_SPD	1		//制动踏板控制	模式-速度
#define	BRK_CTR_MD_FOC	2		//制动踏板控制	模式-力


/*************离合踏板控制状态反馈*******技术规范修改了定义******/
typedef struct
{
  unsigned int :16;
  int iClchMotTrq:5;			//实际离合踏板电机转矩
  unsigned int :10;
  unsigned int uiClchFocL:1;	//计算离合踏板力
  unsigned int uiClchFocH:8;	//计算离合踏板力
  int iClchDspSpd:8;			//离合踏板位移速度
  unsigned int :3;
  int iClchMotSpd:13;			//实际离合踏板电机转速
}T_STATE_OUT_6E8;

typedef struct
{
  unsigned int uiClchFocL:1;	//计算离合踏板力
  unsigned int uiClchFocH:8;	//计算离合踏板力
  unsigned int :23;
}T_STATE_OUT_6E8_SRC;

typedef struct
{
  unsigned int uiClchFoc:9;	//计算离合踏板力
  unsigned int :23;
}T_STATE_OUT_6E8_DEST;

typedef union
{
  T_STATE_OUT_6E8_SRC tStateOut6E8Src;
  T_STATE_OUT_6E8_DEST tStateOut6E8Dest;
}T_STATE_OUT_6E8_UNION;

typedef struct
{
  int iClchMotSpd;			//实际离合踏板电机转速
  int iClchDspSpd;			//离合踏板位移速度
  unsigned int uiClchFoc;		//计算离合踏板力
  int iClchMotTrq;			//实际离合踏板电机转矩
}T_CLCH_CTR_STATUS;


/*************转向控制状态反馈*******技术规范修改了定义******/
typedef struct
{
  unsigned int uiStrCtrMd:2;	//方向盘控制模式
  unsigned int :10;
  int iCcltStrSpd:12;			//方向盘转速
  unsigned int :3;
  int iStrMotSpdL:5;			//实际转向电机转速
  int iStrMotSpdH:8;			//实际转向电机转速
  int iStrMotTrq:5;			//计算转向电机转矩
  unsigned int :5;
  int iStrTrq:14;				//方向盘转矩
}T_STATE_OUT_6E9;

typedef struct
{
  int iStrMotSpdL:5;			//实际转向电机转速
  int iStrMotSpdH:8;			//实际转向电机转速
  unsigned int :19;
}T_STATE_OUT_6E9_SRC;

typedef struct
{
  int iStrMotSpd:13;			//实际转向电机转速
  unsigned int :19;
}T_STATE_OUT_6E9_DEST;

typedef union
{
  T_STATE_OUT_6E9_SRC tStateOut6E9Src;
  T_STATE_OUT_6E9_DEST tStateOut6E9Dest;
}T_STATE_OUT_6E9_UNION;

typedef struct
{
  int iStrTrq;				//方向盘转矩
  int iStrMotTrq;				//计算转向电机转矩
  int iStrMotSpd;				//实际转向电机转速
  int iCcltStrSpd;			//方向盘转速
  unsigned int uiStrCtrMd;	//方向盘控制模式
}T_STR_STATUS;

#define	STR_CTR_MD_SPD	1		//方向盘控制模式-转速
#define	STR_CTR_MD_TRQ	2		//方向盘控制模式-转矩


/*************档位控制状态反馈*******技术规范修改了定义******/
typedef struct
{
  unsigned int uiSftSpdType:2;	//换挡完成类型
  unsigned int :1;
  unsigned int uiRbtSftPsng:5;	//机器人控制的挡位位置
  unsigned int :2;
  int iYSftMotSpd:11;				//实际换挡臂Y电机转速
  unsigned int :1;
  int iXSftMotSpdL:10;			//实际换挡臂X电机转速
  int iXSftMotSpdH:1;				//实际换挡臂X电机转速
  unsigned int :2;
  int iYSftMotTrq:5;				//计算换挡臂Y电机转矩
  int iXSftMotTrq:5;				//计算换挡臂X电机转矩
  unsigned int :1;
  int iYSftMotAng:9;				//实际换挡臂Y电机转角
  int iXSftMotAng:9;				//实际换挡臂X电机转角
}T_STATE_OUT_6EA;

typedef struct
{
  int iXSftMotSpdL:10;			//实际换挡臂X电机转速
  int iXSftMotSpdH:1;				//实际换挡臂X电机转速
  unsigned int :21;
}T_STATE_OUT_6EA_SRC;

typedef struct
{
  int iXSftMotSpd:11;				//实际换挡臂X电机转速
  unsigned int :21;
}T_STATE_OUT_6EA_DEST;

typedef union
{
  T_STATE_OUT_6EA_SRC tStateOut6EASrc;
  T_STATE_OUT_6EA_DEST tStateOut6EADest;
}T_STATE_OUT_6EA_UNION;

typedef struct
{
  int iXSftMotAng;			//实际换挡臂X电机转角
  int iYSftMotAng;			//实际换挡臂Y电机转角
  int iXSftMotTrq;			//计算换挡臂X电机转矩
  int iYSftMotTrq;			//计算换挡臂Y电机转矩
  int iXSftMotSpd;			//实际换挡臂X电机转速
  int iYSftMotSpd;			//实际换挡臂Y电机转速
  unsigned int uiRbtSftPsng;	//机器人控制的挡位位置
  unsigned int uiSftSpdType;	//换挡完成类型
}T_SFT_STATUS;

#define	RBT_SFT_PSNG_BETWEEN		0	//机器人控制的挡位位置-???????????
#define	RBT_SFT_PSNG_PARK			1	//机器人控制的挡位位置-停车档（自动档）
#define	RBT_SFT_PSNG_REVERSE		2	//机器人控制的挡位位置-倒档
#define	RBT_SFT_PSNG_NEUTRAL		3	//机器人控制的挡位位置-空档
#define	RBT_SFT_PSNG_FORWARD_1		4	//机器人控制的挡位位置-前进档1(手动档)
#define	RBT_SFT_PSNG_FORWARD_2		5	//机器人控制的挡位位置-前进档2(手动档)
#define	RBT_SFT_PSNG_FORWARD_3		6	//机器人控制的挡位位置-前进档3(手动档)
#define	RBT_SFT_PSNG_FORWARD_4		7	//机器人控制的挡位位置-前进档4(手动档)
#define	RBT_SFT_PSNG_FORWARD_5		8	//机器人控制的挡位位置-前进档5(手动档)
#define	RBT_SFT_PSNG_FORWARD_6		9	//机器人控制的挡位位置-前进档6(手动档)
#define	RBT_SFT_PSNG_FORWARD_7		10	//机器人控制的挡位位置-前进档7(手动档)
#define	RBT_SFT_PSNG_FORWARD_8		11	//机器人控制的挡位位置-前进档8(手动档)
#define	RBT_SFT_PSNG_DRIVE			12	//机器人控制的挡位位置-前进档（自动档）
#define	RBT_SFT_PSNG_ADD			13	//机器人控制的挡位位置-加
#define	RBT_SFT_PSNG_SUB			14	//机器人控制的挡位位置-减
#define	RBT_SFT_PSNG_UNKOWN			15	//机器人控制的挡位位置-未定义
#define	RBT_SFT_PSNG_SPORT			16	//机器人控制的挡位位置-运动档（自动档）

#define	STF_SPD_TYPE_SLOW		0	//换挡模式-平滑型
#define	STF_SPD_TYPE_FAST		1	//换挡模式-快速型
#define	STF_SPD_TYPE_FASTER		2	//换挡模式-激进型


/*************系统运行状态参数反馈1*************/
typedef struct
{
  unsigned int uiClchMotCur:8;	//实际离合电机电流
  unsigned int uiYSftMotCur:8;	//实际换挡臂Y电机电流
  unsigned int uiXSftMotCur:8;	//实际换挡臂X电机电流
  unsigned int uiStrMotCur:8;		//实际转向电机电流
  unsigned int uiBrkMotCur:8;		//实际制动电机电流
  unsigned int uiAccMotCur:8;		//实际油门踏板电机电流
  unsigned int :5;
  unsigned int uiRbtRunTime:11;	//系统运行时间
}T_STATE_OUT_6EB;

typedef struct
{
  unsigned int uiRbtRunTime;		//系统运行时间
  unsigned int uiAccMotCur;		//实际油门踏板电机电流
  unsigned int uiBrkMotCur;		//实际制动电机电流
  unsigned int uiStrMotCur;		//实际转向电机电流
  unsigned int uiXSftMotCur;		//实际换挡臂X电机电流
  unsigned int uiYSftMotCur;		//实际换挡臂Y电机电流
  unsigned int uiClchMotCur;		//实际离合电机电流
}T_SYS_INFO_1;


/*************系统运行状态参数反馈?????*************/
typedef struct
{
  int a;//实际各电机电压
  int b;
}T_STATE_OUT_6EC;


/*************系统运行状态参数反馈2******技术规范修改了定义*******/
typedef struct
{
  unsigned int uiYSftArmCtrAlive:4;	//换挡臂Y控制器心跳
  unsigned int :3;
  unsigned int uiYSftMotClbr:1;		//换挡臂Y控制标零状态
  unsigned int uiXSftArmCtrAlive:4;	//换挡臂X控制器心跳
  unsigned int :3;
  unsigned int uiXSftMotClbr:1;		//换挡臂X控制标零状态
  unsigned int uiStrCtrAlive:4;		//转向控制器心跳
  unsigned int :3;
  unsigned int uiStrMotClbr:1;		//转向控制电机标零状态
  unsigned int uiClchCtrAlive:4;		//离合控制器心跳
  unsigned int :3;
  unsigned int uiClchMotClbr:1;		//离合控制电机标零状态
  unsigned int uiBrkCtrAlive:4;		//制动控制器心跳
  unsigned int :3;
  unsigned int uiBrkMotClbr:1;		//制动控制电机标零状态
  unsigned int uiAccCtrAlive:4;		//油门踏板控制器心跳
  unsigned int :3;
  unsigned int uiAccMotClbr:1;		//油门踏板控制电机标零状态
  unsigned int uiRCUAlive:4;			//系统RCU心跳
  unsigned int :4;
  unsigned int uiCtrMdStat:3;			//实际系统控制状态
  unsigned int :1;
  unsigned int uiRbtCfgMdStat:1;		//机器人实际配置模式
  unsigned int :3;
}T_STATE_OUT_6ED;

typedef struct
{
  unsigned int uiCtrMdStat;		//实际系统控制状态
  unsigned int uiRbtCfgMdStat;	//机器人实际配置模式
  unsigned int uiRCUAlive;		//系统RCU心跳
  unsigned int uiAccCtrAlive;		//油门踏板控制器心跳
  unsigned int uiAccMotClbr;		//油门踏板控制电机标零状态
  unsigned int uiBrkCtrAlive;		//制动控制器心跳
  unsigned int uiBrkMotClbr;		//制动控制电机标零状态
  unsigned int uiClchCtrAlive;	//离合控制器心跳
  unsigned int uiClchMotClbr;		//离合控制电机标零状态
  unsigned int uiStrCtrAlive;		//转向控制器心跳
  unsigned int uiStrMotClbr;		//转向控制电机标零状态
  unsigned int uiXSftArmCtrAlive;	//换挡臂X控制器心跳
  unsigned int uiXSftMotClbr;		//换挡臂X控制标零状态
  unsigned int uiYSftArmCtrAlive;	//换挡臂Y控制器心跳
  unsigned int uiYSftMotClbr;		//换挡臂Y控制标零状态
}T_SYS_INFO_2;

#define	CTR_MD_STAT_AUTO_LEARN		0	//实际系统控制状态-自动学习模式
#define	CTR_MD_STAT_HAND_LEARN		1	//实际系统控制状态-手动学习模式
#define	CTR_MD_STAT_AUTO_DRIVE		2	//实际系统控制状态-自动驾驶模式
#define	CTR_MD_STAT_HAND_DRIVE		3	//实际系统控制状态-人工驾驶模式
#define	CTR_MD_STAT_ROBOT_CHECK		4	//实际系统控制状态-机器人自检模式
#define	CTR_MD_STAT_ROBOT_DEBUG		5	//实际系统控制状态-机器人调试模式

#define	MOT_CLBR_NO			0		//未标零
#define	MOT_CLBR_YES		1		//已标零


/*************系统学习状态反馈******技术规范修改了定义*******/
typedef struct
{
  unsigned int uiClch0LrnStat:2;	//离合零位学习状态
  unsigned int uiClchMaxLrnStat:2;//离合最大值学习状态
  unsigned int :4;
  unsigned int uiStr0LrnStat:2;	//转向零位学习状态
  unsigned int uiStrLLrnStat:2;	//转向左极限学习状态
  unsigned int uiStrRLrnStat:2;	//转向右极限学习状态
  unsigned int :2;
  unsigned int uiAcc0LrnStat:2;	//油门零位学习状态
  unsigned int uiAccMaxLrnStat:2;	//油门最大值学习状态
  unsigned int uiBrk0LrnStat:2;	//制动零位学习状态
  unsigned int uiBrkMaxLrnStat:2;	//制动最大值学习状态
  unsigned int uiMLrnStat:2;		//M档学习状态
  unsigned int uiUpLrnStat:2;		//升学习状态
  unsigned int uiDpLrnStat:2;		//降学习状态
  unsigned int :2;
  unsigned int uiPLrnStat:2;		//P档学习状态
  unsigned int uiNLrnStat:2;		//N档学习状态
  unsigned int uiRLrnStat:2;		//R档学习状态
  unsigned int uiDLrnStat:2;		//D档学习状态
  unsigned int uiSftLrnStat:2;	//挡位学习状态
  unsigned int :6;
  unsigned int uiClchLrnStat:2;	//离合学习状态
  unsigned int uiClutchLrnWay:2;	//离合学习方式
  unsigned int uiStrLrnStat:2;	//转向学习状态
  unsigned int uiStrLrnWay:2;		//转向学习方式
  unsigned int uiAccLrnStat:2;	//油门学习状态
  unsigned int uiAccLrnWay:2;		//油门学习方式
  unsigned int uiBrkLrnStat:2;	//制动学习状态
  unsigned int uiBrkLrnWay:2;		//制动学习方式
}T_STATE_OUT_6EE;

typedef struct
{
  unsigned int uiAccLrnStat;		//油门学习状态
  unsigned int uiAccLrnWay;		//油门学习方式
  unsigned int uiBrkLrnStat;		//制动学习状态
  unsigned int uiBrkLrnWay;		//制动学习方式
  unsigned int uiClchLrnStat;		//离合学习状态
  unsigned int uiClutchLrnWay;	//离合学习方式
  unsigned int uiStrLrnStat;		//转向学习状态
  unsigned int uiStrLrnWay;		//转向学习方式
  unsigned int uiSftLrnStat;		//挡位学习状态
  unsigned int uiPLrnStat;		//P档学习状态
  unsigned int uiNLrnStat;		//N档学习状态
  unsigned int uiRLrnStat;		//R档学习状态
  unsigned int uiDLrnStat;		//D档学习状态
  unsigned int uiMLrnStat;		//M档学习状态
  unsigned int uiUpLrnStat;		//升学习状态
  unsigned int uiDpLrnStat;		//降学习状态
  unsigned int uiAcc0LrnStat;		//油门零位学习状态
  unsigned int uiAccMaxLrnStat;	//油门最大值学习状态
  unsigned int uiBrk0LrnStat;		//制动零位学习状态
  unsigned int uiBrkMaxLrnStat;	//制动最大值学习状态
  unsigned int uiClch0LrnStat;	//离合零位学习状态
  unsigned int uiClchMaxLrnStat;	//离合最大值学习状态
  unsigned int uiStr0LrnStat;		//转向零位学习状态
  unsigned int uiStrLLrnStat;		//转向左极限学习状态
  unsigned int uiStrRLrnStat;		//转向右极限学习状态
}T_SYS_LRN;

#define	LRN_STAT_NOT			0	//未学习
#define	LRN_STAT_ING			1	//学习中
#define	LRN_STAT_COMPLETE		2	//学习完成
//#define	LRN_STAT_TERMINATION	3	//学习终止

#define LRN_WAY_VIHICLE_DEPEND	0	//联机学习
#define LRN_WAY_STAND_ALONE		1	//脱机学习
#define LRN_WAY_UNKNOW			3	//未知

#define	SLFCHK_NOT				0	//自检未开始
#define	SLFCHK_ING				1	//自检进行中
#define	SLFCHK_COMPLETE			2	//自检通过
#define	SLFCHK_FAULT			3	//自检失败
#define	SLFCHK_TERMINATION		4	//自检终止

/*************档位学习位置反馈2******技术规范添加了定义*******/
typedef struct
{
  unsigned int :1;
  int iDownYMotPst:9;		//降挡Y电机位置
  int iDownXMotPst:9;		//降挡X电机位置
  int iUpYMotPst:9;		//升挡Y电机位置
  int iUpXMotPstL:4;		//升挡X电机位置
  int iUpXMotPstH:5;		//升挡X电机位置
  int iMYMotPst:9;		//M挡Y电机位置
  int iMXMotPst:9;		//M挡X电机位置
  int iDYMotPst:9;		//D挡Y电机位置
}T_STATE_OUT_6EF;

typedef struct
{
  int iUpXMotPstL:4;		//升挡X电机位置
  int iUpXMotPstH:5;		//升挡X电机位置
  unsigned int :23;
}T_STATE_OUT_6EF_SRC;

typedef struct
{
  int iUpXMotPst:9;		//升挡X电机位置
  unsigned int :23;
}T_STATE_OUT_6EF_DEST;

typedef union
{
  T_STATE_OUT_6EF_SRC tStateOut6EFSrc;
  T_STATE_OUT_6EF_DEST tStateOut6EFDest;
}T_STATE_OUT_6EF_UNION;

typedef struct
{
  int iDYMotPst;		//D挡Y电机位置
  int iMXMotPst;		//M挡X电机位置
  int iMYMotPst;		//M挡Y电机位置
  int iUpXMotPst;		//升挡X电机位置
  int iUpYMotPst;		//升挡Y电机位置
  int iDownXMotPst;	//降挡X电机位置
  int iDownYMotPst;	//降挡Y电机位置
}T_SFT_LRN_PST_2;


/*************系统监控状态反馈******技术规范修改了定义*******/
typedef struct
{
  unsigned int :24;
  unsigned int uiDrvCtrCanState:1;	//驱动器与控制器通讯状态
  unsigned int :1;
  unsigned int uiIOCtrCommunicateState:1;			//IO板与控制器通讯状态
  unsigned int :5;
  int iCtrTmp:8;						//控制器环境温度
  unsigned int uiRCUSftwAbnStat:2;	//RCU控制软件状态
  unsigned int :2;
  unsigned int uiRbtSlfChkStat:3;		//机器人自检反馈
  unsigned int :1;
  unsigned int uiClchAbnStat:1;	//离合异常状态
  unsigned int :1;
  unsigned int uiXSftAbnStat:1;	//换挡臂X异常状态
  unsigned int :1;
  unsigned int uiYSftAbnStat:1;	//换挡臂Y异常状态
  unsigned int :1;
  unsigned int uiRCUHdwAbnStat:1;		//RCU控制器硬件异常状态
  unsigned int :1;
  unsigned int uiSysAbnStat:1;		//系统异常状态
  unsigned int :1;
  unsigned int uiAccAbnStat:1;	//油门异常状态
  unsigned int :1;
  unsigned int uiBrkAbnStat:1;		//制动异常状态
  unsigned int :1;
  unsigned int uiStrAbnStat:1;		//转向异常状态
  unsigned int :1;
}T_STATE_OUT_6D0;

typedef struct
{
  unsigned int uiSysAbnStat;			//系统异常状态
  unsigned int uiAccAbnStat;		//油门异常状态
  unsigned int uiBrkAbnStat;		//制动异常状态
  unsigned int uiStrAbnStat;		//转向异常状态
  unsigned int uiClchAbnStat;		//离合异常状态
  unsigned int uiXSftAbnStat;		//换挡臂X异常状态
  unsigned int uiYSftAbnStat;		//换挡臂Y异常状态
  unsigned int uiRCUHdwAbnStat;		//RCU控制器硬件异常状态
  unsigned int uiRCUSftwAbnStat;		//RCU控制软件状态
  unsigned int uiRbtSlfChkStat;		//机器人自检反馈
  int iCtrTmp;		//控制器环境温度
  unsigned int uiDrvCtrCanState;		//驱动器与控制器通讯状态
  unsigned int uiIOCtrCommunicateState;	//IO板与控制器通讯状态
}T_SYS_MONITOR;

#define	ABN_STAT_NO_ERROR		0		//无异常
#define	ABN_STAT_ERROR			1		//异常

#define	RCU_SFTW_ABN_STAT_STOP		0	//stop
#define	RCU_SFTW_ABN_STAT_RUN		1	//run
#define	RCU_SFTW_ABN_STAT_ERROR		2	//error

#define	RBT_SLF_CHK_STAT_NO_START		0	//自检未开始
#define	RBT_SLF_CHK_STAT_ING			1	//自检进行中
#define	RBT_SLF_CHK_STAT_OK				2	//自检通过
#define	RBT_SLF_CHK_STAT_FAILED			3	//自检失败
#define	RBT_SLF_CHK_STAT_TERMINATION	4	//自检终止

#define CAN_STAT_NORMAL     0    //通讯状态-正常
#define CAN_STAT_EXCEPTIOM  1    //通讯状态-异常


/************系统报警******技术规范修改了定义*******/
typedef struct
{
  unsigned int :32;
  unsigned int uiSysErro:1;
  unsigned int uiSysErrp:1;
  unsigned int uiSysErrq:1;
  unsigned int uiSysErrr:1;
  unsigned int uiSysErrs:1;
  unsigned int uiSysErrt:1;
  unsigned int :2;
  unsigned int uiSysErrg:1;
  unsigned int uiSysErrh:1;
  unsigned int uiSysErri:1;
  unsigned int uiSysErrj:1;
  unsigned int uiSysErrk:1;
  unsigned int uiSysErrl:1;
  unsigned int uiSysErrm:1;
  unsigned int uiSysErrn:1;
  unsigned int uiSysErre:2;
  unsigned int uiSysErrf:2;
  unsigned int :4;
  unsigned int uiSysErra:1;
  unsigned int :1;
  unsigned int uiSysErrb:2;
  unsigned int uiSysErrc:2;
  unsigned int uiSysErrd:2;
}T_STATE_OUT_6D1;

typedef struct
{
  unsigned int uiSysErra;
  unsigned int uiSysErrb;
  unsigned int uiSysErrc;
  unsigned int uiSysErrd;
  unsigned int uiSysErre;
  unsigned int uiSysErrf;
  unsigned int uiSysErrg;
  unsigned int uiSysErrh;
  unsigned int uiSysErri;
  unsigned int uiSysErrj;
  unsigned int uiSysErrk;
  unsigned int uiSysErrl;
  unsigned int uiSysErrm;
  unsigned int uiSysErrn;
  unsigned int uiSysErro;
  unsigned int uiSysErrp;
  unsigned int uiSysErrq;
  unsigned int uiSysErrr;
  unsigned int uiSysErrs;
  unsigned int uiSysErrt;
}T_SYS_ERR;

/************油门报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiAccErrag : 1;
    unsigned int  uiAccErrah : 1;
    unsigned int  uiAccErrai : 1;
    unsigned int : 5;
    unsigned int  uiAccErry : 1;
    unsigned int  uiAccErrz : 1;
    unsigned int  uiAccErraa : 1;
    unsigned int  uiAccErrab : 1;
    unsigned int  uiAccErrac : 1;
    unsigned int  uiAccErrad : 1;
    unsigned int  uiAccErrae : 1;
    unsigned int  uiAccErraf : 1;
    unsigned int  uiAccErrq : 1;
    unsigned int  uiAccErrr : 1;
    unsigned int  uiAccErrs : 1;
    unsigned int  uiAccErrt : 1;
    unsigned int  uiAccErru : 1;
    unsigned int  uiAccErrv : 1;
    unsigned int  uiAccErrw : 1;
    unsigned int  uiAccErrx : 1;
    unsigned int  uiAccErri : 1;
    unsigned int  uiAccErrj : 1;
    unsigned int  uiAccErrk : 1;
    unsigned int  uiAccErrl : 1;
    unsigned int  uiAccErrm : 1;
    unsigned int  uiAccErrn : 1;
    unsigned int  uiAccErro : 1;
    unsigned int  uiAccErrp : 1;
    unsigned int  uiAccErra : 1;
    unsigned int  uiAccErrb : 1;
    unsigned int  uiAccErrc : 1;
    unsigned int  uiAccErrd : 1;
    unsigned int  uiAccErre : 1;
    unsigned int  uiAccErrf : 1;
    unsigned int  uiAccErrg : 1;
    unsigned int  uiAccErrh : 1;
}T_ACC_ERR;

/************制动报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiBrkErrag : 1;
    unsigned int  uiBrkErrah : 1;
    unsigned int  uiBrkErrai : 1;
    unsigned int : 5;
    unsigned int  uiBrkErry : 1;
    unsigned int  uiBrkErrz : 1;
    unsigned int  uiBrkErraa : 1;
    unsigned int  uiBrkErrab : 1;
    unsigned int  uiBrkErrac : 1;
    unsigned int  uiBrkErrad : 1;
    unsigned int  uiBrkErrae : 1;
    unsigned int  uiBrkErraf : 1;
    unsigned int  uiBrkErrq : 1;
    unsigned int  uiBrkErrr : 1;
    unsigned int  uiBrkErrs : 1;
    unsigned int  uiBrkErrt : 1;
    unsigned int  uiBrkErru : 1;
    unsigned int  uiBrkErrv : 1;
    unsigned int  uiBrkErrw : 1;
    unsigned int  uiBrkErrx : 1;
    unsigned int  uiBrkErri : 1;
    unsigned int  uiBrkErrj : 1;
    unsigned int  uiBrkErrk : 1;
    unsigned int  uiBrkErrl : 1;
    unsigned int  uiBrkErrm : 1;
    unsigned int  uiBrkErrn : 1;
    unsigned int  uiBrkErro : 1;
    unsigned int  uiBrkErrp : 1;
    unsigned int  uiBrkErra : 1;
    unsigned int  uiBrkErrb : 1;
    unsigned int  uiBrkErrc : 1;
    unsigned int  uiBrkErrd : 1;
    unsigned int  uiBrkErre : 1;
    unsigned int  uiBrkErrf : 1;
    unsigned int  uiBrkErrg : 1;
    unsigned int  uiBrkErrh : 1;
}T_BRK_ERR;

/************档位X电机报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiXSftErrag : 1;
    unsigned int  uiXSftErrah : 1;
    unsigned int  uiXSftErrai : 1;
    unsigned int : 5;
    unsigned int  uiXSftErry : 1;
    unsigned int  uiXSftErrz : 1;
    unsigned int  uiXSftErraa : 1;
    unsigned int  uiXSftErrab : 1;
    unsigned int  uiXSftErrac : 1;
    unsigned int  uiXSftErrad : 1;
    unsigned int  uiXSftErrae : 1;
    unsigned int  uiXSftErraf : 1;
    unsigned int  uiXSftErrq : 1;
    unsigned int  uiXSftErrr : 1;
    unsigned int  uiXSftErrs : 1;
    unsigned int  uiXSftErrt : 1;
    unsigned int  uiXSftErru : 1;
    unsigned int  uiXSftErrv : 1;
    unsigned int  uiXSftErrw : 1;
    unsigned int  uiXSftErrx : 1;
    unsigned int  uiXSftErri : 1;
    unsigned int  uiXSftErrj : 1;
    unsigned int  uiXSftErrk : 1;
    unsigned int  uiXSftErrl : 1;
    unsigned int  uiXSftErrm : 1;
    unsigned int  uiXSftErrn : 1;
    unsigned int  uiXSftErro : 1;
    unsigned int  uiXSftErrp : 1;
    unsigned int  uiXSftErra : 1;
    unsigned int  uiXSftErrb : 1;
    unsigned int  uiXSftErrc : 1;
    unsigned int  uiXSftErrd : 1;
    unsigned int  uiXSftErre : 1;
    unsigned int  uiXSftErrf : 1;
    unsigned int  uiXSftErrg : 1;
    unsigned int  uiXSftErrh : 1;
}T_XSFT_ERR;

/************档位Y电机报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiYSftErrag : 1;
    unsigned int  uiYSftErrah : 1;
    unsigned int  uiYSftErrai : 1;
    unsigned int : 5;
    unsigned int  uiYSftErry : 1;
    unsigned int  uiYSftErrz : 1;
    unsigned int  uiYSftErraa : 1;
    unsigned int  uiYSftErrab : 1;
    unsigned int  uiYSftErrac : 1;
    unsigned int  uiYSftErrad : 1;
    unsigned int  uiYSftErrae : 1;
    unsigned int  uiYSftErraf : 1;
    unsigned int  uiYSftErrq : 1;
    unsigned int  uiYSftErrr : 1;
    unsigned int  uiYSftErrs : 1;
    unsigned int  uiYSftErrt : 1;
    unsigned int  uiYSftErru : 1;
    unsigned int  uiYSftErrv : 1;
    unsigned int  uiYSftErrw : 1;
    unsigned int  uiYSftErrx : 1;
    unsigned int  uiYSftErri : 1;
    unsigned int  uiYSftErrj : 1;
    unsigned int  uiYSftErrk : 1;
    unsigned int  uiYSftErrl : 1;
    unsigned int  uiYSftErrm : 1;
    unsigned int  uiYSftErrn : 1;
    unsigned int  uiYSftErro : 1;
    unsigned int  uiYSftErrp : 1;
    unsigned int  uiYSftErra : 1;
    unsigned int  uiYSftErrb : 1;
    unsigned int  uiYSftErrc : 1;
    unsigned int  uiYSftErrd : 1;
    unsigned int  uiYSftErre : 1;
    unsigned int  uiYSftErrf : 1;
    unsigned int  uiYSftErrg : 1;
    unsigned int  uiYSftErrh : 1;
}T_YSFT_ERR;

/************转向电机报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  uiStrErrag : 1;
    unsigned int  uiStrErrah : 1;
    unsigned int  uiStrErrai : 1;
    unsigned int : 5;
    unsigned int  uiStrErry : 1;
    unsigned int  uiStrErrz : 1;
    unsigned int  uiStrErraa : 1;
    unsigned int  uiStrErrab : 1;
    unsigned int  uiStrErrac : 1;
    unsigned int  uiStrErrad : 1;
    unsigned int  uiStrErrae : 1;
    unsigned int  uiStrErraf : 1;
    unsigned int  uiStrErrq : 1;
    unsigned int  uiStrErrr : 1;
    unsigned int  uiStrErrs : 1;
    unsigned int  uiStrErrt : 1;
    unsigned int  uiStrErru : 1;
    unsigned int  uiStrErrv : 1;
    unsigned int  uiStrErrw : 1;
    unsigned int  uiStrErrx : 1;
    unsigned int  uiStrErri : 1;
    unsigned int  uiStrErrj : 1;
    unsigned int  uiStrErrk : 1;
    unsigned int  uiStrErrl : 1;
    unsigned int  uiStrErrm : 1;
    unsigned int  uiStrErrn : 1;
    unsigned int  uiStrErro : 1;
    unsigned int  uiStrErrp : 1;
    unsigned int  uiStrErra : 1;
    unsigned int  uiStrErrb : 1;
    unsigned int  uiStrErrc : 1;
    unsigned int  uiStrErrd : 1;
    unsigned int  uiStrErre : 1;
    unsigned int  uiStrErrf : 1;
    unsigned int  uiStrErrg : 1;
    unsigned int  uiStrErrh : 1;
}T_STR_ERR;

/************油门(acc)、制动(brk)、挡位(sft)X、挡位(sft)Y、转向(str)电机报警******技术规范添加了定义*******/
typedef struct
{
    unsigned int : 24;
    unsigned int  ag : 1;
    unsigned int  ah : 1;
    unsigned int  ai : 1;
    unsigned int : 5;
    unsigned int  y : 1;
    unsigned int  z : 1;
    unsigned int  aa : 1;
    unsigned int  ab : 1;
    unsigned int  ac : 1;
    unsigned int  ad : 1;
    unsigned int  ae : 1;
    unsigned int  af : 1;
    unsigned int  q : 1;
    unsigned int  r : 1;
    unsigned int  s : 1;
    unsigned int  t : 1;
    unsigned int  u : 1;
    unsigned int  v : 1;
    unsigned int  w : 1;
    unsigned int  x : 1;
    unsigned int  i : 1;
    unsigned int  j : 1;
    unsigned int  k : 1;
    unsigned int  l : 1;
    unsigned int  m : 1;
    unsigned int  n : 1;
    unsigned int  o : 1;
    unsigned int  p : 1;
    unsigned int  a : 1;
    unsigned int  b : 1;
    unsigned int  c : 1;
    unsigned int  d : 1;
    unsigned int  e : 1;
    unsigned int  f : 1;
    unsigned int  g : 1;
    unsigned int  h : 1;
}T_CTR_ERR_CODE;

//挡位(sft)X、挡位(sft)Y、转向(str)电机报警
typedef struct
{
    unsigned int  uiErrag;
    unsigned int  uiErrah;
    unsigned int  uiErrai;
    unsigned int  uiErry;
    unsigned int  uiErrz;
    unsigned int  uiErraa;
    unsigned int  uiErrab;
    unsigned int  uiErrac;
    unsigned int  uiErrad;
    unsigned int  uiErrae;
    unsigned int  uiErraf;
    unsigned int  uiErrq;
    unsigned int  uiErrr;
    unsigned int  uiErrs;
    unsigned int  uiErrt;
    unsigned int  uiErru;
    unsigned int  uiErrv;
    unsigned int  uiErrw;
    unsigned int  uiErrx;
    unsigned int  uiErri;
    unsigned int  uiErrj;
    unsigned int  uiErrk;
    unsigned int  uiErrl;
    unsigned int  uiErrm;
    unsigned int  uiErrn;
    unsigned int  uiErro;
    unsigned int  uiErrp;
    unsigned int  uiErra;
    unsigned int  uiErrb;
    unsigned int  uiErrc;
    unsigned int  uiErrd;
    unsigned int  uiErre;
    unsigned int  uiErrf;
    unsigned int  uiErrg;
    unsigned int  uiErrh;
}T_MOT_CTR_ERR;

/************行动元油门制动报警信息*******/
typedef struct
{
  unsigned int  be:1;
  unsigned int  bf:1;
  unsigned int  bg:1;
  unsigned int  bh:1;
  unsigned int  bi:1;
  unsigned int  bj:1;
  unsigned int  bk:1;
  unsigned int  bl:1;
  unsigned int  aw:1;
  unsigned int  ax:1;
  unsigned int  ay:1;
  unsigned int  az:1;
  unsigned int  ba:1;
  unsigned int  bb:1;
  unsigned int  bc:1;
  unsigned int  bd:1;
  unsigned int  ao:1;
  unsigned int  ap:1;
  unsigned int  aq:1;
  unsigned int  ar:1;
  unsigned int  as:1;
  unsigned int  at:1;
  unsigned int  au:1;
  unsigned int  av:1;
  unsigned int  ag:1;
  unsigned int  ah:1;
  unsigned int  ai:1;
  unsigned int  aj:1;
  unsigned int  ak:1;
  unsigned int  al:1;
  unsigned int  am:1;
  unsigned int  an:1;
  unsigned int  y:1;
  unsigned int  z:1;
  unsigned int  aa:1;
  unsigned int  ab:1;
  unsigned int  ac:1;
  unsigned int  ad:1;
  unsigned int  ae:1;
  unsigned int  af:1;
  unsigned int  q:1;
  unsigned int  r:1;
  unsigned int  s:1;
  unsigned int  t:1;
  unsigned int  u:1;
  unsigned int  v:1;
  unsigned int  w:1;
  unsigned int  x:1;
  unsigned int  i:1;
  unsigned int  j:1;
  unsigned int  k:1;
  unsigned int  l:1;
  unsigned int  m:1;
  unsigned int  n:1;
  unsigned int  o:1;
  unsigned int  p:1;
  unsigned int  a:1;
  unsigned int  b:1;
  unsigned int  c:1;
  unsigned int  d:1;
  unsigned int  e:1;
  unsigned int  f:1;
  unsigned int  g:1;
  unsigned int  h:1;

}T_CTR_ERR_CODE_AccBrk;

//油门(acc)、制动(brk)电机报警
typedef struct
{
    unsigned int uiErrbe;
    unsigned int uiErrbf;
    unsigned int uiErrbg;
    unsigned int uiErrbh;
    unsigned int uiErrbi;
    unsigned int uiErrbj;
    unsigned int uiErrbk;
    unsigned int uiErrbl;
    unsigned int uiErraw;
    unsigned int uiErrax;
    unsigned int uiErray;
    unsigned int uiErraz;
    unsigned int uiErrba;
    unsigned int uiErrbb;
    unsigned int uiErrbc;
    unsigned int uiErrbd;
    unsigned int uiErrao;
    unsigned int uiErrap;
    unsigned int uiErraq;
    unsigned int uiErrar;
    unsigned int uiErras;
    unsigned int uiErrat;
    unsigned int uiErrau;
    unsigned int uiErrav;
    unsigned int uiErrag;
    unsigned int uiErrah;
    unsigned int uiErrai;
    unsigned int uiErraj;
    unsigned int uiErrak;
    unsigned int uiErral;
    unsigned int uiErram;
    unsigned int uiErran;
    unsigned int uiErry;
    unsigned int uiErrz;
    unsigned int uiErraa;
    unsigned int uiErrab;
    unsigned int uiErrac;
    unsigned int uiErrad;
    unsigned int uiErrae;
    unsigned int uiErraf;
    unsigned int uiErrq;
    unsigned int uiErrr;
    unsigned int uiErrs;
    unsigned int uiErrt;
    unsigned int uiErru;
    unsigned int uiErrv;
    unsigned int uiErrw;
    unsigned int uiErrx;
    unsigned int uiErri;
    unsigned int uiErrj;
    unsigned int uiErrk;
    unsigned int uiErrl;
    unsigned int uiErrm;
    unsigned int uiErrn;
    unsigned int uiErro;
    unsigned int uiErrp;
    unsigned int uiErra;
    unsigned int uiErrb;
    unsigned int uiErrc;
    unsigned int uiErrd;
    unsigned int uiErre;
    unsigned int uiErrf;
    unsigned int uiErrg;
    unsigned int uiErrh;
}T_MOT_CTR_ERR_AccBrk;

/*************方向盘位置时间控制请求*************/
typedef struct
{
  unsigned int :32;
  unsigned int :16;
  unsigned int uiNumReq:8;		//请求实际点序号
  unsigned int uiSequenceIDReq:8;	//请求序列ID
}T_STATE_OUT_6D7;

typedef struct
{
  unsigned int uiSequenceIDReq;	//请求序列ID
  unsigned int uiNumReq;			//请求实际点序号

  //协议外的字段，用以标注是否发送改请求，避免序列执行完成后，重复申请，造成命令的重复执行
  unsigned char ucUpdataFlag;
}T_STR_PST_TIME_REQ;

/******************************ESTOP状态信息******************************/
/************Estop系统信息*************/
typedef struct
{
  unsigned int :32;
  unsigned int uiEstpAbnStat:1;	//Estop异常状态
  unsigned int :7;
  unsigned int uiPmpEnStat:1;		//急停制动气泵使能状态
  unsigned int :1;
  unsigned int uiPmpPressureStat:1;		//急停制动气泵压力状态
  unsigned int :1;
  unsigned int uiPmpValveStat:1;		//急停制动气泵手阀状态
  unsigned int :3;
  unsigned int uiEstpEnStat:2;	//机器人Estop使能状态
  unsigned int :6;
  unsigned int uiEstpAlive:4;		//Estop心跳
  unsigned int :4;
}T_ESTOP_2E0;

typedef struct
{
  unsigned int uiEstpAlive;		//Estop心跳
  unsigned int uiEstpEnStat;		//机器人Estop使能状态
  unsigned int uiPmpEnStat;		//急停制动气泵手阀状态
  unsigned int uiPmpPressureStat;  //急停制动气泵压力状态
  unsigned int uiPmpValveStat;    //急停制动气泵使能状态
  unsigned int uiEstpAbnStat;		//Estop异常状态
}T_ESTOP_INFO;

#define ESTP_EN_STAT_DISABLE	0	//机器人Estop使能状态-急停解除
#define ESTP_EN_STAT_DOING		1	//机器人Estop使能状态-急停动作执行中
#define ESTP_EN_STAT_DONE	 	2	//机器人Estop使能状态-急停动作已完成；
#define ESTP_EN_STAT_EXCEPTION	 	3	//机器人Estop使能状态-急停执行异常

#define PMP_EN_STAT_DISABLE		0	//急停制动气泵使能状态-禁能
#define PMP_EN_STAT_ENABLE		1	//急停制动气泵使能状态-使能

#define ESTP_ABN_STAT_NO_ERROR	0	//Estop异常状态-无异常
#define ESTP_ABN_STAT_ERROR		1	//Estop异常状态-异常

#define PMP_PRESSURE_STAT_NORMAL  0  //急停制动气泵压力状态-正常
#define PMP_PRESSURE_STAT_LOW     1  //急停制动气泵压力状态-气罐压力过低

#define PMP_VALVE_STAT_NORMAL     0   //急停制动气泵手阀状态-正常
#define PMP_VALVE_STAT_CLOSE      1   //急停制动气泵手阀状态-手阀处于关闭状态


/************Estop故障码******技术规范添加了定义*******/
typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int uiEspErra:1;
  unsigned int uiEspErrb:1;
  unsigned int uiEspErrc:1;
  unsigned int uiEspErrd:1;
  unsigned int uiEspErre:1;
  unsigned int uiEspErrf:1;
  unsigned int :2;
}T_ESTOP_2E1;

typedef struct
{
  unsigned int uiEspErra;	//机器人急停按钮急停
  unsigned int uiEspErrb;	//伺服系统异常
  unsigned int uiEspErrc;	//IO板通信异常
  unsigned int uiEspErrd;	//上位机急停
  unsigned int uiEspErre;	//ACC自动驾驶时触发急停
  unsigned int uiEspErrf;	//BRK自动驾驶时触发急停
}T_ESTOP_ERR;


/************************踏板期望位置计算值*************************/
typedef struct
{
  unsigned int :32;
  unsigned int :6;
  unsigned int uiBrkPsngCtrReqCalt:10;	//制动踏板期望位置计算值
  unsigned int :6;
  unsigned int uiAccPsngCtrReqCalt:10;	//油门踏板期望位置计算值
}T_STATE_OUT_6DF;

typedef struct
{
  unsigned int uiAccPsngCtrReqCalt;	//油门踏板期望位置计算值
  unsigned int uiBrkPsngCtrReqCalt;	//制动踏板期望位置计算值
}T_PSNG_CTRL_REQ_CALT;

/************************电压状态**************************/
typedef struct
{
  unsigned int :32;
  unsigned int :24;
  unsigned int uiState:1;		//电压状态
  unsigned int :7;
}T_STATE_OUT_100;

typedef struct
{
  unsigned int uiState;		//电压状态;
}T_VOLTAGE_STATE;

/*************档位 Z 轴状态反馈*******技术规范修改了定义******/
typedef struct
{
  unsigned int :16;
  unsigned int uiZSftArmCtrAlive:4;//换挡臂 Z 控制器心跳
  unsigned int :3;
  unsigned int uiZSftMotClbr:1;//换挡臂 Z 电机标零状态
  unsigned int uiZSftMotCur:8; //实际换挡臂 Z电机电流
  unsigned int :3;
  int iZSftMotSpd:13;		//实际换挡臂 Z电机转速
  int iZSftMotTrq:5;		//计算换挡臂 Z 电机转矩
  unsigned int :2;
  int iZSftMotAng:9;		//实际换挡臂 Z 电机转角
}T_STATE_OUT_6F5;

typedef struct
{
  int iZSftMotAng;		//实际换挡臂 Z 电机转角
  int iZSftMotTrq;		//计算换挡臂 Z 电机转矩
  int iZSftMotSpd;		//实际换挡臂 Z电机转速
  unsigned int uiZSftMotCur;  //实际换挡臂 Z电机电流
  unsigned int uiZSftMotClbr;  //换挡臂 Z 电机标零状态
  unsigned int uiZSftArmCtrAlive;   //换挡臂 Z 控制器心跳
}T_ZSFT_STATUS;

/************OBD实际油门踏板行程*******/
typedef struct
{
    unsigned int obdAccPsng : 16;		//实际油门踏板行程
    unsigned int : 16;
    unsigned int : 32;
}T_OBD_070;

//实际油门踏板行程
typedef struct
{
    unsigned int obdAccPsng;			//实际油门踏板行程
}T_OBD_ACC_PSNG;
//最小值 005   最大值09DA

/************MYSQL数据库 系统参数数据表*******/
typedef struct
{
  long Id;
  std::string CarName;
  double P;
  double I;
  double D;
  double KP;
  double KI;
  double KD;
  int BrkFactor;
  int BrkMax;
} RACK_SYS_PARA;

/************SocketCan通信 *******/
typedef struct
{
  unsigned int StdId;    /* CAN标准帧ID，占11bit，范围：0~0x7FF */
  unsigned int ExtId;    /* CAN扩展帧ID，占29bit，范围：0~0x1FFFFFFF */
  unsigned char IDE;     /* CAN报文ID类型，CAN_ID_STD 或 CAN_ID_EXT */
  unsigned char RTR;     /* CAN报文类型，CAN_RTR_DATA 或 CAN_RTR_REMOTE */
  unsigned char DLC;     /* CAN报文数据长度， 范围：0~8  */
  unsigned char Data[8]; /* CAN报文数据内容，每个字节范围：0~0xFF*/
} CanTxMsg;

typedef struct
{
  unsigned int StdId;    /* CAN标准帧ID，范围：0~0x7FF */
  unsigned int ExtId;    /* CAN扩展帧ID，范围：0~0x1FFFFFFF */
  unsigned char IDE;     /* CAN报文ID类型，CAN_ID_STD 或 CAN_ID_EXT */
  unsigned char RTR;     /* CAN报文类型，CAN_RTR_DATA 或 CAN_RTR_REMOTE */
  unsigned char DLC;     /* CAN报文数据长度， 范围：0~8  */
  unsigned char Data[8]; /* CAN报文数据内容，每个字节范围：0~0xFF*/
  unsigned char FMI;     /* 过滤模式，总共有14中，定义有宏，其值依次为0x1,0x2,0x4,0x8,0x10,0x20,0x40……,此处可以不考虑，忽略*/
} CanRxMsg;




typedef unsigned int __u32;
typedef unsigned char __u8;
typedef unsigned short __u16;
#define DWORD  __u32
#define WORD   __u16
#define BYTE   __u8
/*
 * MSGTYPE bits of element MSGTYPE in structure TPCANMsg
 */
#define MSGTYPE_STANDARD      0x00     // marks a standard frame
#define MSGTYPE_RTR           0x01     // marks a remote frame
#define MSGTYPE_EXTENDED      0x02     // declares a extended frame
#define MSGTYPE_SELFRECEIVE   0x04     // self-received message
#define MSGTYPE_STATUS        0x80     // used to mark a status TPCANMsg
typedef struct pcan_msg {
  DWORD ID;              // 11/29 bit code
  BYTE  MSGTYPE;         // bits of MSGTYPE_*
  BYTE  LEN;             // count of data bytes (0..8)
  BYTE  DATA[8];         // data bytes, up to 8
} TPCANMsg;
typedef struct pcan_rd_msg {
  TPCANMsg Msg;          // the above message
  DWORD    dwTime;       // a timestamp in msec, read only
  WORD     wUsec;        // remainder in micro-seconds
} TPCANRdMsg; // for PCAN_WRITE_MSG

} // namespace canbus

} // namespace robot


#endif /* CAN_DATA_TYPE_H_*/
