/*
 * PID.cpp
 *
 *  Created on: Dec 7, 2023
 *      Author: siasun
 */
#include "pid.h"
namespace  robot{
namespace canbusSpace {
PID::PID()  //构造函数
{


    num_area = 8;
    qdetail_kp = 0;               //增量kp对应论域中的值
    qdetail_ki = 0;               //增量ki对应论域中的值
    qdetail_kd = 0;               //增量kd对应论域中的值

    /****************隶属度*******************/
    //输入e的隶属值
    e_membership_values[0] = -6;
    e_membership_values[1] = -4;
    e_membership_values[2] = -2;
    e_membership_values[3] = 0;
    e_membership_values[4] = 2;
    e_membership_values[5] = 4;
    e_membership_values[6] = 6;

    //输入de/dt的隶属值
    ec_membership_values[0] = -6;
    ec_membership_values[1] = -4;
    ec_membership_values[2] = -2;
    ec_membership_values[3] = 0;
    ec_membership_values[4] = 2;
    ec_membership_values[5] = 4;
    ec_membership_values[6] = 6;

    //输出增量kp的隶属值
    kp_menbership_values[0] = -1;//输出增量kp的隶属值
    kp_menbership_values[1] = -0.6667;
    kp_menbership_values[2] = -0.3333;
    kp_menbership_values[3] = 0;
    kp_menbership_values[4] = 0.3333;
    kp_menbership_values[5] = 0.6667;
    kp_menbership_values[6] = 1;

    //输出增量ki的隶属值
    ki_menbership_values[0] = -1; //输出增量ki的隶属值
    ki_menbership_values[1] = -0.6667;
    ki_menbership_values[2] = -0.3333;
    ki_menbership_values[3] = 0;
    ki_menbership_values[4] = 0.3333;
    ki_menbership_values[5] = 0.6667;
    ki_menbership_values[6] = 1;

    //输出增量kd的隶属值
    kd_menbership_values[0] = -1;  //输出增量kd的隶属值
    kd_menbership_values[1] = -0.6667;
    kd_menbership_values[2] = -0.3333;
    kd_menbership_values[3] = 0;
    kd_menbership_values[4] = 0.3333;
    kd_menbership_values[5] = 0.6667;
    kd_menbership_values[6] = 1;
}

PID::~PID()//析构函数
{
}


/**********************************位置PID****************************************/

float PID::Pid(int TargetSpeed  ,int ActualSpeed)
{
  float error = 0; //误差
  static float error_1 = 0; //上一次的误差
  static float error_i = 0;
  float out = 0;  // 输出
  float out_i =0;  // 输出

  float kp= 0.1;
  float ki= 0.001;
  float kd= 0.1;


  error = TargetSpeed  - ActualSpeed;
  error_i += error;
  out_i =  ki * error_i;


  if(out_i > 1000.0)
    out_i = 1000.0;
  if(out_i < -1000.0)
    out_i = -1000.0;

  out = kp * error  + out_i + kd*(error -error_1);
  error_1 = error;


    return out;

}


/**********************************增量PID******************************************/


float PID::ADD_PID(int Target ,int Actual)
{
  float error; //误差
  static float error_1; //上一次的误差
  static float error_2; //上上次的误差
  float out;  // 输出
  static float out_last; //上次输出

  float kp= 0.1;
  float ki= 0.1;
  float kd= 0.1;


  error = Target - Actual;
  float  increamentspeed = kp*(error -error_1) + ki*error_1 + kd*(error + error_2 -2*error_1);
  out = out_last  + increamentspeed;
  out_last  =  out;
  error_1 = error;
  error_2 = error_1;

    return out;

}




/****************************** 模糊PID**************************************/

//输入e与de/dt隶属度计算函数
void PID::Get_grad_membership(float erro,float erro_c)
{
    if (erro > e_membership_values[0] && erro < e_membership_values[6])
    {
        for (int i = 0; i < num_area - 2; i++)
        {
            if (erro >= e_membership_values[i] && erro <= e_membership_values[i + 1])
            {
                e_gradmembership[0] = -(erro - e_membership_values[i + 1]) / (e_membership_values[i + 1] - e_membership_values[i]);
                e_gradmembership[1] = 1+(erro - e_membership_values[i + 1]) / (e_membership_values[i + 1] - e_membership_values[i]);
                e_grad_index[0] = i;
                e_grad_index[1] = i + 1;
                break;
            }
        }
    }
    else
    {
        if (erro <= e_membership_values[0])
        {
            e_gradmembership[0] = 1;
            e_gradmembership[1] = 0;
            e_grad_index[0] = 0;
            e_grad_index[1] = -1;
        }
        else if (erro >= e_membership_values[6])
        {
            e_gradmembership[0] = 1;
            e_gradmembership[1] = 0;
            e_grad_index[0] = 6;
            e_grad_index[1] = -1;
        }
    }

    if (erro_c > ec_membership_values[0] && erro_c < ec_membership_values[6])
    {
        for (int i = 0; i < num_area - 2; i++)
        {
            if (erro_c >= ec_membership_values[i] && erro_c <= ec_membership_values[i + 1])
            {
                ec_gradmembership[0] = -(erro_c - ec_membership_values[i + 1]) / (ec_membership_values[i + 1] - ec_membership_values[i]);
                ec_gradmembership[1] = 1 + (erro_c - ec_membership_values[i + 1]) / (ec_membership_values[i + 1] - ec_membership_values[i]);
                ec_grad_index[0] = i;
                ec_grad_index[1] = i + 1;
                break;
            }
        }
    }
    else
    {
        if (erro_c <= ec_membership_values[0])
        {
            ec_gradmembership[0] = 1;
            ec_gradmembership[1] = 0;
            ec_grad_index[0] = 0;
            ec_grad_index[1] = -1;
        }
        else if (erro_c >= ec_membership_values[6])
        {
            ec_gradmembership[0] = 1;
            ec_gradmembership[1] = 0;
            ec_grad_index[0] = 6;
            ec_grad_index[1] = -1;
        }
    }

}

//获取输出增量kp,ki,kd的总隶属度
void PID::GetSumGrad()
{

  int NB = -3, NM = -2, NS = -1, ZO = 0, PS = 1, PM = 2, PB = 3; //论域隶属值

    int  Kp_rule_list[7][7] = { {PB,PB,PM,PM,PS,ZO,ZO},     //kp规则表
                                {PB,PB,PM,PS,PS,ZO,NS},
                                {PM,PM,PM,PS,ZO,NS,NS},
                                {PM,PM,PS,ZO,NS,NM,NM},
                                {PS,PS,ZO,NS,NS,NM,NM},
                                {PS,ZO,NS,NM,NM,NM,NB},
                                {ZO,ZO,NM,NM,NM,NB,NB} };

    int  Ki_rule_list[7][7] = { {NB,NB,NM,NM,NS,ZO,ZO},     //ki规则表
                                {NB,NB,NM,NS,NS,ZO,ZO},
                                {NB,NM,NS,NS,ZO,PS,PS},
                                {NM,NM,NS,ZO,PS,PM,PM},
                                {NM,NS,ZO,PS,PS,PM,PB},
                                {ZO,ZO,PS,PS,PM,PB,PB},
                                {ZO,ZO,PS,PM,PM,PB,PB} };

    int  Kd_rule_list[7][7] = { {PS,NS,NB,NB,NB,NM,PS},    //kd规则表
                                {PS,NS,NB,NM,NM,NS,ZO},
                                {ZO,NS,NM,NM,NS,NS,ZO},
                                {ZO,NS,NS,NS,NS,NS,ZO},
                                {ZO,ZO,ZO,ZO,ZO,ZO,ZO},
                                {PB,NS,PS,PS,PS,PS,PB},
                                {PB,PM,PM,PM,PS,PS,PB} };



    for (int i = 0; i < num_area - 1; i++)
    {
        KpgradSums[i] = 0;
        KigradSums[i] = 0;
        KdgradSums[i] = 0;

    }
   for (int i=0;i<2;i++)
   {
      if (e_grad_index[i] == -1)
      {
       continue;
      }
      for (int j = 0; j < 2; j++)
      {
          if (ec_grad_index[j] != -1 && e_grad_index[i] != -1)
          {
              int indexKp = Kp_rule_list[e_grad_index[i]][ec_grad_index[j]] + 3;//求出下标
              int indexKi = Ki_rule_list[e_grad_index[i]][ec_grad_index[j]] + 3;
              int indexKd = Kd_rule_list[e_grad_index[i]][ec_grad_index[j]] + 3;

              KpgradSums[indexKp]= KpgradSums[indexKp] + (e_gradmembership[i] * ec_gradmembership[j]);
              KigradSums[indexKi] = KigradSums[indexKi] + (e_gradmembership[i] * ec_gradmembership[j]);
              KdgradSums[indexKd] = KdgradSums[indexKd] + (e_gradmembership[i] * ec_gradmembership[j]);


          }
          else
          {
            continue;
          }

      }
  }

}

//计算输出增量kp,kd,ki对应论域值
void PID::GetOUT()
{

    for (int i = 0; i < num_area - 1; i++)
    {
        qdetail_kp += kp_menbership_values[i] * KpgradSums[i];
        qdetail_ki += ki_menbership_values[i] * KigradSums[i];
        qdetail_kd += kd_menbership_values[i] * KdgradSums[i];
    }
}

//模糊PID控制实现函数
float PID::FuzzyPIDcontroller(float TargetSpeed,float ActualSpeed,float Calculation_Time,
                              float KP_BaseValue,float KI_BaseValue,float KD_BaseValue,
                              float KP_Factor,float KI_Factor,float KD_Factor)
{
    float kp = KP_BaseValue;
    float ki = KI_BaseValue;
    float kd = KD_BaseValue;


    float detail_kp = 0;               //输出增量kp
    float detail_ki = 0;               //输出增量ki
    float detail_kd = 0;               //输出增量kd
    float qerro = 0;                    //输入e对应论域中的值
    float qerro_c = 0;                  //输入de/dt对应论域中的值

  float erro = 0;//误差
  float erro_c = 0;//误差变化
  static float erro_pre = 0;//上次误差
  static float errp_ppre = 0;//上上次误差
  float output = 0;//输出
  static float output_last = 0;//上次输出
  static float error_i;//累计误差
  float out_i = 0;//i输出


  erro = TargetSpeed - ActualSpeed;//本次误差e（目标位置-实际位置）
  if(Calculation_Time <= 0)
  {
    erro_c = 0;
  }
  else
  {
      erro_c = (erro - erro_pre)/Calculation_Time;//误差变化量 需要除以时间 de/dt
  }

  erro_pre = erro;

    qerro = Quantization( erro, ERROR );//将误差映射在-6～6之间
    qerro_c = Quantization( erro_c, ERROR_C );//将误差的差值映射在-6～6之间
    Get_grad_membership(qerro, qerro_c); //找出误差 e和de/dt的隶属度
    GetSumGrad();//获取输出增量kp,ki,kd
    GetOUT();//计算输出增量kp,kd,ki对应论域值

    //增量kp,kd,ki乘以系数
    detail_kp = Inverse_quantization(KP_Factor ,qdetail_kp);
    detail_ki = Inverse_quantization(KI_Factor ,qdetail_ki);
    detail_kd = Inverse_quantization(KD_Factor ,qdetail_kd);


//    printf(" kp= %f   ki= %f  kd= %f  detail_kp= %f  detail_ki= %f d etail_kd= %f\n",
//    		kp  ,ki, kd ,detail_kp, detail_ki ,detail_kd);



#  if 0
    //增量pid

    kp = kp + detail_kp;
    ki = ki + detail_ki;
    kd = kd + detail_kd;

    float  increamentspeed  = kp*(erro - erro_pre) + ki * erro + kd * (erro - 2 * erro_pre + errp_ppre);

    output =  output_last + increamentspeed ;
    output_last = output;

    erro_pre = erro;
    errp_ppre =  erro_pre;

#endif


#if 1
    //位置pid

  error_i += erro;//累计误差的和



  if( TargetSpeed == 0 && ActualSpeed < 0.05) //目标速度为0时清空积分的值 或 if( ActualSpeed == 0 )
  {
    error_i = 0;
  }
    kp = kp + detail_kp;
    ki = ki + detail_ki;
    kd = kd + detail_kd;
  out_i = ki * error_i;//积分项输出

  output = kp * erro  + out_i + kd*(erro - erro_pre);//输出

//  LOG(INFO)<< "PID P :"<<kp;
//  LOG(INFO)<< "PID P erro:"<<erro;
//  LOG(INFO)<< "PID i :"<<ki;
//  LOG(INFO)<< "PID i erro:"<<error_i;

  //积分限值
  if(ki != 0 && (error_i > (MaxValue  / ki)))
  {
    error_i = MaxValue /ki;
  }
  if(ki != 0 &&( error_i < ( MinValue / ki)))
  {
    error_i =  MinValue /ki;
  }


    //输出限值
  if(output > MaxValue )
  {
    output = MaxValue ;
  }
  if(output <  MinValue)
  {
    output =  MinValue;
  }

    qdetail_kd = 0;
    qdetail_ki = 0;
    qdetail_kp = 0;

#endif



    return output;
}

///区间映射函数 将e ec映射在[-6～6]之间
float PID::Quantization(float x,float y)
{
    float qvalues =  6 * x / y;
    return qvalues;
}


//反区间映射函数  增量kp,kd,ki乘以系数
float PID::Inverse_quantization( float k,float qvalues)
{
    float x = k * qvalues ;
    return x;
}

int PID::test()
{
  return 2023;
}
}// namespace canbusSpace
}// namespace robot

